Package: DepecheR
Version: 1.28.0
Date: 2024-08-22
Title: Determination of essential phenotypic elements of clusters in
        high-dimensional entities
biocViews:
        Software,CellBasedAssays,Transcription,DifferentialExpression,
        DataRepresentation,ImmunoOncology,Transcriptomics,Classification,Clustering,
        DimensionReduction,FeatureExtraction,FlowCytometry,RNASeq,SingleCell,
        Visualization
Authors@R: c(
    person('Jakob', 'Theorell', email = "jakob.theorell@ki.se",
        role = c("aut", "cre"), comment=c(ORCID="0000-0001-8752-3151")), 
    person('Axel', 'Theorell', email = "axel.theorell@gmail.com", 
        role = c("aut")))
Description: The purpose of this package is to identify traits in a dataset that
    can separate groups. This is done on two levels. First, clustering is 
    performed, using an implementation of sparse K-means. Secondly, the 
    generated clusters are used to predict outcomes of groups of individuals 
    based on their distribution of observations in the different clusters. As 
    certain clusters with separating information will be identified, and these 
    clusters are defined by a sparse number of variables, this method can reduce
    the complexity of data, to only emphasize the data that actually matters.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: false
RoxygenNote: 7.3.1
Depends: R (>= 4.0)
Imports: ggplot2 (>= 3.1.0), MASS (>= 7.3.51), Rcpp (>= 1.0.0), dplyr
        (>= 0.7.8), gplots (>= 3.0.1), viridis (>= 0.5.1), foreach (>=
        1.4.4), doSNOW (>= 1.0.16), matrixStats (>= 0.54.0), mixOmics
        (>= 6.6.1), moments (>= 0.14), grDevices (>= 3.5.2), graphics
        (>= 3.5.2), stats (>= 3.5.2), utils (>= 3.5), methods (>= 3.5),
        parallel (>= 3.5.2), reshape2 (>= 1.4.3), beanplot (>= 1.2),
        FNN (>= 1.1.3), robustbase (>= 0.93.5), gmodels (>= 2.18.1),
        collapse (>= 1.9.2), ClusterR (>= 1.3.2)
LinkingTo: Rcpp, RcppEigen
Suggests: uwot, testthat, knitr, rmarkdown, BiocStyle
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/DepecheR
git_branch: RELEASE_3_23
git_last_commit: 19670a0
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 21:41:55 UTC; biocbuild
Author: Jakob Theorell [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8752-3151>),
  Axel Theorell [aut]
Maintainer: Jakob Theorell <jakob.theorell@ki.se>

# Generated by roxygen2: do not edit by hand

export(dColorPlot)
export(dColorVector)
export(dContours)
export(dDensityPlot)
export(dResidualPlot)
export(dScale)
export(dSplsda)
export(dViolins)
export(dWilcox)
export(depeche)
export(groupProbPlot)
export(nUniqueNeighDons)
export(neighSmooth)
importFrom(ClusterR,KMeans_rcpp)
importFrom(FNN,knnx.index)
importFrom(MASS,kde2d)
importFrom(Rcpp,evalCpp)
importFrom(beanplot,beanplot)
importFrom(collapse,fmode)
importFrom(doSNOW,registerDoSNOW)
importFrom(dplyr,sample_n)
importFrom(foreach,"%do%")
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(gmodels,fast.prcomp)
importFrom(gplots,heatmap.2)
importFrom(gplots,rich.colors)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,densCols)
importFrom(grDevices,dev.off)
importFrom(grDevices,palette)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(grDevices,rainbow)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,contour)
importFrom(graphics,hist)
importFrom(graphics,image)
importFrom(graphics,legend)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.new)
importFrom(graphics,text)
importFrom(matrixStats,rowMedians)
importFrom(methods,is)
importFrom(mixOmics,splsda)
importFrom(mixOmics,tune.splsda)
importFrom(moments,kurtosis)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(reshape2,melt)
importFrom(robustbase,colMedians)
importFrom(stats,as.dendrogram)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,median)
importFrom(stats,order.dendrogram)
importFrom(stats,p.adjust)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(utils,globalVariables)
importFrom(utils,tail)
importFrom(utils,write.csv)
importFrom(viridis,inferno)
importFrom(viridis,magma)
importFrom(viridis,plasma)
importFrom(viridis,viridis)
useDynLib(DepecheR)

# DepecheR is now on Bioconductor! Please go to https://bioconductor.org/packages/release/bioc/html/DepecheR.html to get access to vinjettes, and to install the latest release or development versions. 

DepecheR is an R package for clustering cytometry and single-cell RNA sequencing data according to a minimal set of biological markers. 
This is done using repeated application of a penalized k-means algorithm, which penalizes partitions that depend on a large number of markers. DepecheR tunes the level of penalization by testing a range of penalization levels, thus minimizing the need for assumptions and arbitrary decisions. 
DepecheR however also contains a function suite for statistical interpretation of group differences, and for visualization of said group differences, in addition to population densities, marker expressions, et cetera. 

## ----style, echo=FALSE, results='hide', message=FALSE-------------------------
library(BiocStyle)
library(knitr)
opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
knitr::opts_chunk$set(echo = TRUE)

## ----eval = FALSE-------------------------------------------------------------
# if(!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("DepecheR")

## -----------------------------------------------------------------------------
library(DepecheR)
data("testData")
data("testDataSNE")

## ----eval = FALSE-------------------------------------------------------------
# dataTrans <-
#   testData[, c("SYK", "CD16", "CD57", "EAT.2", "CD8", "NKG2C", "CD2", "CD56")]
# 
# testData$groupProb <- groupProbPlot(xYData = testDataSNE$Y,
#                                     groupVector = testData$label,
#                                     groupName1 = "Group_1",
#                                     groupName2 = "Group_2",
#                                     dataTrans = dataTrans)
# ## [1] "Done with k-means"
# ## [1] "Now the first bit is done, and the iterative part takes off"
# ## [1] "Clusters 1 to 7 smoothed in 2.9159369468689 . Now, 13 clusters are
# ## [1] left."
# ## [1] "Clusters 8 to 14 smoothed in 0.925199031829834 . Now, 6 clusters are
# ## [1] left."
# ## [1] "Clusters 15 to 20 smoothed in 0.905373096466064 . Now, 0 clusters are
# ## [1] left."

## -----------------------------------------------------------------------------
sessionInfo()

## ----style, echo=FALSE, results='hide', message=FALSE-------------------------
library(BiocStyle)
library(knitr)
opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
knitr::opts_chunk$set(echo = TRUE)

## ----eval = FALSE-------------------------------------------------------------
# if(!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("DepecheR")

## -----------------------------------------------------------------------------
library(DepecheR)
data('testData')
str(testData)

## ----eval=FALSE---------------------------------------------------------------
# testDataDepeche <- depeche(testData[, 2:15])

## -----------------------------------------------------------------------------
## [1] "Files will be saved to ~/Desktop"
## [1] "As the dataset has less than 100 columns, peak centering is applied."
## [1] "Set 1 with 7 iterations completed in 14 seconds."
## [1] "Set 2 with 7 iterations completed in 6 seconds."
## [1] "Set 3 with 7 iterations completed in 6 seconds."
## [1] "The optimization was iterated 21 times."

## ----echo=FALSE---------------------------------------------------------------
data("testDataDepeche")

## -----------------------------------------------------------------------------
str(testDataDepeche)

## ----eval=FALSE---------------------------------------------------------------
# library(Rtsne)
# testDataSNE <- Rtsne(testData[,2:15], pca=FALSE)

## ----echo=FALSE---------------------------------------------------------------
    data("testDataSNE")

## -----------------------------------------------------------------------------
dColorPlot(colorData = testDataDepeche$clusterVector, xYData = testDataSNE$Y, 
           colorScale = "dark_rainbow", plotName = "Cluster")

## -----------------------------------------------------------------------------
dColorPlot(colorData = testData[2], xYData = testDataSNE$Y)


## -----------------------------------------------------------------------------
densContour <- dContours(testDataSNE$Y)

dDensityPlot(xYData = testDataSNE$Y, plotName = 'All_events', 
             colorScale="purple3", densContour = densContour)

#Here the data for the first group is plotted
dDensityPlot(xYData = testDataSNE$Y[testData$label==0,], plotName = 'Group_0', 
             colorScale="blue", densContour = densContour)

#And here comes the second group
dDensityPlot(xYData = testDataSNE$Y[testData$label==1,], plotName = 'Group_1', 
             colorScale="red", densContour = densContour)


## -----------------------------------------------------------------------------
dResidualPlot(
    xYData = testDataSNE$Y, groupVector = testData$label,
    clusterVector = testDataDepeche$clusterVector)

## -----------------------------------------------------------------------------
dWilcoxResult <- dWilcox(
    xYData = testDataSNE$Y, idsVector = testData$ids,
    groupVector = testData$label, clusterVector = testDataDepeche$clusterVector)

## ----eval=FALSE---------------------------------------------------------------
# sPLSDAObject <- dSplsda(xYData = testDataSNE$Y, idsVector = testData$ids,
#                         groupVector = testData$label,
#                         clusterVector = testDataDepeche$clusterVector)
# ## Saving 3 x 3 in image
# 
# ## [1] "The separation of the datasets was perfect, with no overlap between
# ## the groups"
# 
# ## [1] "Files were saved at /Users/jakthe/Labbet/GitHub/DepecheR/vignettes"
# 

## ----eval---------------------------------------------------------------------
dViolins(testDataDepeche$clusterVector, inDataFrame = testData, 
         plotClusters = 3, plotElements = testDataDepeche$essenceElementList)

## -----------------------------------------------------------------------------
sessionInfo()

# This function is used by dDensityPlot.
# It calculates the density at each point of a 2-dimensional field of
# data, creates a color density gradient based on this, and resorts the events
# so that the lowest density areas will be plotted first, and so on.
# The multiple color option allows for multiple separate densities to be 
# plotted simulataneously, for example if multiple clusters should be displayed
# together with density.
# For information on the parameters, see dDensityPlot

dDensityPlotCoFunction <- function(xYData, idsVector, uniqueIds, color,
                                   colorList, plotName, densContour, bandColor,
                                   dotSize, title, plotDir,
                                   createOutput = FALSE) {
    if (missing(idsVector)) {
        df <- xYData
        colnames(df) <- c("V1", "V2")
        ## Use densCols() output to get density at
        ## each point. The colors here are only
        ## supporting the coming order of the rows
        ## further down the script.
        df$col <- densCols(df$V1, df$V2,
            colramp =
                colorRampPalette(c("black", "grey", color))
        )
        df$dens <- col2rgb(df$col)[1, ] + 1L
    } else {
        # Divide the dataframe according to which
        # color annotation the event has
        dfList <- list()
        for (i in seq_along(uniqueIds)) {
            df <- xYData[idsVector == uniqueIds[i], ]
            colnames(df) <- c("V1", "V2")
            ## Use densCols() output to get density at
            ## each point. The colors here are only
            ## supporting the coming order of the rows
            ## further down the script.
            df$col <- densCols(df$V1, df$V2,
                colramp =
                    colorRampPalette(c(
                        "black",
                        "grey", color[i]
                    ))
            )
            df$dens <- col2rgb(df$col)[1, ] + 1L
            dfList[[i]] <- df
        }

        df <- do.call("rbind", dfList)
    }

    # Plot it, reordering rows so that
    # densest points are plotted on top
    newXyData <- df[order(df$dens), ]
    dPlotCoFunction(
        colorVariable = newXyData$col, plotName = plotName,
        xYData = newXyData[, seq_len(2)], title = title,
        densContour = densContour, bandColor = bandColor,
        dotSize = dotSize, plotDir = plotDir,
        createOutput = createOutput
    )
}

#' Display third variable as color on a 2D plot
#'
#'
#' Function to overlay one variable for a set of observations on a field
#' created by two other variables known for the same observations. The plot is
#' constructed primarily for displaying variables on 2D-stochastic neighbour
#' embedding fields, but can be used for any sets of (two or) three variables
#' known for the same observations. As the number of datapoints is often very
#' high, the files would, if saved as pdf of another vector based file type
#' become extremely big. For this reason, the plots are saved as jpeg and no
#' axes or anything alike are added, to simplify usage in publications.
#' @importFrom gplots rich.colors
#' @importFrom parallel detectCores makeCluster stopCluster
#' @importFrom doSNOW registerDoSNOW
#' @importFrom foreach foreach %dopar%
#' @param colorData A numeric matrix or dataframe or a vector, be it numeric,
#' charater or factor, that should be used to define the colors on the plot.
#' A pre-made vector of colors is also accepted.
#' @param controlData Optional. A numeric/integer vector or dataframe of values
#' that could be used to define the range of the colorData. If no control data
#' is present, the function defaults to using the colorData as control data.
#' @param xYData These variables create the field on which the colorData will
#' be displayed. It needs to be a matrix or dataframe with two columns and the
#' same number of rows as the colorData object.
#' @param plotName The name(s) for the plot(s). 'default' returns the column
#' names of the colorData object in the case this is a dataframe and otherwise
#' returns the somewhat generic name 'testVariable'. It can be substituted with
#' a string (in the case colorData is a vector) or vector of strings, as long as
#' it has the same length as the number of columns in colorData.
#' @param colorScale This argument controls the colors in the plot. See
#' \code{\link{dColorVector}} for alternatives.
#' @param densContour If density contours should be created for the plot(s) or
#' not. Defaults to TRUE. If a density object, as generated by dContours, is
#' included, this will be used instead.
#' @param title If there should be a title displayed on the plotting field. As
#' the plotting field is saved a jpeg, this title cannot be removed as an object
#' afterwards, as it is saved as coloured pixels. To simplify usage for
#' publication, the default is FALSE, as the files are still named, eventhough
#' no title appears on the plot.
#' @param plotDir If different from the current directory. If specified and
#' non-existent, the function creates it. If "." is specified, the plots will be
#' saved at the current directory. By default, a new directory is added if the
#' created plots will be more than 1.
#' @param truncate If truncation of the most extreme values should be performed
#' for the visualizations. Three possible values: TRUE, FALSE, and a vector
#' with two values indicating the low and high threshold quantiles for
#' truncation.
#' @param bandColor The color of the contour bands. Defaults to black.
#' @param dotSize Simply the size of the dots. The default makes the dots
#' maller the more observations that are included.
#' @param continuous Boolean. Is the colorData parameter continuous? If
#' default, then only numeric vectors with more than 20 values are considered
#' continuous.This only applies to situations with single vectors. In situations
#' where a dataframe is added as colorData, all variables are considered
#' continuous.
#' @param multiCore If the algorithm should be performed on multiple cores.
#' This increases the speed if the dataset is medium-large (>100000 rows) and
#' has at least 5 columns. Default is TRUE when these above criteria are met and
#' FALSE otherwise.
#' @param nCores If multiCore is TRUE, then this sets the number of parallel
#' processes. The default is currently 87.5 percent with a cap on 10 cores, as
#' no speed increase is generally seen above 10 cores for normal computers.
#' @param createOutput For testing purposes. Defaults to TRUE. If FALSE, no
#' plots are generated.
#' @seealso \code{\link{dDensityPlot}}, \code{\link{dResidualPlot}},
#' \code{\link{dWilcox}}, \code{\link{dColorVector}}
#' @return Plots showing the colorData displayed as color on the field created
#' by xYData.
#' @examples
#'
#' # Load some data
#' data(testData)
#' \dontrun{
#' # Load or create the dimensions that you want to plot the result over.
#' # uwot::umap recommended due to speed, but tSNE or other method would
#' # work as fine.
#' data(testDataSNE)
#'
#' # Run the function for two of the variables
#' dColorPlot(colorData = testData[2:3], xYData = testDataSNE$Y)
#'
#' # Now each depeche cluster is plotted separately and together.
#'
#' # Run the clustering function. For more rapid example execution,
#' # a depeche clustering of the data is included
#' # testDataDepeche <- depeche(testData[,2:15])
#' data(testDataDepeche)
#'
#' dColorPlot(
#'     colorData = testDataDepeche$clusterVector,
#'     xYData = testDataSNE$Y, plotName = "clusters"
#' )
#' }
#' @export dColorPlot
dColorPlot <- function(colorData, controlData, xYData,
                       colorScale = "rich_colors", plotName = "default",
                       densContour = TRUE, title = FALSE, plotDir = "default",
                       truncate = TRUE, bandColor = "black",
                       dotSize = 500 / sqrt(nrow(xYData)),
                       continuous = "default",
                       multiCore = "default",
                       nCores = "default", createOutput = TRUE) {
    if (is.matrix(colorData)) {
        colorData <- as.data.frame(colorData)
    }

    if (is.matrix(xYData)) {
        xYData <- as.data.frame(xYData)
    }

    if (plotDir == "default") {
        if (is.vector(colorData)) {
            plotDir <- "."
        } else {
            plotDir <- paste0("Marker tSNE distributions")
        }
    }

    if (plotDir != ".") {
        dir.create(plotDir)
    }

    if (plotName == "default") {
        plotName <- if (is.data.frame(colorData)) {
            plotName <- colnames(colorData)
        } else {
            plotName <- "Ids"
        }
    }

    if (continuous == "default"){
        if(is.numeric(colorData) && length(unique(colorData)) > 20){
            continuous <- TRUE
        } else {
            continuous <- FALSE
        }
    }
    minScaleVal <- 0
    maxScaleVal <- 1
    if (missing(controlData)) {
        controlData <- colorData
    }

    if(continuous){
        colorData <- as.data.frame(colorData)
        controlData <- as.data.frame(controlData)
        colnames(colorData) <- colnames(controlData) <- "x"
        minScaleVal <- min(controlData[,1])
        maxScaleVal <- max(controlData[,1])
    }

    # Create the density matrix for xYData.
    if (is.logical(densContour)) {
        if (densContour) {
            densContour <- dContours(xYData)
        }
    }

    if (is.vector(colorData) || is.factor(colorData)) {
        wasFactor <- FALSE
        colorDataIsColVec <- FALSE
        if (is.character(colorData)) {
            # Here, we make an exception for pre-made color vectors
            if (nchar(colorData[1]) %in% c(7, 9) &&
                substr(colorData[1], 1, 1) == "#") {
                colorVector <- colorData
                colorDataIsColVec <- TRUE
            }
            colorData <- as.factor(colorData)
        }
        #Now, all are factors, and can be treated as such.
        if (is.factor(colorData)) {
            plotNames <- as.character(unique(colorData))
            colorData <- as.numeric(colorData)
            wasFactor <- TRUE
        }
        uniqueNumsRaw <- unique(colorData)
        uniqueNums <- uniqueNumsRaw[order(uniqueNumsRaw)]
        if (wasFactor) {
            plotNames <- plotNames[order(uniqueNumsRaw)]
        } else {
            plotNames <- uniqueNums
        }
        if (colorDataIsColVec == FALSE) {
            colorVector <- dColorVector(c(colorData),
                colorOrder = uniqueNums,
                colorScale = colorScale
            )
        }
        dPlotCoFunction(
            colorVariable = colorVector, plotName = plotName,
            xYData = xYData, title = title,
            densContour = densContour, bandColor = bandColor,
            dotSize = dotSize, plotDir = plotDir,
            createOutput = createOutput
        )
    } else {
        colorDataRound <- round(dScale(
            x = colorData, control = controlData,
            scale = c(0, 1), robustVarScale = FALSE,
            center = FALSE, multiplicationFactor = 50,
            truncate = truncate
        ))
        colorVectors <- apply(colorDataRound, 2, dColorVector,
            colorScale = colorScale, colorOrder = c(0:50)
        )
        if (multiCore == "default") {
            if (nrow(colorData) > 1e+05 && ncol(colorData) > 4) {
                multiCore <- TRUE
            } else {
                multiCore <- FALSE
            }
        }
        if (multiCore) {
            if (nCores == "default") {
                nCores <- floor(detectCores() * 0.875)
                if (nCores > 10) {
                    nCores <- 10
                }
            }
            cl <- makeCluster(nCores, type = "SOCK")
            registerDoSNOW(cl)
            i <- 1
            return_all <-
                foreach(
                    i = seq_len(ncol(colorVectors)),
                    .packages = "DepecheR"
                ) %dopar%
                dPlotCoFunction(
                    colorVariable = colorVectors[, i],
                    plotName = plotName[i], xYData = xYData,
                    title = title, densContour = densContour,
                    bandColor = bandColor, dotSize = dotSize, plotDir = plotDir,
                    createOutput = createOutput
                )
            stopCluster(cl)
        } else {
            mapply(dPlotCoFunction,
                as.data.frame.matrix(colorVectors, stringsAsFactors = FALSE),
                plotName,
                MoreArgs = list(
                    xYData = xYData,
                    title = title,
                    densContour = densContour,
                    bandColor = bandColor,
                    dotSize = dotSize, plotDir = plotDir,
                    createOutput = createOutput
                )
            )
        }
    }
    # Create a suitable legend for the task
    if (createOutput) {
        if (is.vector(colorData) || continuous) {
            pdf(file.path(plotDir, paste0(plotName, "_legend.pdf")))
        } else {
            pdf(file.path(plotDir, "Color_legend.pdf"))
        }

        if (is.vector(colorData) && length(unique(colorData)) < 50) {
            if (colorDataIsColVec) {
                colorIdsDataFrame <- data.frame(colorData)
            }
            colorIdsDataFrame <- data.frame(
                dColorVector(uniqueNums, colorScale = colorScale), plotNames,
                stringsAsFactors = FALSE
            )

            plot.new()
            legend("center",
                legend = colorIdsDataFrame[, 2],
                col = colorIdsDataFrame[, 1], cex = 7.5 / length(uniqueNums),
                pch = 19
            )
        } else {
            yname <- "Expression level"
            topText <- "Highly expressed"
            bottomText <- "Not expressed"
            par(fig = c(0.35, 0.65, 0, 1), xpd = NA)
            z <- matrix(seq_len(49), nrow = 1)
            x <- 1
            y <- seq(minScaleVal, maxScaleVal, len = 49)
            image(x, y, z,
                col = dColorVector(seq.int(1, 50),
                    colorScale = colorScale
                ),
                axes = FALSE, xlab = "", ylab = yname
            )
            axis(2)
            text(1, maxScaleVal+((maxScaleVal-minScaleVal)*0.1), labels = topText, cex = 1.1)
            text(1,minScaleVal-((maxScaleVal-minScaleVal)*0.1), labels = bottomText, cex = 1.1)
            box()
        }
        dev.off()
    }
}

#' Create a vector of colors of the same length as the data
#'
#'
#' This function takes a vector x and a shorter ordering vector with all the
#' unique values of the x vector in the specific order that the colors should be
#' in and returns a vector of RGB colors the same length as the initial x
#' vector.
#' @importFrom viridis inferno magma plasma viridis
#' @importFrom gplots rich.colors
#' @importFrom grDevices rainbow
#' @param x Any vector.
#' @param colorOrder The order that the colors should be in in the output
#' vector. Defaults to the order that the unique values in x occurs.
#' @param colorScale The color scale. Inherited from the viridis, gplots and
#' grDevices packages (and the package-specific 'dark_rainbow'). Seven possible
#' scales are pre-made: inferno, magma, plasma, viridis, rich_colors, rainbow
#' and dark_rainbow. User specified vectors of colors
#' (e.g. c('#FF0033', '#03AF49')) are also accepted.
#' @return A vector, the same length as x with each unique value substitutet
#' with a color.
#' @seealso \code{\link{dDensityPlot}}, \code{\link{dColorPlot}},
#' \code{\link{dViolins}}
#' @examples
#' # Load some data
#' data(testData)
#'
#' testColor <- dColorVector(testData$ids, colorScale = "plasma")
#'
#' # In this case, each of the 97 individual donors in the dataset has gotten
#' # their own color code:
#' table(testColor)
#' @export dColorVector
dColorVector <- function(x, colorOrder = unique(x), colorScale = "viridis") {
    if (is.factor(x)) {
        x <- as.character(x)
        colorOrder <- as.character(colorOrder)
    }
    nColors <- length(colorOrder)
    if (length(colorScale) > 1) {
        orderColors <- colorRampPalette(colorScale)(nColors)
    } else {
        orderColors <- switch(colorScale,
            "inferno" = inferno(nColors),
            "viridis" = viridis(nColors),
            "plasma" = plasma(nColors),
            "magma" = magma(nColors),
            "rich_colors" = rich.colors(nColors),
            "rainbow" = rainbow(nColors),
            "dark_rainbow" =
                colorRampPalette(c(
                    "#990000", "#FFCC00",
                    "#336600", "#000066",
                    "#660033"
                ))(nColors)
        )
    }

    # Here, a vector with the same length as
    # the x vector is generated, but where
    # the x info has been substituted with a
    # color.
    colorVector <- x
    for (i in seq(0, length(colorOrder))) {
        colorVector[x == colorOrder[i]] <- orderColors[i]
    }

    return(colorVector)
}

#' @importFrom gplots heatmap.2
#' @importFrom stats as.dendrogram order.dendrogram hclust dist
# This function is part of the depeche function. See depeche and dAllocate for 
# most flags. 
# inDataFrameUsed: the scaled indata, where a zoom-in has been made based on
# the samplingSubset.
# reducedClusterCenters: the selected cluster centers, where only variables
# that have not been completely sparsed out are kept. 
depecheHeatMap <- function(inDataFrameUsed, reducedClusterCenters, 
                           plotDir, logCenterSd, createOutput){
    # Here, a heatmap over the cluster
    # centers is saved. Only true if the
    # number of clusters exceeds one.
    if (ncol(reducedClusterCenters) > 500) {
        message("As the number of variables in the result exceeds 500, ",
                "it is not meaningful to produce a cluster center heatmap, ",
                "so it is omitted")
    } else if (nrow(reducedClusterCenters) > 1 &&
               ncol(reducedClusterCenters) > 1) {
        graphicClusterCenters <- reducedClusterCenters
        # Here we scale each center value to the
        # range between the lowest and highest
        # permille of the observations in the
        # inDataScaled for that variable
        for (i in seq_len(ncol(graphicClusterCenters))) {
            scaledFocus <-
                inDataFrameUsed[, colnames(inDataFrameUsed) ==
                                    colnames(graphicClusterCenters)[i]]
            graphicClusterCenters[, i] <- dScale(reducedClusterCenters[, i],
                                                 scaledFocus,
                                                 robustVarScale = FALSE,
                                                 center = FALSE,
                                                 truncate = TRUE)
        }
        
        # Here, the order of the columns are changed, so that the
        # most similar ones are the closest to each other
        colOrder <-
            order.dendrogram(
                as.dendrogram(hclust(dist(t(graphicClusterCenters)))))
        
        graphicClusterCenters[reducedClusterCenters == 0] <- NA
        graphicClusterCenters <- graphicClusterCenters[, colOrder]
        
        colorLadder <- dColorVector(seq_len(11),
                                    colorScale = c(
                                        "#0D0887FF", "#6A00A8FF", "#900DA4FF",
                                        "#B12A90FF", "#CC4678FF", "#E16462FF", 
                                        "#F1844BFF", "#FCA636FF", "#FCCE25FF"
                                    )
        )
        
        if (createOutput) {
            # First, the name is defined, depending on two criteria: if the data
            # was log transformed, and if a directory should be created or not.
            logOrNoLogName <- ifelse(logCenterSd[[1]] == FALSE, "Cluster",
                                     "Log2_transformed_cluster"
            )
            clusterCenterName <- file.path(plotDir, paste0(
                logOrNoLogName,
                "_centers.pdf"
            ))
            pdf(clusterCenterName)
            heatmap.2(as.matrix(graphicClusterCenters),
                      Rowv = FALSE,
                      Colv = FALSE, dendrogram = "none", scale = "none",
                      col = colorLadder, breaks = seq(0, 1, length.out = 12),
                      trace = "none", keysize = 1.5, density.info = "none",
                      key.xlab =
                          "0=no expression, 1=high expression\ngrey=penalized",
                      na.color = "#A2A2A2"
            )
            dev.off()
        }
    }
}

#' Display density on 2D plot
#'
#'
#' Function to show density for a set of observations on a field created by two
#' variables. The plot is constructed primarily for displaying density of
#' 2D-stochastic neighbour embedding fields, but can be used for any sets of
#' two known for the same observations. As the number of datapoints is often
#' very high, the files would, if saved as pdf of another vector based file
#' type become big. For this reason, the plots are saved as jpeg and no axes or
#' anything alike are added, to simplify usage in publications.
#' @param xYData A dataframe or matrix with two columns. Each row contains
#' information about the x and y positition in the field for that observation.
#' @param plotName A name that is common to all density plots created. It can
#' be the groups name, e.g. 'Malaria patients' or 'Clusters'. If only one plot
#' is created, the name is still taken from here.
#' @param idsVector Optional. Vector with the same length as xYData containing
#' information about the id of each observation. If provided, density plots for
#' each individual id and all ids together are produced.
#' @param colorScale This gives the specific color for the densest part of the
#' plot(s). It has three possible values:
#' \describe{
#'     \item{A specific color, e.g. 'red' or '#FF0000'}{If no idsVector
#'     provided}
#'     \item{A color scale from dColorVector}{If idsVector provided. See
#'     \code{\link{dColorVector}} for alternatives.}
#'     \item{"default"}{"One color (blue) if no idsVector is provided, and
#'     otherwise the viridis color scale.}
#' }
#' @param densContour If density contours should be created for the plot(s) or
#' not. Defaults to TRUE. If a density object, as generated by dContours, is
#' included, this will be used for the internal scaling of the plot, allowing
#' for density distribution checks of different subcompartments of the data with
#' the same scaling.
#' @param title If there should be a title displayed on the plotting field. As
#' the plotting field is saved as a png, this title cannot be removed as an
#' object afterwards, as it is saved as coloured pixels. To simplify usage for
#' publication, the default is FALSE, as the files are still named, eventhough
#' no title appears on the plot.
#' @param plotDir If different from the current directory. If not "." and
#' non-existent, the function creates it. Default is "." if idsVector is not
#' specified and otherwise paste0("Density plots for ", plotName, "s").
#' @param bandColor The color of the contour bands. Defaults to black.
#' @param dotSize Simply the size of the dots. The default makes the dots
#' smaller the more observations that are included.
#' @param createOutput For testing purposes. Defaults to TRUE. If FALSE, no
#' output is generated.
#' @seealso \code{\link{dColorPlot}}, \code{\link{dResidualPlot}},
#' \code{\link{dWilcox}}, \code{\link{dColorVector}}
#' @return Plots showing the densities of the specific xYData (subset) displayed
#' as color on the field created by the same xYData (subset).
#' @examples
#'
#' # Load some data
#' data(testData)
#' \dontrun{
#' # Load or create the dimensions that you want to plot the result over.
#' # uwot::umap recommended due to speed, but tSNE or other method would
#' # work as fine.
#' data(testDataSNE)
#'
#' # Plot all data together
#' dDensityPlot(xYData = testDataSNE$Y)
#'
#' # Now each depeche cluster is plotted separately and together.
#'
#' # Run the clustering function. For more rapid example execution,
#' # a depeche clustering of the data is included
#' # testDataDepeche <- depeche(testData[,2:15])
#' data(testDataDepeche)
#'
#' dDensityPlot(
#'     xYData = testDataSNE$Y,
#'     idsVector = testDataDepeche$clusterVector,
#'     plotName = "cluster"
#' )
#' }
#' @export dDensityPlot
dDensityPlot <- function(xYData, colorScale = "default",
                         plotName = "All_density", idsVector,
                         densContour = TRUE, title = FALSE, plotDir = "default",
                         bandColor = "black",
                         dotSize = 500 / sqrt(nrow(xYData)),
                         createOutput = TRUE) {
    if (plotDir == "default") {
        if (missing(idsVector)) {
            plotDir <- "."
        } else {
            plotDir <- paste0("Density plots for ", plotName, "s")
        }
    }

    if (plotDir != ".") {
        dir.create(plotDir)
    }

    if (is.matrix(xYData)) {
        xYData <- as.data.frame(xYData)
    }

    # Create the density matrix for xYData.
    if (is.logical(densContour)) {
        if (densContour) {
            densContour <- dContours(xYData)
        }
    }

    if (missing(idsVector)) {
        color <- ifelse(colorScale == "default", "blue", colorScale)

        dDensityPlotCoFunction(
            xYData = xYData, color = color,
            plotName = plotName,
            densContour = densContour,
            bandColor = bandColor, dotSize = dotSize,
            title = title, plotDir = plotDir,
            createOutput = createOutput
        )
    } else {
        if (colorScale == "default") {
            colorScale <- "viridis"
        }

        uniqueIds <- sort(unique(idsVector))
        colors <- dColorVector(x = uniqueIds, colorScale = colorScale)

        for (i in seq_along(uniqueIds)) {
            dDensityPlotCoFunction(
                xYData = xYData[idsVector == uniqueIds[i], ],
                color = colors[i],
                plotName = paste(plotName, uniqueIds[i],
                    "density",
                    sep = " "
                ),
                densContour = densContour, bandColor = bandColor,
                dotSize = dotSize, title = title, plotDir = plotDir,
                createOutput = createOutput
            )
        }
        dDensityPlotCoFunction(
            xYData = xYData, idsVector = idsVector,
            uniqueIds = uniqueIds, color = colors,
            plotName = plotName, densContour = densContour,
            bandColor = bandColor, dotSize = dotSize,
            title = title, plotDir = plotDir,
            createOutput = createOutput
        )
        # Legend creation
        colorIdsDataFrame <- data.frame(colors, uniqueIds,
            stringsAsFactors = FALSE
        )
        if (createOutput) {
            pdf(file.path(plotDir, paste0(plotName, "_legend.pdf")))
            plot.new()
            legend("center",
                legend = colorIdsDataFrame[, 2],
                col = colorIdsDataFrame[, 1],
                cex = 7.5 / length(uniqueIds), pch = 19
            )
            dev.off()
        }
    }
}

#' Scaling of a vector or a dataframe.
#'
#'
#' This is a scaling function with a number of alternatives. This method for
#' scaling takes the shape of the data into somewhat more of a consideration
#' than minMaxScale does, but still gives less influence of outliers than more
#' conventional scalin alternatives, such as unit variance scaling.
#' @importFrom parallel detectCores makeCluster stopCluster
#' @importFrom doSNOW registerDoSNOW
#' @importFrom foreach foreach %dopar% %do%
#' @param x A numeric/integer vector or dataframe
#' @param control A numeric/integer vector or dataframe of values that could be
#' used to define the range. If no control data is present, the function
#' defaults to using the indata as control data.
#' @param scale If scaling should be performed. Three possible values: a vector
#' with two values indicating the low and high threshold quantiles for the
#' scaling, TRUE, which equals the vector 'c(0.001, 0.999)', and FALSE.
#' @param robustVarScale If the data should be scaled to its standard deviation
#' within the quantiles defined by the scale values above. If TRUE
#' (the default), the data is unit variance scaled based on the standard
#' deviation of the data within the range defined by scale.
#' @param center If centering should be performed. Alternatives are mean',
#' 'peak' and FALSE. 'peak' results in centering around the highest peak in the
#' data, which is useful in most cytometry situations. 'mean' results in mean
#' centering.
#' @param truncate If truncation of the most extreme values should be performed.
#' Three possible values: TRUE, FALSE, and a vector with two values indicating
#' the low and high threshold quantiles for truncation.
#' @param nCores If the function is run in multicore mode, which it will if the
#' dataset is large (nrow*ncol>10^6), this decides the number of cores. The
#' default is currently 87.5 percent with a cap on 10 cores, as no speed
#' increase is generally seen above 10 cores for normal computers to date.
#' @param multiplicationFactor A value that all values will be multiplied with.
#' Useful e.g. if the results preferrably should be returned as percent.
#' @param returnCenter Boolean. If center=TRUE, should the value at the center
#' be returned?
#' @return A vector or dataframe with the same size but where all values in the
#' vector or column of the dataframe have been internally scaled. In addition,
#' if returnCenter=TRUE, a value, or a vector if x is a matrix or a data frame.
#' @examples
#' # Load some data
#' data(testData)
#'
#' # Retrieve the first column
#' x <- testData[, 2]
#'
#' # The maximum and minimum values are
#' max(x)
#' min(x)
#'
#' # Run the function without mean centering and with the quantiles set to 0
#' # and 1.
#' y <- dScale(x, scale = c(0, 1), robustVarScale = FALSE, center = FALSE)
#'
#' # And the data has been scaled to the range between 0 and 1.
#' max(y)
#' min(y)
#'
#' # Now run the default function for a dataframe
#' summary(testData[, 2:15])
#'
#' y_df <- dScale(testData[, 2:15])
#'
#' # Here, the data has first been truncated to the default percentiles, then
#' # scaled to the standard deviation in the remaining interval and finally the
#' # center has been placed where the highest peak in the data is present.
#' # NB! Here, no truncation has been performed in the scaling, only to obtain
#' # the scaling values.
#'
#' summary(y_df)
#' @export dScale
dScale <- function(x, control, scale = TRUE, robustVarScale = TRUE,
                   center = "peak", truncate = FALSE, multiplicationFactor = 1,
                   returnCenter = FALSE, nCores = "default") {
    if (is.matrix(x)) {
        x <- as.data.frame(x)
    }

    if (is.numeric(x) == FALSE &&
        is.integer(x) == FALSE &&
        is.data.frame(x) == FALSE) {
        stop("The data is incorrectly formatted, as it is not a numeric vector,
             a matrix or a dataframe. Change this and try again.")
    }

    if (missing("control")) {
        control <- x
    } else {
        if (is.matrix(control)) {
            control <- as.data.frame(control)
        }
        if (is.data.frame(x)) {
            control <- rbind(x, control)
        } else {
            control <- c(x, control)
        }
    }

    if (is.logical(scale)) {
        if (scale) {
            scale <- c(0.001, 0.999)
        }
    }

    if (is.logical(truncate)) {
        if (truncate) {
            if (is.logical(scale)) {
                truncate <- c(0.001, 0.999)
            } else {
                truncate <- scale
            }
        }
    }

    if (is.data.frame(x) == FALSE) {
        result <- dScaleCoFunction(x,
            control = control, scale = scale,
            robustVarScale = robustVarScale,
            truncate = truncate, center = center,
            multiplicationFactor = multiplicationFactor,
            returnCenter = returnCenter
        )
    }
    if (is.data.frame(x)) {
        if (ncol(x) * nrow(x) > 10000000 && nCores != 1) {
            if (nCores == "default") {
                nCores <- floor(detectCores() * 0.875)
                if (nCores > 10) {
                    nCores <- 10
                }
            }
            cl <- makeCluster(nCores, type = "SOCK")
            registerDoSNOW(cl)
            i <- 1
            result <- foreach(i = seq_len(ncol(x)), .inorder = TRUE) %dopar%
                dScaleCoFunction(x[, i],
                    control = control[, i],
                    scale = scale,
                    robustVarScale = robustVarScale,
                    truncate = truncate,
                    center = center,
                    multiplicationFactor
                    = multiplicationFactor,
                    returnCenter = returnCenter
                )
            stopCluster(cl)
        } else {
            result <- foreach(i = seq_len(ncol(x)), .inorder = TRUE) %do%
                dScaleCoFunction(x[, i],
                    control = control[, i],
                    scale = scale,
                    robustVarScale = robustVarScale,
                    truncate = truncate,
                    center = center,
                    multiplicationFactor
                    = multiplicationFactor,
                    returnCenter = returnCenter
                )
        }
        if (returnCenter == FALSE) {
            result <- as.data.frame(result)
            colnames(result) <- colnames(x)
        } else {
            resultDf <- as.data.frame(do.call(
                "cbind",
                lapply(result, "[[", 1)
            ))
            colnames(resultDf) <- colnames(x)
            resultCenters <- unlist(lapply(result, "[[", 2))
            result <- list(resultDf, resultCenters)
        }
    }

    return(result)
}

###################### dOptPenalty
context("dOptPenatly")
set.seed(11)
x <- DepecheR:::generateBimodalData(observations = 200, dataCols = 10)
x_scaled <- dScale(x$samples, center = FALSE)
dOPR <- DepecheR:::dOptPenalty(x_scaled, k = 30, maxIter = 20,
                               minARIImprovement = 0.01, sampleSize = 100,
                               penalties = c(0, 2, 4, 8, 16, 32, 128),
                               createOutput = FALSE, optimARI = 0.95,
                               nCores = 2, plotDir = ".")

ARI <- dOPR[[2]][, 1]
nClust <- dOPR[[2]][, 2]
bestPenalty <- dOPR$bestPenalty
rowNamesdOpr <- row.names(dOPR[[2]])
nClust[which(rowNamesdOpr == bestPenalty)]
ARI[which(rowNamesdOpr == bestPenalty)]

test_that("dOptPenalty expected output", {
    expect_equal(nClust[which(rowNamesdOpr == bestPenalty)], 2)
    expect_equal(ARI[which(rowNamesdOpr == bestPenalty)], 1)
})

#' Sparse partial least squares discriminant analysis with paired and unpaired
#' data
#'
#'
#' This function is used to compare groups of individuals from whom comparable
#' cytometry or other complex data has been generated. It is superior to just
#' running a Wilcoxon analysis in that it does not consider each cluster
#' individually, but instead uses a sparse partial least squares discriminant
#' analysis to first identify which vector thourgh the multidimensional data
#' cloud, created by the cluster-donor matrix, that optimally separates the
#' groups, and as it is a sparse algorithm, applies a penalty to exclude the
#' clusters that are orthogonal, or almost orthogonal to the discriminant
#' vector, i.e. that do not contribute to separating the groups. This is in
#' large a wrapper for the \code{\link[mixOmics]{splsda}} function from
#' the mixOmics package.
#' @importFrom matrixStats rowMedians
#' @importFrom mixOmics splsda tune.splsda
#' @importFrom ggplot2 ggplot aes geom_density scale_fill_manual
#' scale_x_continuous theme element_blank element_rect ggsave
#' @param xYData A dataframe or matrix with two columns. Each row contains
#' information about the x and y positition in the field for that observation.
#' @param idsVector Vector with the same length as xYData containing information
#' about the id of each observation.
#' @param groupVector Vector with the same length as xYData containing
#' information about the group identity of each observation.
#' @param clusterVector Vector with the same length as xYData containing
#' information about the cluster identity of each observation.
#' @param displayVector Optionally, if the dataset is very large
#' (>100 000 observations) and hence the SNE calculation becomes impossible to
#' perform for the full dataset, this vector can be included. It should contain
#' the set of rows from the data used for statistics, that has been used to
#' generate the xYData.
#' @param testSampleRows Optionally, if a train-test setup is wanted, the rows
#' specified in this vector are used to divide the dataset into a training set,
#' used to generate the analysis, and a test set, where the outcome is predicted
#' based on the outcome of the training set. All rows that are not labeled as
#' test rows are assumed to be train rows.
#' @param paired Defaults to FALSE, i.e. no assumption of pairing is made and
#' Wilcoxon rank sum-test is performed. If true, the software will by default
#' pair the first id in the first group with the firs id in the second group
#' and so forth, so make sure the order is correct!
#' @param plotName The main name for the graph and the analysis.
#' @param densContour If density contours should be created for the plot(s) or
#' not. Defaults to TRUE. a
#' @param groupName1 The name for the first group
#' @param groupName2 The name for the second group
#' @param thresholdMisclassRate This threshold corresponds to the usefulness of
#' the model in separating the groups: a misclassification rate of the default
#' 0.05 means that 5 percent of the individuals are on the wrong side of the
#' theoretical robust middle line between the groups along the sPLS-DA axis,
#' defined as the middle point between the 3:rd quartile of the lower group and
#' the 1:st quartile of the higher group.
#' @param title If there should be a title displayed on the plotting field. As
#' the plotting field is saved as a png, this title cannot be removed as an
#' object afterwards, as it is saved as coloured pixels. To simplify usage for
#' publication, the default is FALSE, as the files are still named, eventhough
#' no title appears on the plot.
#' @param plotDir If different from the current directory. If specified and
#' non-existent, the function creates it. If "." is specified, the plots will be
#' saved at the current directory.
#' @param bandColor The color of the contour bands. Defaults to black.
#' @param dotSize Simply the size of the dots. The default makes the dots
#' smaller the more observations that are included.
#' @param createOutput For testing purposes. Defaults to TRUE. If FALSE, no
#' output is generated.
#' @seealso \code{\link[mixOmics]{splsda}}, \code{\link{dColorPlot}},
#' \code{\link{dDensityPlot}}, \code{\link{dResidualPlot}}
#' @return This function returns the full result of the sPLS-DA. It also returns
#' a SNE based plot showing which events that belong to a cluster dominated by
#' the first or the second group defined by the sparse partial least squares
#' loadings of the clusters.
#' @examples
#'
#' # Load some data
#' data(testData)
#' \dontrun{
#' # Load or create the dimensions that you want to plot the result over.
#' # uwot::umap recommended due to speed, but tSNE or other method would
#' # work as fine.
#' data(testDataSNE)
#'
#' # Run the clustering function. For more rapid example execution,
#' # a depeche clustering of the data is inluded
#' # testDataDepeche <- depeche(testData[,2:15])
#' data(testDataDepeche)
#'
#'
#' # Run the function. This time without pairing.
#' sPLSDAObject <- dSplsda(
#'     xYData = testDataSNE$Y, idsVector = testData$ids,
#'     groupVector = testData$label,
#'     clusterVector = testDataDepeche$clusterVector
#' )
#'
#'
#' # Here is an example of how the display vector can be used.
#' subsetVector <- sample(1:nrow(testData), size = 10000)
#'
#' # Now, the SNE for this displayVector could be created
#' # testDataSubset <- testData[subsetVector, 2:15]
#' # testDataSNESubset <- Rtsne(testDataDisplay, pca=FALSE)$Y
#' # But we will just subset the testDataSNE immediately
#' testDataSNESubset <- testDataSNE$Y[subsetVector, ]
#'
#' # And now, this new SNE can be used for display, although all
#' # the data is used for the sPLS-DA calculations
#' sPLSDAObject <- dSplsda(
#'     xYData = testDataSNESubset, idsVector = testData$ids,
#'     groupVector = testData$label, clusterVector =
#'         testDataDepeche$clusterVector,
#'     displayVector = subsetVector
#' )
#'
#' # Finally, an example of a train-test set situation, where a random half the
#' # dataset is used for training and the second half is used for testing. It
#' # is naturally more biologically interesting to use two independent datasets
#' # for training and testing in the real world.
#' sPLSDAObject <- dSplsda(
#'     xYData = testDataSNE$Y, idsVector = testData$ids,
#'     groupVector = testData$label, clusterVector =
#'         testDataDepeche$clusterVector, testSampleRows = subsetVector
#' )
#' }
#' @export dSplsda
dSplsda <- function(xYData, idsVector, groupVector, clusterVector,
                    displayVector, testSampleRows, paired = FALSE,
                    densContour = TRUE, plotName = "default",
                    groupName1 = unique(groupVector)[1],
                    groupName2 = unique(groupVector)[2],
                    thresholdMisclassRate = 0.05, title = FALSE, plotDir = ".",
                    bandColor = "black", dotSize = 500 / sqrt(nrow(xYData)),
                    createOutput = TRUE) {
    if (plotDir != ".") {
        dir.create(plotDir)
    }


    if (length(unique(groupVector)) != 2) {
        stop("More or less than two groups are present. This is currently not
             supported.")
    }

    if (length(unique(idsVector)) < 8) {
        warning("NB! The number of unique ids is smaller than 8, so statistical
                comparison is not suitable. Use dResidualPlot instead to view
                differences.")
    }

    if (plotName == "default") {
        plotName <- paste0(groupName1, "_vs_", groupName2)
    }

    if (paired) {
        # As the algorithm does not like repeated
        # ids, if the id vector contains the same
        # values for the first group and the
        # second, a new id vector is introduced
        # here
        if (identical(
            unique(idsVector[groupVector == unique(groupVector)[1]]),
            unique(idsVector[groupVector == unique(groupVector)[2]])
        )) {
            pairingVector <- idsVector
            idsVector <- paste0(
                idsVector,
                groupVector
            )
        } else if (length(unique(idsVector[groupVector ==
            unique(groupVector)[1]])) ==
            length(unique(idsVector[groupVector ==
                unique(groupVector)[2]]))) {
            pairingVector <- c(
                idsVector[groupVector == unique(groupVector)[1]],
                idsVector[groupVector == unique(groupVector)[1]]
            )
        } else {
            stop("Pairing cannot be performed, as the first and second" ,
                 "datasets contain different number of individual Ids")
        }
    }

    if (is.matrix(xYData)) {
        xYData <- as.data.frame(xYData)
    }

    if (missing(testSampleRows) == FALSE) {
        clusterVectorTrain <- clusterVector[-testSampleRows]
        idsVectorTrain <- idsVector[-testSampleRows]
        groupVectorTrain <- groupVector[-testSampleRows]
        clusterVectorTest <- clusterVector[testSampleRows]
        idsVectorTest <- idsVector[testSampleRows]
        groupVectorTest <- groupVector[testSampleRows]

        if (paired) {
            pairingVectorTrain <- pairingVector[-testSampleRows]
            pairingVectorTest <- pairingVector[testSampleRows]
        }
    } else {
        clusterVectorTrain <- clusterVector
        idsVectorTrain <- idsVector
        groupVectorTrain <- groupVector
        if (paired) {
            pairingVectorTrain <- pairingVector
        }
    }

    if (paired == FALSE) {
        dSplsdaInData <- dSplsdaPreCalculations(clusterVectorTrain,
            idsVectorTrain,
            groupVectorTrain,
            groupName1 = groupName1,
            groupName2 = groupName2
        )
    } else {
        dSplsdaInData <- dSplsdaPreCalculations(clusterVectorTrain,
            idsVectorTrain,
            groupVectorTrain,
            groupName1 = groupName1,
            groupName2 = groupName2,
            pairingVector
            = pairingVectorTrain
        )
    }

    # Here, the number of clusters that
    # should be kept in the sPLS-DA is chosen
    nVarSPLSDA <- tune.splsda(
        X = t(dSplsdaInData[[1]]), Y = dSplsdaInData[[2]],
        ncomp = 1, logratio = "none",
        test.keepX = dSplsdaInData[[3]],
        validation = "loo", dist = "mahalanobis.dist",
        multilevel = dSplsdaInData[[4]]
    )

    # And here the sPLS-DA is performed.
    # Scaling is performed internally in the
    # algorithm.
    sPLSDAObject <- splsda(
        X = t(dSplsdaInData[[1]]), Y = dSplsdaInData[[2]],
        ncomp = 1, keepX = nVarSPLSDA$choice.keepX,
        multilevel = dSplsdaInData[[4]]
    )

    # Retrieve the x variates for plotting
    sPLSDA_vector <- data.frame(sPLSDAObject$variates$X)
    Group <- dSplsdaInData[[2]]
    densityHist <- cbind(sPLSDA_vector, Group)

    colnames(densityHist) <- c("sPLSDA_vector", "Group")

    # Density plots with semi-transparent
    # fill
    ggplot(densityHist, aes(x = sPLSDA_vector, fill = Group)) +
        geom_density(adjust = 0.2, alpha = 0.4) +
        scale_fill_manual(values = c("red", "blue")) +
        scale_x_continuous(
            limits =
                c(min(densityHist$sPLSDA_vector) -
                    abs(max(densityHist$sPLSDA_vector) -
                        min(densityHist$sPLSDA_vector)) *
                        0.3, max(densityHist$sPLSDA_vector)
                + abs(max(densityHist$sPLSDA_vector) -
                        min(densityHist$sPLSDA_vector))
                    * 0.3)
        ) +
        theme(
            line = element_blank(),
            panel.background = element_rect(fill = "white")
        )

    if (createOutput) {
        ggsave(file.path(plotDir, paste0(plotName, "_sPLSDA_vector.pdf")),
            dpi = 300
        )
    }

    # Retrieve the sparse loadings
    sPLSDALoadings <- sPLSDAObject$loadings$X

    # Here, the maximum values for the
    # plotting are defined. If not added by
    # the user, they are related to the
    # overlap between the populations.
    # Furthermore, the data is re-scaled in
    # here, to be more visually
    # understandable.
    group1SplsDa <-
        densityHist$sPLSDA_vector[which(densityHist$Group == groupName1)]
    group2SplsDa <-
        densityHist$sPLSDA_vector[which(densityHist$Group == groupName2)]

    # Now, the border between the populations
    # is defined

    if (min(group1SplsDa) > min(group2SplsDa)) {
        lowGroup <- group2SplsDa
        highGroup <- group1SplsDa
    } else {
        lowGroup <- group1SplsDa
        highGroup <- group2SplsDa
    }
    groupBorder <- (quantile(lowGroup, probs = 0.75) +
        quantile(highGroup, probs = 0.25)) / 2
    lowErrors <- length(which(highGroup < groupBorder))
    highErrors <- length(which(lowGroup > groupBorder))
    misclassRate <- (lowErrors + highErrors) / sum(
        length(highGroup),
        length(lowGroup)
    )
    if (max(lowGroup) < min(highGroup)) {
        message("The separation of the datasets was perfect, with no overlap ",
              "between the groups")
        lowestPlottedOverlap <- 0
        absSPLSDALoadings <- abs(sPLSDALoadings)
    } else if (misclassRate <= thresholdMisclassRate) {
        message(
            "The separation of the datasets was very clear, with the ",
            "misclassification rate being ", round(100 * misclassRate),
            " percent"
        )
        lowestPlottedOverlap <- misclassRate
        absSPLSDALoadings <- abs(sPLSDALoadings)
    } else {
        message(
            "The separation of the datasets was not clear, with the ",
            "misclassification rate being ", round(100 * misclassRate),
            " percent"
        )
        scalingValue <- thresholdMisclassRate / misclassRate
        absSPLSDALoadings <- abs(sPLSDALoadings * scalingValue)
    }


    # Now, the sign is changed of the
    # splsda-loadings, to make the function
    # generate a similar visual output to the
    # dWilcox function.  Now the median for
    # each group and cluster is calculated
    group1Data <- dSplsdaInData[[1]][, which(
        as.character(dSplsdaInData[[2]]) == groupName1)]
    group2Data <- dSplsdaInData[[1]][, which(
        as.character(dSplsdaInData[[2]]) == groupName2)]

    median1 <- rowMedians(group1Data)
    median2 <- rowMedians(group2Data)
    correctSPLSDALoadings <- absSPLSDALoadings
    for (i in seq_len(nrow(group1Data))) {
        if (median1[i] >= median2[i]) {
            correctSPLSDALoadings[i] <- absSPLSDALoadings[i]
        } else {
            correctSPLSDALoadings[i] <- -absSPLSDALoadings[i]
        }
    }

    # Here, a vector with the same length as
    # the cluster vector is generated, but
    # where the cluster info has been
    # substituted with the statistic.  If a
    # displayVector has been included, it is
    # used here, to subset the clusterVector
    if (missing(displayVector) == FALSE) {
        statisticVector <- clusterVector[displayVector]
        clusterVectorUsed <- clusterVector[displayVector]
    } else {
        statisticVector <- clusterVector
        clusterVectorUsed <- clusterVector
    }

    for (i in seq_len(nrow(correctSPLSDALoadings))) {
        statisticVector[clusterVectorUsed ==
            rownames(correctSPLSDALoadings)[i]] <-
            correctSPLSDALoadings[i]
    }

    # Here the data that will be used for
    # plotting is scaled.
    colnames(xYData) <- c("V1", "V2")

    brks <- seq(-1, 1, length.out = 10)

    # assign each value to a bin
    grps <- cut(statisticVector, breaks = brks, include.lowest = TRUE)
    colors <- colorRampPalette(c("#FF0000", "white", "#0000FF"))(9)
    xYData$col <- colors[grps]

    dPlotCoFunction(
        colorVariable = xYData$col, plotName =
            paste0(plotName, "_sPLSDA_result"),
        xYData = xYData, title = title,
        densContour = densContour, bandColor = bandColor,
        dotSize = dotSize, plotDir = plotDir,
        createOutput = createOutput
    )

    # Create a color legend with text

    yname <- "Misclass-corrected sPLS-DA loadings"

    topText <- paste0(groupName1, " is more abundant")
    bottomText <- paste0(groupName2, " is more abundant")

    if (createOutput) {
        pdf(file.path(plotDir, paste0(plotName, "_sPLS-DA_scale.pdf")))
        par(fig = c(0.35, 0.65, 0, 1), xpd = NA)
        z <- matrix(seq_len(9), nrow = 1)
        x <- 1
        y <- seq(-1, 1, len = 9)
        image(x, y, z, col = colors, axes = FALSE, xlab = "", ylab = yname)
        axis(2)
        text(1, 1 * 1.2, labels = topText, cex = 1.1)
        text(1, -1 * 1.2, labels = bottomText, cex = 1.1)
        box()
        dev.off()
    }

    # Return data from the sPLS-DA that was
    # needed for the generation of the graphs

    if (createOutput) {
        write.csv(
            sPLSDALoadings,
            file.path(plotDir, paste0(plotName, "_sPLSDA_loadings.csv"))
        )
        write.csv(
            sPLSDA_vector,
            file.path(plotDir, paste0(plotName, "_sPLSDA_vector.csv"))
        )
    }

    # Now, prediction is performed, if the
    # setup is train-test.
    if (missing(testSampleRows) == FALSE) {
        if (paired == FALSE) {
            dSplsdaInDataTest <-
                dSplsdaPreCalculations(clusterVectorTest, idsVectorTest,
                    groupVectorTest,
                    groupName1 = groupName1,
                    groupName2 = groupName2
                )
        } else {
            dSplsdaInDataTest <- dSplsdaPreCalculations(clusterVectorTest,
                idsVectorTest,
                groupVectorTest,
                groupName1 = groupName1,
                groupName2 = groupName2,
                pairingVector =
                    pairingVectorTest
            )
        }
        # And here the preciction is performed.
        sPLSDAPredictObject <- predict(
            object = sPLSDAObject,
            newdata = t(dSplsdaInDataTest[[1]]),
            multilevel = dSplsdaInDataTest[[4]]
        )

        # Retrieve the x variates for plotting
        sPLSDAX <- data.frame(sPLSDAPredictObject$variates)
        densityHist <- cbind(sPLSDAX, dSplsdaInDataTest[[2]])

        colnames(densityHist) <- c("sPLSDA_vector", "Group")

        # Density plots with semi-transparent
        # fill
        ggplot(densityHist, aes(x = sPLSDA_vector, fill = Group)) +
            geom_density(adjust = 0.2, alpha = 0.4) +
            scale_fill_manual(values = c("red", "blue")) +
            scale_x_continuous(
                limits
                = c(min(densityHist$sPLSDA_vector) -
                    abs(max(densityHist$sPLSDA_vector) -
                        min(densityHist$sPLSDA_vector))
                    * 0.3, max(densityHist$sPLSDA_vector)
                + abs(max(densityHist$sPLSDA_vector) -
                        min(densityHist$sPLSDA_vector)) *
                        0.3)
            ) +
            theme(
                line = element_blank(),
                panel.background
                = element_rect(
                    fill
                    = "white"
                )
            )
        if (createOutput) {
            ggsave(file.path(
                plotDir,
                paste0(plotName, "_sPLSDA_vector_predicted.pdf")
            ),
            dpi = 300
            )
        }
    }

    if (missing(testSampleRows)) {
        return(sPLSDAObject)
    } else {
        return(list(sPLSDAObject, sPLSDAPredictObject))
    }
}

#' Perform optimization and penalized K-means clustering
#'
#'
#' This is the central function of the package. As input, only a dataset is
#' required. It starts by performing optimizations and then performs clustering
#' based on the values identified in the optimization step.
#' @importFrom grDevices col2rgb colorRampPalette densCols dev.off palette pdf
#' png
#' @importFrom graphics axis contour hist image legend mtext par plot plot.new
#' text box
#' @importFrom stats median p.adjust predict quantile rnorm sd var wilcox.test
#' runif
#' @importFrom utils write.csv tail
#' @importFrom methods is
#' @importFrom ggplot2 ggplot aes geom_line ggtitle xlab ylab ylim ggsave
#' @importFrom dplyr sample_n
#' @importFrom parallel detectCores makeCluster stopCluster
#' @importFrom doSNOW registerDoSNOW
#' @importFrom foreach foreach %dopar%
#' @param inDataFrame A dataframe or matrix with the data that will be used to
#' create the clustering. Cytometry data should be transformed using
#' biexponential, arcsinh transformation or similar, and day-to-day
#' normalizations should to be performed for all data if not all data has been
#' acquired on the same run. Scaling, etc, is on the other hand performed
#' within the function.
#' @param samplingSubset If the dataset is made up of an unequal number of cells
#' from multiple individuals, it might be wise to pre-define a subset of the
#' rows, which includes equal or near-equal numbers of cells from each
#' individual, to avoid a few outliers to dominate the analysis. This can be
#' done here. Should be a vector of row numbers in the inDataFrame.
#' @param penalties This argument decides whether a single penalty will be used
#' for clustering, or if multiple penalties will be evaluated to identify the
#' optimal one. A single value, a vector of values, or possibly a list of two
#' vectors, if dual clustering is performed can be given here. The suggested
#' default values are empirically defined and might not be optimal for a
#' specific dataset, but the algorithm will warn if the most optimal values are
#' on the borders of the range. Note that when the penalty is 0, there is no
#' penalization, which means that the algorithm runs standard K-means
#' clustering.
#' @param sampleSize This controls what fraction of the dataset that will be
#' used to run the penalty optimization. 'default' results in the full file in
#' files up to 10000 events. In cases where the sampleSize argument is larger
#' than 10000, default leads to the generation of a random subset to the same
#' size also for the selectionSampleSize. A user specified number is also
#' accepted.
#' @param selectionSampleSize The size of the dataset used to find the optimal
#' solution out of the many generated by the penalty optimization at each sample
#' size. 'default' results in the full file in files up to 10000 events. In
#' cases where the sampleSize argument is larger than 10000, default leads to
#' the generation of a random subset to the same size also for the
#' selectionSampleSize. A user specified number is also accepted.
#' @param k Number of initial cluster centers. The higher the number, the
#' greater the precision of the clustering, but the computing time also
#' increases linearly with the number of starting points. Default is 30. If
#' penalties=0, k-means clustering with k clusters will be performed.
#' @param minARIImprovement This is the stop criterion for the penalty
#' optimization algorithm: the more iterations that are run, the smaller will
#' the improvement of the corrected Rand index be, and this sets the threshold
#' when the inner iterations stop. Defaults to 0.01.
#' @param maxIter The maximal number of iterations that are performed in the
#' penalty optimization.
#' @param log2Off If the automatic detection for high kurtosis, and followingly,
#' the log2 transformation, should be turned off.
#' @param optimARI Above this level of ARI, all solutions are considered equally
#' valid, and the median solution is selected among them.
#' @param center If centering should be performed. Alternatives are 'default',
#' 'mean', 'peak', FALSE and a vector of numbers with the same length as the
#' number of columns in the inDataFrame. 'peak' results in centering around the
#' highest peak in the data, which is useful in most cytometry situations.
#' 'mean' results in  mean centering. 'default' gives different results
#' depending on the data: datasets with 100+ variables are mean centered,
#' and otherwise, peak centering is used. If a numeric vector is provided, it
#' is used to center the values to the numbers. This is preferable to
#' pre-centering the data and using the FALSE command, as it will lead to better
#' internal visualization procedures, etc. FALSE results in no centering, mainly
#' for testing purposes.
#' @param scale If scaling should be performed. If TRUE, the dataset will be
#' divided by the combined standard deviation of the whole dataset. If a number
#' is provided, the dataset is divided by this number. This scaling
#' procedure makes the default penalties fit most datasets with some precision.
#' @param nCores If multiCore is TRUE, then this sets the number of parallel
#' processes. The default is currently 87.5 percent with a cap on 10 cores, as
#' no speed increase is generally seen above 10 cores for normal computers.
#' @param plotDir Where should the diagnostic plots be printed?
#' @param createOutput For testing purposes. Defaults to TRUE. If FALSE, no
#' plots are generated.
#' @return A nested list:
#' \describe{
#'    \item{clusterVector}{A vector with the same length as number of rows in
#'    the inDataFrame, where the cluster identity of each observation is
#'    noted.}
#'    \item{clusterCenters}{A matrix containing information
#'    about where the centers are in all the variables that contributed to
#'    creating the cluster with the given penalty term. An exact zero here
#'    indicates that the variable in question was sparsed out for that cluster.
#'    If a variable did not contribute to the separation of any cluster, it will
#'    not be present here.}
#'    \item{essenceElementList}{A per-cluster list of the items that were
#'    used to separate that cluster from the rest, i.e. the items that survived
#'    the penalty.}
#'    \item{penaltyOptList}{A list of two dataframes:
#'    \describe{
#'              \item{penaltyOpt.df}{A one row dataframe with the settings for
#'              the optimal penalty.}
#'              \item{meanOptimDf}{A dataframe with the information about the
#'              results with all tested penalty values.}
#'            }
#'     }
#'     \item{logCenterScale}{The values used to center and scale the
#'     data and information on if the data was log transformed. This
#'     information is used internally in dAllocate.}
#' }
#'
#' @examples
#' # Load some data
#' data(testData)
#'
#' # Here a run with the standard settings
#' \dontrun{
#' testDataDepecheResult <- depeche(testData[, 2:15])
#'
#' # Look at the result
#' str(testDataDepecheResult)
#' }
#'
#' @export depeche
depeche <- function(inDataFrame, samplingSubset = seq_len(nrow(inDataFrame)),
                    penalties = 2^seq(0, 5, by = 0.5),
                    sampleSize = "default", selectionSampleSize = "default",
                    k = 30, minARIImprovement = 0.01, optimARI = 0.95,
                    maxIter = 100, log2Off = FALSE, center = "default",
                    scale = TRUE, nCores = "default", plotDir = ".",
                    createOutput = TRUE) {
    if (is.matrix(inDataFrame)) {
        inDataFrame <- as.data.frame.matrix(inDataFrame)
    }
    if (plotDir != ".") {
        dir.create(plotDir)
    }

    logCenterSdResult <- depecheLogCenterSd(inDataFrame, log2Off,
                                               center, scale)
    inDataFrameScaled <- logCenterSdResult[[1]]
    logCenterSd <- logCenterSdResult[[2]]

    # Here, if the dataset is big, a subset of it is used to sample from.
    # Here we have also introduced
    # the samplingSubset that makes it possible to balance a dataset so that each
    # individual gets an equal number of cells as input.
    if (length(samplingSubset) > 1e+06) {
        warning("This dataset is very large, and it is predicted to run for ",
                "a considerable time, without clear benefits in terms of the ",
                "result. A preferred option is to generate the model with a ",
                "sampling subset and then allocate the remaining data points ",
                "using dAllocate.")
    }
    inDataFrameUsed <- inDataFrameScaled[samplingSubset,]

    # Here, the sampleSize is set in cases it
    # is 'default'.
    if (sampleSize == "default") {
        if (nrow(inDataFrameUsed) <= 10000) {
            sampleSize <- nrow(inDataFrameUsed)
        } else {
            sampleSize <- 10000
        }
    }

    if (nCores == "default") {
        nCores <- floor(detectCores() * 0.875)
        if (nCores > 10) {
            nCores <- 10
        }
    }

    penaltyOptResult <- depechePenaltyOpt(inDataFrameUsed,
                                     k = k, maxIter = maxIter,
                                     sampleSize = sampleSize,
                                     penalties = penalties,
                                     createOutput = createOutput,
                                     minARIImprovement = minARIImprovement,
                                     optimARI = optimARI, nCores = nCores,
                                     plotDir = plotDir
    )

    # Now over to creating the final solution
    # Here, the selectionDataSet is created
    if (selectionSampleSize == "default") {
        selectionSampleSize <- sampleSize
    }
    if (nrow(inDataFrameUsed) <= selectionSampleSize) {
        selectionDataSet <- inDataFrameUsed
    } else {
        selectionDataSet <-
            inDataFrameUsed[sample(seq_len(nrow(inDataFrameUsed)),
                                   selectionSampleSize,
                                   replace = TRUE
            ), ]
    }

    # If the dataset is small, a new set of
    # seven clusterings are performed (on all
    # the data or on a subsample, depending
    # on the sample size), and the maximum
    # likelihood solution is returned as the
    # result
    if (nrow(inDataFrameUsed) < 10000) {
        penalty <- penaltyOptResult$bestPenalty
        depecheAllDataResult <-
            depecheAllData(inDataFrameUsed,
                           penalty = penalty, k = k,
                           nCores = nCores
            )
        clusterVector <- depecheAllDataResult[[1]]
        reducedClusterCenters <- depecheAllDataResult[[2]]
    } else {
        # Now, the best run amongst all the runs
        # with the largest sample size is
        # defined, by identifying the solution
        # that gives the highest mean f-measure
        # for all the others.
        reducedClusterCenters <-
            depecheClusterCenterSelection(allSolutions = penaltyOptResult[[3]],
                                          selectionDataSet = selectionDataSet,
                                          k = k, nCores = nCores)

        # And here, the optimal solution is created with the full dataset
        clusterVector <-
            dAllocate(inDataFrameScaled, depModel =
                                   list("clusterCenters" =
                                            reducedClusterCenters,
                                        "logCenterSd" = FALSE))

    }

    # Now, the clusters are ordered according to their size, and re-numbered
    clusterSizeNums <- as.numeric(names(sort(table(clusterVector),
                                             decreasing = TRUE)))
    nClust <- length(clusterSizeNums)
    newNums <- seq(1, nClust)
    clusterVectorNewNums <- clusterVector
    sortedClustSizeNums <- sort(clusterSizeNums)
    row.names(reducedClusterCenters) <- sortedClustSizeNums
    for (i in seq_len(nClust)) {
        localNum <- clusterSizeNums[i]
        clusterVectorNewNums[clusterVector == localNum] <- newNums[i]
        #A ridiculous value is added here to make sure that the row names stay
        #unique.
        row.names(reducedClusterCenters)[sortedClustSizeNums == localNum] <-
            newNums[i]+10000000
    }

    # And here, the rows of the reducedClusterCenters are reordered
    reducedClusterCentersTemp <- reducedClusterCenters[
        order(as.numeric(row.names(reducedClusterCenters))),]
    #Here, we once again risk loosing the data frame format.
    if(is.numeric(reducedClusterCentersTemp)){
        clusterCenterColNames <- colnames(reducedClusterCenters)
        if(nrow(reducedClusterCenters) == 1){
            reducedClusterCenters <-
                as.data.frame(t(reducedClusterCentersTemp))
        } else {
            reducedClusterCenters <-
                data.frame(reducedClusterCentersTemp)
        }
        colnames(reducedClusterCenters) <- clusterCenterColNames
    } else {
        reducedClusterCenters <- reducedClusterCentersTemp
    }

    #Here, the ridiculous cluster number names are removed
    row.names(reducedClusterCenters) <-
        as.numeric(row.names(reducedClusterCenters))-10000000
    # Here, the optPenalty information is
    # retrieved from the optimal sample size
    # run.
    optPenalty <- list("bestPenalty" = penaltyOptResult[[1]],
                       "allPenalties" = penaltyOptResult[[2]])

    #Here, we generate a matrix shoing the ground truth of where the cluster
    #centres lie in the data, before all the normalisation procedures.
    groundClusterCenters <-
        depecheGroundCenters(reducedClusterCenters, logCenterSd)

    # Here, a list of the essential elements for each cluster is generated.
    essenceElementList <-
        lapply(seq_len(nrow(reducedClusterCenters)),
               function(x) {
                   return(colnames(reducedClusterCenters)
                          [which(reducedClusterCenters[x, ] != 0)])
                   })

    #Now, in applicable cases, we produce a heatmap of the cluster centres
    depecheHeatMap(inDataFrameUsed, reducedClusterCenters,
                   plotDir, logCenterSd, createOutput)


    # And now, the result that should be
    # returned is compiled
    depecheResult <- list(
        "clusterVector" = clusterVectorNewNums,
        "clusterCenters" = groundClusterCenters,
        "essenceElementList" = essenceElementList,
        "penaltyOptList" = optPenalty,
        "logCenterSd" = logCenterSd
    )

    if (logCenterSd[[1]]) {
        names(depecheResult)[2] <- "log2ClusterCenters"
    }

    depecheResult
}

#' A 14 color flow cytometry dataset for example execution and playing around
#'
#' This dataset is a 14 color pre-compensated, transformed flow cytometry
#' dataset focusing on cytotoxic lymphocytes, where dead cells have been
#' removed. To make examples very obvious, differences have been artificially
#' exaggregated in the data. The dataset is produced by J. Theorell.
#'
#'
#' @docType data
#'
#' @usage data("testData")
#'
#' @format An object of class \code{'data.frame'};
#'
#' @keywords datasets
"testData"

# This function is used for all major plots in the package. Hence the functions
# dColorPlot, dDensityPlot, dResidualPlot, dSplsda, and dWilcox uses it.
# "Unique" parameters
# colorVariable: the color associated with each point for plotting.
# For information on the other parameters, see any of the plotting functions.
dPlotCoFunction <- function(colorVariable, plotName, xYData, title = FALSE,
                            densContour, bandColor, dotSize, plotDir,
                            createOutput = TRUE) {
    colnames(xYData) <- c("V1", "V2")

    # Create the density matrix for xYData.
    if (is.logical(densContour)) {
        if (densContour) {
            densContour <- dContours(xYData)
        }
    }

    if (length(densContour) > 1) {
        xlim <- c(min(densContour[[1]]), max(densContour[[1]]))
        ylim <- c(min(densContour[[2]]), max(densContour[[2]]))
    } else {
        range1 <- range(xYData[, 1])
        sideDist1 <- 0.05 * (range1[2] - range1[1])
        range2 <- range(xYData[, 2])
        sideDist2 <- 0.05 * (range2[2] - range2[1])

        xlim <- c(
            range1[1] - sideDist1,
            range1[2] + sideDist1
        )
        ylim <- c(
            range2[1] - sideDist2,
            range2[2] + sideDist2
        )
    }

    # Plot it
    main <- ifelse(title, plotName, "")



    if (createOutput) {
        png(file.path(plotDir, paste0(plotName, ".png")),
            width = 2500,
            height = 2500, units = "px", bg = "transparent"
        )

        plot(V2 ~ V1,
            data = xYData, main = main, pch = 20, cex = dotSize,
            cex.main = 5, col = colorVariable, xlim = xlim, ylim = ylim,
            axes = FALSE, xaxs = "i", yaxs = "i"
        )

        if (length(densContour) > 1) {
            par(fig = c(0, 1, 0, 1), mar = c(6, 4.5, 4.5, 2.5), new = TRUE)
            contour(
                x = densContour$x, y = densContour$y, z = densContour$z,
                xlim = xlim, ylim = ylim, nlevels = 10, col = bandColor,
                lwd = 8, drawlabels = FALSE, axes = FALSE, xaxs = "i",
                yaxs = "i"
            )
        }
        dev.off()
    }
}

#' Create violin plots for any variables of choise
#'
#' Here, assymetrical violin plots for each cluster vs all other clusters are
#' plotted for variables either retrieved from a depeche analysis or
#' user-defined.
#' @param clusterVector Vector with the same length as inDataFrame containing
#' information about the cluster identity of each observation.
#' @param inDataFrame The data used to generate the depecheObject
#' @param plotClusters This vector of numbers define which cluster(s) to plot
#' the violins for. Defaults to all.
#' @param plotElements This provides information on which features to plot. In
#' the typical case, this is the essenceElementList from a depeche run. Other
#' input formats are however accepted: if a vector of column names is provided,
#' then these features will be plotted for all clusters. A custom list of
#' features specific for each cluster is also accepted. A final alternative is
#' to return "all" (default), in which case all markers will be plotted for all
#' clusters.If more than a 100 markers are provided, however, this will return
#' an error.
#' @param colorOrder The order of the cluster colors. Defaults to the order
#' that the unique values in clusterVector occurs.
#' @param colorScale The color scale. Options identical to dColorVector.
#' @param plotDir The name of the created directory.
#' @param createOutput For testing purposes. Defaults to TRUE. If FALSE, no
#' plots are generated.
#' @return One graph is created for each cluster, containing a bean per
#' specified variable.
#' @seealso \code{\link{dDensityPlot}}, \code{\link{dColorPlot}},
#' \code{\link{dColorVector}}, \code{\link{depeche}}
#' @examples
#' # Load some data
#' data(testData)
#'
#' # Run the clustering function. For more rapid example execution,
#' # a depeche clustering of the data is inluded
#' # testDataDepeche <- depeche(testData[,2:15])
#' data(testDataDepeche)
#'
#' # Create the plots of the variables that contribute to creating cluster 3
#'
#' \dontrun{
#' dViolins(testDataDepeche$clusterVector,
#'     inDataFrame = testData,
#'     plotClusters = 3, plotElements = testDataDepeche$essenceElementList
#' )
#' }
#' @export dViolins
dViolins <- function(clusterVector, inDataFrame,
                     plotClusters = unique(clusterVector), plotElements = "all",
                     colorOrder = plotClusters, colorScale = "viridis",
                     plotDir = "dViolin_result", createOutput = TRUE) {
    if (createOutput) {
        dir.create(plotDir)
    }

    allClusterCols <- dColorVector(plotClusters,
        colorOrder = colorOrder,
        colorScale = colorScale
    )

    if (length(plotElements) == 1) {
        if (plotElements == "all") {
            plotElements <- colnames(inDataFrame)
            if (length(plotElements) > 100) {
                stop("All features should be plotted, and these are more than ",
                     "100 in total, which is unfeasible. Please specify ",
                     "something more reasonable for plotElements and try ",
                     "again.")
            }
        }
    }

    if (is.list(plotElements) == FALSE) {
        localPlotElementList <-
            list(plotElements)[rep(1, length(plotClusters))]
        plotElements <- localPlotElementList
    }

    # To reduce the size of the objects involved, all features that should
    # not be plotted at all are excluded.

    allPlotElements <- unique(unlist(plotElements))

    inDataFrameReduced <- inDataFrame[, allPlotElements]

    # Here the remaining data is truncated.
    inDataFrameTrunc <- dScale(inDataFrameReduced,
        scale = TRUE,
        robustVarScale = FALSE, center = FALSE,
        truncate = TRUE
    )

    # And now, the inner function is applied once for each cluster
    lapply(seq_along(plotClusters), function(x) {
        dViolinsCoFunction(
            clusterNum = plotClusters[x],
            plotElement = plotElements[[x]],
            clusterVector = clusterVector, inDataFrameTrunc = inDataFrameTrunc,
            clusterCol = allClusterCols[x], plotDir = plotDir,
            createOutput = createOutput
        )
    })
}

#This function is used internally in the depeche function.
#allSolutions: this is an object that should be derived from the
#depechePenaltyOpt function output, which saves all the cluster matrices that
#are generated in the optimization process.
#selectionDataSet: the name speaks for itself
#k: same meaning as in depeche, i.e. the number of startpoint cluster centers.
depecheClusterCenterSelection <- function(allSolutions, selectionDataSet, k,
                                          nCores){
    # Now, all clusterCenters are used to
    # allocate the selectionDataSet.
    allocationResultList <-
        lapply(allSolutions, function(x) {
            dAllocate(inDataFrame = selectionDataSet, depModel =
                                   list("clusterCenters" = x,
                               "logCenterSd" = FALSE))
        })

    # Here, the corrected Rand index with
    # each allocationResult as the first
    # vector vector and all the others as
    # individual second vectors is identified
    cl <- makeCluster(nCores, type = "SOCK")
    registerDoSNOW(cl)
    meanARIList <-
        foreach(i = seq_len(length(allocationResultList))) %dopar%
        mean(vapply(allocationResultList,
                    FUN.VALUE = 0.5, rand_index,
                    inds2 = allocationResultList[[i]], k = k
        ))
    stopCluster(cl)
    meanARIVector <- unlist(meanARIList)

    # Now the solution being the most similar
    # to all the others is retrieved
    optimalClusterCenters <-
        unlist(allSolutions[[which(meanARIVector ==
                                       max(meanARIVector))[1]]])
    colnames(optimalClusterCenters) <- colnames(selectionDataSet)
    # Remove all rows and columns that do not
    # contain any information
    reducedClusterCenters <-
        optimalClusterCenters[
            which(rowSums(optimalClusterCenters) != 0),
            which(colSums(optimalClusterCenters) != 0)
            ]
    #If the clusters are separated by
    #one factor only, or if, by some strange coincidence, it has been possible to create
    #only one cluster, which should be impossible,
    #then we need to turn this back into a dataframe here.
    if(is.numeric(reducedClusterCenters)){
        if (length(which(rowSums(optimalClusterCenters) != 0)) == 1) {
            reducedClusterCenters <- as.data.frame(t(reducedClusterCenters))
        } else if (length(which(colSums(optimalClusterCenters) != 0)) == 1) {
            reducedClusterCenters <- data.frame(reducedClusterCenters)
        }
        colnames(reducedClusterCenters) <-
            colnames(optimalClusterCenters)[which(colSums(optimalClusterCenters) != 0)]
    }
    reducedClusterCenters
}

# This function is used by depeche
# The reason this level of complexity is needed is to handle both single-run
# depeche and dual-run depeche. In the latter case, this function is called
# multiple times.
# "Unique" parameters
# inDataFrameScaled: the scaled version of inDataFrame. See initial part of
# depeche function for details.
# firstClusterNumber: this defines if the first number for the cluster
# definitions.
# plotDir: should a directory be created? In practice, FALSE for single
# depeche and TRUE for the second part of dual depeche.
# For information on the other parameters, see depeche.
#' @importFrom gplots heatmap.2
#' @importFrom graphics box
#' @importFrom ggplot2 ggplot aes geom_line ggtitle xlab ylab ylim ggsave
#' @importFrom dplyr sample_n
#' @importFrom parallel detectCores makeCluster stopCluster
#' @importFrom doSNOW registerDoSNOW
#' @importFrom foreach foreach %dopar%
#' @importFrom stats as.dendrogram order.dendrogram hclust dist
#' @useDynLib DepecheR
depecheCoFunction <- function(inDataFrameScaled, samplingSubset, 
                              firstClusterNumber = 1,
                              plotDir, penalties, sampleSize,
                              selectionSampleSize, k, minARIImprovement,
                              optimARI, maxIter, newNumbers, nCores,
                              createOutput, logCenterSd) {
    if (plotDir != ".") {
        dir.create(plotDir)
    }

    # Here, if the dataset is big, a subset of it is used to sample from. 
    # Here we have also introduced
    # the sampleSubset that makes it possible to balance a dataset so that each
    # individual gets an equal number of cells as input. 
    inDataFrameUsed <- inDataFrameScaled[samplingSubset,]
    
    if (nrow(inDataFrameUsed) > 1e+06) {
        sampleRows <- sample(seq_len(nrow(inDataFrameUsed)), 1e+06)
        inDataFrameUsed <- inDataFrameUsed[sampleRows, ]
    }


    # Here, the sampleSize is set in cases it
    # is 'default'.
    if (sampleSize == "default") {
        if (nrow(inDataFrameUsed) <= 10000) {
            sampleSize <- nrow(inDataFrameUsed)
        } else {
            sampleSize <- 10000
        }
    }

    if (nCores == "default") {
        nCores <- floor(detectCores() * 0.875)
        if (nCores > 10) {
            nCores <- 10
        }
    }

    dOptPenaltyResult <- dOptPenalty(inDataFrameUsed,
        k = k, maxIter = maxIter,
        sampleSize = sampleSize,
        penalties = penalties,
        createOutput = createOutput,
        minARIImprovement = minARIImprovement,
        optimARI = optimARI, nCores = nCores,
        plotDir = plotDir
    )

    # Now over to creating the final solution
    # Here, the selectionDataSet is created
    if (selectionSampleSize == "default") {
        selectionSampleSize <- sampleSize
    }
    if (nrow(inDataFrameUsed) <= selectionSampleSize) {
        selectionDataSet <- inDataFrameUsed
    } else {
        selectionDataSet <-
            inDataFrameUsed[sample(seq_len(nrow(inDataFrameUsed)),
                selectionSampleSize,
                replace = TRUE
            ), ]
    }

    # If the dataset is small, a new set of
    # seven clusterings are performed (on all
    # the data or on a subsample, depending
    # on the sample size), and the maximum
    # likelihood solution is returned as the
    # result
    if (nrow(inDataFrameUsed) < 10000) {
        penalty <- dOptPenaltyResult$bestPenalty
        depecheAllDataResult <-
            depecheAllData(inDataFrameUsed,
                penalty = penalty, k = k,
                nCores = nCores
            )
        clusterVector <- depecheAllDataResult[[1]]
        reducedClusterCenters <- depecheAllDataResult[[2]]
    } else {
        # Now, the best run amongst all the runs
        # with the largest sample size is
        # defined, by identifying the solution
        # that gives the highest mean f-measure
        # for all the others.  First, all
        # solutions are retrieved
        allSolutions <- dOptPenaltyResult[[3]]

        # Now, all clusterCenters are used to
        # allocate the selectionDataSet.
        allocationResultList <-
            lapply(allSolutions, function(x) {
                  dAllocate(
                      inDataFrame = selectionDataSet, depModel = x
                  )
              })

        # Here, the corrected Rand index with
        # each allocationResult as the first
        # vector vector and all the others as
        # individual second vectors is identified
        cl <- makeCluster(nCores, type = "SOCK")
        registerDoSNOW(cl)
        meanARIList <-
            foreach(i = seq_len(length(allocationResultList))) %dopar%
            mean(vapply(allocationResultList,
                FUN.VALUE = 0.5, rand_index,
                inds2 = allocationResultList[[i]], k = k
            ))
        stopCluster(cl)
        meanARIVector <- unlist(meanARIList)

        # Now the solution being the most similar
        # to all the others is retrieved
        optimalClusterCenters <- unlist(allSolutions[[which(meanARIVector ==
            max(meanARIVector))[1]]])
        colnames(optimalClusterCenters) <- colnames(inDataFrameUsed)
        # Remove all rows and columns that do not
        # contain any information
        reducedClusterCenters <-
            optimalClusterCenters[
                which(rowSums(optimalClusterCenters) != 0),
                which(colSums(optimalClusterCenters) != 0)
            ]

        # In the specific case that only one row
        # is left, due to a high penalty, the
        # data needs to be converted back to a
        # matrix from a vector. The same is done
        # if the number of informative variables
        # is just one.
        if (length(which(rowSums(optimalClusterCenters) != 0)) == 1) {
            reducedClusterCenters <- t(reducedClusterCenters)
        } else if (length(which(colSums(optimalClusterCenters) != 0) == 1)) {
            reducedClusterCenters <- as.matrix(reducedClusterCenters)
        }

        # And here, the optimal solution is
        # created with the full dataset
        clusterVector <- dAllocate(inDataFrameScaled,
            reducedClusterCenters)
    }

    # Now, the clusters are ordered according to their size, and re-numbered
    clusterSizeNums <- as.numeric(names(sort(table(clusterVector),
        decreasing = TRUE
    )))
    nClust <- length(clusterSizeNums)
    newNums <- seq(firstClusterNumber, firstClusterNumber + (nClust - 1))
    clusterVectorNewNums <- clusterVector
    sortedClustSizeNums <- sort(clusterSizeNums)
    row.names(reducedClusterCenters) <- sortedClustSizeNums
    for (i in seq_len(nClust)) {
        localNum <- clusterSizeNums[i]
        clusterVectorNewNums[clusterVector == localNum] <- newNums[i]
        row.names(reducedClusterCenters)[sortedClustSizeNums == localNum] <-
            newNums[i]
    }

    # And here, the rows of the reducedClusterCenters are reordered
    reducedClusterCenters <- reducedClusterCenters[
        order(as.numeric(row.names(reducedClusterCenters))),
    ]

    # Here, the optPenalty information is
    # retrieved from the optimal sample size
    # run.
    optPenalty <- list("bestPenalty" = dOptPenaltyResult[[1]], 
                       "allPenalties" = dOptPenaltyResult[[2]])

    # Here, a cluster center matrix is
    # created that relates its values to the
    # original, indata variables, which
    # increases the interpretability vastly.
    # First, the cluster centers are
    # multiplied by the standar deviation of
    # the data
    clusterCentersMultSd <- reducedClusterCenters * logCenterSd[[3]]

    # Then, the center term is added to all
    # variables separately
    if (is.logical(logCenterSd[[2]]) == FALSE) {
        correctClusterCentersList <- list()
        for (i in seq_len(ncol(clusterCentersMultSd))) {
            focusMean <- logCenterSd[[2]][which(names(logCenterSd[[2]]) %in%
                colnames(clusterCentersMultSd)[i])]
            correctClusterCentersList[[i]] <- clusterCentersMultSd[, i]
            +focusMean
        }
        correctClusterCenters <- do.call("cbind", correctClusterCentersList)
        colnames(correctClusterCenters) <- colnames(reducedClusterCenters)
    } else {
        correctClusterCenters <- clusterCentersMultSd
        colnames(correctClusterCenters) <- colnames(reducedClusterCenters)
    }

    # Here, a list of the essential elements for each cluster is generated.
    essenceElementList <- lapply(seq_len(nrow(reducedClusterCenters)), 
                                 function(x) {
        return(colnames(
            reducedClusterCenters)[which(reducedClusterCenters[x, ] != 0)])
    })

    # Here, a heatmap over the cluster
    # centers is saved. Only true if the
    # number of clusters exceeds one.
    if (ncol(reducedClusterCenters) > 500) {
        message("As the number of variables in the result exceeds 500, ",
                "it is not meaningful to produce a cluster center heatmap, ",
                "so it is omitted")
    } else if (nrow(reducedClusterCenters) > 1 &&
        ncol(reducedClusterCenters) > 1) {
        graphicClusterCenters <- reducedClusterCenters
        # Here we scale each center value to the
        # range between the lowest and highest
        # permille of the observations in the
        # inDataScaled for that variable
        for (i in seq_len(ncol(graphicClusterCenters))) {
            scaledFocus <-
                inDataFrameUsed[, colnames(inDataFrameUsed) ==
                    colnames(graphicClusterCenters)[i]]
            graphicClusterCenters[, i] <- dScale(reducedClusterCenters[, i],
                scaledFocus,
                robustVarScale = FALSE,
                center = FALSE,
                truncate = TRUE
            )
        }

        # Here, the order of the columns are changed, so that the
        # most similar ones are the closest to each other
        colOrder <-
            order.dendrogram(
                as.dendrogram(hclust(dist(t(graphicClusterCenters))))
            )

        graphicClusterCenters[reducedClusterCenters == 0] <- NA

        graphicClusterCenters <- graphicClusterCenters[, colOrder]

        colorLadder <- dColorVector(seq_len(11),
            colorScale = c(
                "#0D0887FF", "#6A00A8FF", "#900DA4FF", "#B12A90FF",
                "#CC4678FF", "#E16462FF", "#F1844BFF", "#FCA636FF",
                "#FCCE25FF"
            )
        )

        if (createOutput) {
            # First, the name is defined, depending on two criteria: if the data
            # was log transformed, and if a directory should be created or not.
            logOrNoLogName <- ifelse(logCenterSd[[1]] == FALSE, "Cluster",
                "Log2_transformed_cluster"
            )
            clusterCenterName <- file.path(plotDir, paste0(
                logOrNoLogName,
                "_centers.pdf"
            ))
            pdf(clusterCenterName)
            heatmap.2(as.matrix(graphicClusterCenters),
                Rowv = FALSE,
                Colv = FALSE, dendrogram = "none", scale = "none",
                col = colorLadder, breaks = seq(0, 1, length.out = 12),
                trace = "none", keysize = 1.5, density.info = "none",
                key.xlab =
                    "0=no expression, 1=high expression\ngrey=penalized",
                na.color = "#A2A2A2"
            )
            dev.off()
        }
    }

    # And now, the result that should be
    # returned is compiled
    depecheResult <- list(
        clusterVectorNewNums, correctClusterCenters,
        essenceElementList, optPenalty
    )
    names(depecheResult) <- c(
        "clusterVector", "clusterCenters",
        "essenceElementList", "penaltyOptList"
    )
    if (logCenterSd[[1]]) {
        names(depecheResult)[2] <- "log2ClusterCenters"
    }

    return(depecheResult)
}

#' @importFrom moments kurtosis
#This function is used internally in depeche and dAllocate.
#For all options, see depeche or dAllocate, to whom this function is a
#sub-module. It generates a list containing a scaled and centered dataframe, as
#well as a list that clarifies if and how log, center and scale were applied.
depecheLogCenterSd <- function(inDataFrame, log2Off, center, scale){
    logCenterSd <- list(FALSE, FALSE, 1)
    # Here it is checked if the data has very
    # extreme tails, and if so, the data is
    # log2 transformed
    if (log2Off == FALSE && kurtosis(as.vector(as.matrix(inDataFrame))) >
        100) {
        kurtosisValue1 <- kurtosis(as.vector(as.matrix(inDataFrame)))
        # Here, the log transformation is
        # performed. In cases where the lowest
        # value is 0, everything is simple. In
        # other cases, a slightly more
        # complicated formula is needed
        if (min(inDataFrame) >= 0) {
            inDataFrame <- log2(inDataFrame + 1)
            logCenterSd[[1]] <- TRUE
        } else {
            # First, the data needs to be reasonably
            # log transformed to not too extreme
            # values, but still without loosing
            # resolution.
            inDataMatrixLog <- log2(apply(
                inDataFrame, 2,
                function(x) x - min(x)
            ) + 1)
            # Then, the extreme negative values will
            # be replaced by 0, as they give rise to
            # artefacts.
            inDataMatrixLog[which(is.nan(inDataMatrixLog))] <- 0
            inDataFrame <- as.data.frame(inDataMatrixLog)
            logCenterSd[[1]] <- TRUE
        }

        kurtosisValue2 <- kurtosis(as.vector(as.matrix(inDataFrame)))
        message(
            "The data was found to be heavily tailed (kurtosis ",
            kurtosisValue1, "). Therefore, it was log2-transformed, leading to
            a new kurtosis value of ", kurtosisValue2, "."
        )
    }

    # Centering and overall scaling is
    # performed
    if (length(center) == 1 && (center == "peak" ||
        (center == "default" && ncol(inDataFrame) <= 100))) {
        inDataFrameScaleList <- dScale(inDataFrame,
                                       scale = FALSE,
                                       center = "peak", returnCenter = TRUE
        )
        inDataFramePreScaled <- inDataFrameScaleList[[1]]
        logCenterSd[[2]] <- inDataFrameScaleList[[2]]
        if(ncol(inDataFrame) <= 100){
            message("As the dataset has less than 100 columns, ",
                    "peak centering is applied.")
        } else {
            message("Peak centering is applied although the data has ",
                    "more than 100 columns")
        }
    } else if (length(center) == 1 && (center == "mean" ||
               (center == "default" && ncol(inDataFrame) > 100))) {
        inDataFrameScaleList <- dScale(inDataFrame,
                                       scale = FALSE,
                                       center = "mean", returnCenter = TRUE
        )
        inDataFramePreScaled <- inDataFrameScaleList[[1]]
        logCenterSd[[2]] <- inDataFrameScaleList[[2]]
        if(ncol(inDataFrame) <= 100){
            message("Mean centering is applied although the data has ",
                    "less than 100 columns")
        } else {
            message("As the dataset has more than 100 columns, ",
                    "mean centering is applied.")
        }
    } else if (length(center) > 1){
        if(length(center) == length(inDataFrame)){
            inDataFramePreScaled <-
                as.data.frame(do.call(
                    "cbind",lapply(seq_len(ncol(inDataFrame)),
                                   function(x){
                                       inDataFrame[,x]-center[x]
                                       })))
            dimnames(inDataFramePreScaled) <- dimnames(inDataFrame)
            logCenterSd[[2]] <- center
            message("Centering based on the provded values")
        } else {
            stop("Mismatch between the number of columns and the length of",
                 " the centering vector")
        }

    } else if (center == FALSE) {
        message("No centering performed")
        inDataFramePreScaled <- inDataFrame
    }

    # Here, all the data is divided by the
    # standard deviation of the full dataset
    if(is.logical(scale) && scale){
        sdInDataFramePreScaled <- sd(as.matrix(inDataFramePreScaled))
        inDataFrameScaled <-
            as.data.frame(inDataFramePreScaled / sdInDataFramePreScaled)
        logCenterSd[[3]] <- sdInDataFramePreScaled
    } else if(is.numeric(scale)){
        inDataFrameScaled <-
            as.data.frame(inDataFramePreScaled / scale)
        logCenterSd[[3]] <- scale
        message("Scaling value taken directly from input")
    }

    return(list(inDataFrameScaled, logCenterSd))
}

# This function is part of hte depeche function
# Here, a cluster center matrix is created that relates its values to the
# original indata variables, which increases the interpretability vastly.
# "Unique" flags:
# reducedClusterCenters: the selected cluster centers, where only variables
# that have not been completely sparsed out are kept.
depecheGroundCenters <- function(reducedClusterCenters, logCenterSd){

    # First, the cluster centers are multiplied by the standard deviation of
    # the data
    clusterCentersMultSd <- reducedClusterCenters * logCenterSd[[3]]

    # Then, the center term is added to all
    # variables separately
    if (is.logical(logCenterSd[[2]]) == FALSE) {

        groundClusterCenters <-
            do.call("cbind",lapply(seq_len(ncol(clusterCentersMultSd)),
                                   function(i){
                clusterCentersMultSd[, i] + logCenterSd[[2]][i]
        }))
        groundClusterCenters[which(reducedClusterCenters == 0)] <- 0
    } else {
        groundClusterCenters <- clusterCentersMultSd
    }
    dimnames(groundClusterCenters) <- dimnames(reducedClusterCenters)
    groundClusterCenters
}

###################### dWilcox
context("dWilcox")
data("testData")
data("testDataDepeche")
data("testDataSNE")
allDataRows <- c(
    1:200, 1001:1200, 2001:2200, 3001:3200, 4001:4200,
    10001:10200, 11001:11200, 12001:12200, 13001:13200,
    14001:14200
)
xYData <- testDataSNE$Y[allDataRows, ]
idsVector <- testData$ids[allDataRows]
groupVector <- testData$label[allDataRows]
clusterVector <- testDataDepeche$clusterVector[allDataRows]

result <- dWilcox(
    xYData = xYData, idsVector = idsVector,
    groupVector = groupVector,
    clusterVector = clusterVector, createOutput = FALSE
)

# Alternative usage
subsetVector <- sample(1:nrow(testData), size = 1000)

result <- dWilcox(
    xYData = xYData, idsVector = idsVector,
    groupVector = groupVector, clusterVector = clusterVector,
    displayVector = subsetVector, createOutput = FALSE
)

###################### dViolins
context("dViolins")
data("testData")
data("testDataDepeche")
dViolins(
    clusterVector = testDataDepeche$clusterVector, inDataFrame = testData,
    plotClusters = 1, plotElements = testDataDepeche$essenceElementList,
    createOutput = FALSE
)

# Used internally in dSplsda.
# This function creates the necessary tables with each patient id and each
# cluster separated and the settings for the sPLS-DA analysis.
# For information about hte parameters, see dSplsda.
dSplsdaPreCalculations <- function(clusterVector,
                                   idsVector, groupVector, pairingVector, 
                                   groupName1, groupName2) {
    # Here, the statistical evaluation is
    # performed.  First, the data is divided
    # into each group.
    clusterVectorGroup1 <- clusterVector[groupVector == unique(groupVector)[1]]
    clusterVectorGroup2 <- clusterVector[groupVector == unique(groupVector)[2]]
    idsVectorGroup1 <-
        as.character(idsVector[groupVector == unique(groupVector)[1]])
    idsVectorGroup2 <-
        as.character(idsVector[groupVector == unique(groupVector)[2]])

    # Now, a table with the percentage of
    # cells in each cluster for each
    # individual is created for both groups

    clusterTable1 <-
        as.matrix(as.data.frame.matrix(table(
            clusterVectorGroup1,
            idsVectorGroup1
        )))
    clusterTable2 <-
        as.matrix(as.data.frame.matrix(table(
            clusterVectorGroup2,
            idsVectorGroup2
        )))

    # In the very unlikely event that there
    # is not a single observation for one
    # cluster from one of the groups, this
    # cluster is substituted to that table
    # with a row of zeros.
    if (nrow(clusterTable1) < length(unique(clusterVector))) {
        zeroMat <- matrix(data = 0, nrow = length(unique(clusterVector)) -
            nrow(clusterTable1), ncol = ncol(clusterTable1))

        colnames(zeroMat) <- colnames(clusterTable1)
        allRowNames <- as.character(sort(unique(clusterVector)))
        row.names(zeroMat) <-
            allRowNames[-which(allRowNames %in% row.names(clusterTable1))]
        # Here, rows are added to the cluster
        # table to make the number of rows the
        # same as the unique values of the
        # cluster vector.
        clusterTable1big <- rbind(clusterTable1, zeroMat)

        # The rows of the table are re-sorted
        clusterTable1bigResorted <-
            clusterTable1big[order(as.numeric(row.names(clusterTable1big))), ]
        clusterTable1 <- clusterTable1bigResorted
    }

    # And the same procedure is done for the
    # second group
    if (nrow(clusterTable2) < length(unique(clusterVector))) {
        zeroMat <- matrix(data = 0, nrow = length(unique(clusterVector)) -
            nrow(clusterTable2), ncol = ncol(clusterTable2))

        colnames(zeroMat) <- colnames(clusterTable2)
        allRowNames <- as.character(sort(unique(clusterVector)))
        row.names(zeroMat) <- allRowNames[-which(allRowNames %in%
            row.names(clusterTable2))]
        # Here, rows are added to the cluster
        # table to make the number of rows the
        # same as the unique values of the
        # cluster vector.
        clusterTable2big <- rbind(clusterTable2, zeroMat)

        # The rows of the table are re-sorted
        clusterTable2bigResorted <-
            clusterTable2big[order(as.numeric(row.names(clusterTable2big))), ]
        clusterTable2 <- clusterTable2bigResorted
    }


    countTable1 <- table(idsVectorGroup1)
    countTable2 <- table(idsVectorGroup2)

    clusterFractionsForAllIds1 <- clusterTable1
    clusterFractionsForAllIds2 <- clusterTable2


    for (i in seq_len(length(countTable1))) {
        x <- clusterTable1[, i] / countTable1[i]
        clusterFractionsForAllIds1[, i] <- x
    }

    for (i in seq_len(length(countTable2))) {
        x <- clusterTable2[, i] / countTable2[i]
        clusterFractionsForAllIds2[, i] <- x
    }

    if (missing(pairingVector) == FALSE) {
        # Here, a comparable pairing vector pair
        # is created if a multilevel sPLS-DA
        # should be performed.

        pairingVectorGroup1 <-
            as.character(pairingVector[groupVector == unique(groupVector)[1]])
        pairingVectorGroup2 <-
            as.character(pairingVector[groupVector == unique(groupVector)[2]])

        pairingShortGroup1 <- clusterFractionsForAllIds1[1, ]

        for (i in seq_len(ncol(clusterFractionsForAllIds1))) {
            pairingShortGroup1[i] <-
                pairingVectorGroup1[
                    which(colnames(clusterFractionsForAllIds1)[i]
                    == idsVectorGroup1)[1]
                ]
        }

        pairingShortGroup2 <- clusterFractionsForAllIds2[1, ]

        for (i in seq_len(ncol(clusterFractionsForAllIds2))) {
            pairingShortGroup2[i] <-
                pairingVectorGroup2[
                    which(colnames(clusterFractionsForAllIds2)[i]
                    == idsVectorGroup2)[1]
                ]
        }

        pairingAll <- c(pairingShortGroup1, pairingShortGroup2)
    } else {
        pairingAll <- NULL
    }

    # A group vector is created with the same
    # length as the number of columns in the
    # tables
    groupId <- as.factor(c(
        rep(groupName1, ncol(clusterFractionsForAllIds1)),
        rep(groupName2, ncol(clusterFractionsForAllIds2))
    ))

    # These two tables are combined to one
    clusterFractionsForAllIds <- as.matrix(cbind(
        clusterFractionsForAllIds1,
        clusterFractionsForAllIds2
    ))

    # Here, the number of possible clusters
    # to be saved in the sPLS-DA is chosen.
    # The number of tested clusters can be up
    # to five, but if the number of clusters
    # is low, the lowest tested variant will
    # be 1 and that will be tested only once.
    testKeepAlternatives <- vector()
    for (i in seq_len(5)) {
        if (i == 1) {
            testKeepAlternatives[i] <- nrow(clusterFractionsForAllIds)
        } else {
            if (testKeepAlternatives[i - 1] > 1) {
                testKeepAlternatives[i] <- 
                    round(testKeepAlternatives[i - 1] / 2)
            } else {
                break
            }
        }
    }

    return(list(
        clusterFractionsForAllIds, groupId, testKeepAlternatives,
        pairingAll
    ))
}

#' Create density contours for two-dimensional data.
#'
#'
#' Here, contour lines for two-dimensional data are construced. It is primarily
#' thought to be used in the context of SNE plots in this package. This function
#' is used both internally in other functions suchas sneFluoroPlot and
#' sneDensityPlot, but also as a standalone function, as it increases speed
#' greatly to generate the density curves only once per overall analysis.
#' @importFrom MASS kde2d
#' @param xYData A dataframe with two columns containing position information
#' for each observation in the dataset. Typically, this is the raw result from
#' the SNE analysis.
#' @param control A numeric/integer vector or dataframe of values that could be
#' used to define the range in the internal dScale. If no control data is
#' present, the function defaults to using the indata as control data.
#' @param n The number fo grid points. Default is 100.
#' @seealso \code{\link{dColorPlot}}, \code{\link{dDensityPlot}},
#' \code{\link{dResidualPlot}}, \code{\link{dWilcox}}
#' @return A list of three components
#' \describe{
#'     \item{x, y}{The x and y coordinates of the grid points,
#'     vectors of length n.}
#'     \item{z}{An n[1] by n[2] matrix of the estimated density: rows
#'     correspond to the value of x, columns to the value of y.}
#' }
#'
#' @examples
#' # Load the test SNE data
#' data(testDataSNE)
#'
#'
#' # Run the function
#' contour_result <- dContours(testDataSNE$Y)
#' @export dContours
dContours <- function(xYData, control, n = 100) {
    if (missing(control)) {
        control <- xYData
    }

    range1 <- range(control[, 1])
    sideDist1 <- 0.05 * (range1[2] - range1[1])
    range2 <- range(control[, 2])
    sideDist2 <- 0.05 * (range2[2] - range2[1])

    lims <- c(
        range1[1] - sideDist1,
        range1[2] + sideDist1,
        range2[1] - sideDist2,
        range2[2] + sideDist2
    )

    # Construct the third dimension with
    # smooth kernel density estimate
    den3d <- kde2d(xYData[, 1], xYData[, 2], n = n, lims = lims)

    return(den3d)
}

# This fucntion is used internally in dScaleCoFunction.
# Ther purpose with the truncation is to decrease the influence of extreme
# outliers on variance calculations and on visualizations.
# For information on the different parameters, see dScale.
truncateData <- function(x, control, lowQuantile = 1e-04,
                         highQuantile = 0.9999) {
    high <- quantile(control, highQuantile)
    low <- quantile(control, lowQuantile)

    x[x > high] <- high
    x[x < low] <- low
    return(x)
}

###################### truncateData
context("truncateData")
x <- c(1000, 4000, -9, 100, 101)
xTruncated <- DepecheR:::truncateData(x,
    control = c(0, 100), lowQuantile = 0,
    highQuantile = 1
)
x[3] <- 1
test_that("truncateData expected output", {
    expect_equal(c(100, 100, 0, 100, 100), xTruncated)
})

# Used internally in dScale.
# The purpose of having this deeper function is to easily allow for different
# input formats, such as vectors and dataframes.
# For information about the parameters, see dScale.
dScaleCoFunction <- function(x, control, scale, robustVarScale, truncate,
                             center, multiplicationFactor,
                             returnCenter = FALSE) {
    if (is.logical(scale) && scale == FALSE) {
        if (is.logical(truncate)) {
            responseVector <- multiplicationFactor * x
        }
        if (length(truncate) == 2) {
            xTruncReal <- truncateData(x, control,
                lowQuantile = truncate[1],
                highQuantile = truncate[2]
            )
            responseVector <- multiplicationFactor * xTruncReal
        }
    }

    if (length(scale) == 2) {
        # Define quantile
        bottom <- quantile(control, probs = scale[1], se = FALSE, na.rm = TRUE)
        top <- quantile(control, probs = scale[2], se = FALSE, na.rm = TRUE)

        if (robustVarScale == FALSE) {
            if (is.logical(truncate)) {
                responseVector <- multiplicationFactor *
                    ((x - bottom) / (top - bottom))
            }
            if (length(truncate) == 2) {
                xTruncReal <- truncateData(x, control,
                    lowQuantile = truncate[1],
                    highQuantile = truncate[2]
                )
                responseVector <- multiplicationFactor *
                    ((xTruncReal - bottom) / (top - bottom))
            }
        }

        if (robustVarScale) {
            # First truncate the data to the
            # quantiles defined by the quantiles
            xTruncated <- truncateData(x, control,
                lowQuantile = scale[1],
                highQuantile = scale[2]
            )

            sdxTruncated <- sd(xTruncated)

            # Now the data is scaled
            if (is.logical(truncate)) {
                responseVector <- multiplicationFactor * x / sdxTruncated
            }
            if (length(truncate) == 2) {
                xTruncReal <- truncateData(x, control,
                    lowQuantile = truncate[1],
                    highQuantile = truncate[2]
                )
                responseVector <- multiplicationFactor * 
                    xTruncReal / sdxTruncated
            }
        }
    }

    if (center == "mean") {
        meanValue <- mean(responseVector)
        responseVector <- responseVector - meanValue
        if (returnCenter) {
            responseList <- list(responseVector, meanValue)
        }
    }

    if (center == "peak") {
        # The peak of the data is defined
        if (length(x) < 500) {
            nBreaks <- 10
        } else {
            nBreaks <- length(x) / 50
        }
        histdata <- hist(responseVector,
            breaks = nBreaks, plot = FALSE
        )
        zeroPosition <- histdata$mids[match(
            max(histdata$counts),
            histdata$counts
        )]

        # And the position for this this peak is
        # subtracted from all points
        responseVector <- responseVector -
            zeroPosition
        if (returnCenter) {
            responseList <- list(
                responseVector,
                zeroPosition
            )
        }
    }


    if (returnCenter == FALSE) {
        return(responseVector)
    } else {
        return(responseList)
    }
}

#' @importFrom beanplot beanplot
#' @importFrom reshape2 melt
# This function is called by dViolins. It provides the core functionality,
# and is called for each cluster separately.
dViolinsCoFunction <- function(clusterNum, plotElement, clusterVector,
                               inDataFrameTrunc, clusterCol, plotDir,
                               createOutput) {
    # First, the plot elements of interest are selected
    inDataFrameRelevant <- inDataFrameTrunc[, plotElement]

    # In the case that this turns the dataframe into a vector, this is corrected
    # here for systematic reasons.
    if (is.vector(inDataFrameRelevant)) {
        inDataFrameRelevant <- as.data.frame(inDataFrameRelevant)
    }

    # Here, a splitting factor, turning all non-relevant clusters into 0 is
    # created
    clusterVector[clusterVector != clusterNum] <- 0

    # Now, the dataframe is split based on this
    inDataFrameSplit <- split(inDataFrameRelevant, f = clusterVector)

    # Now, if any of the two dataframes contain more than 5000 events, a random
    # subsample is used downstream
    for (i in seq_len(2)) {
        if (nrow(inDataFrameSplit[[i]]) > 5000) {
            inDataFrameSplit[[i]] <-
                inDataFrameSplit[[i]][sample(
                    seq_len(nrow(inDataFrameSplit[[i]])), 5000
                ), ]
        }
    }

    # And now, again, to avoid errors due to the exception with only one
    # variable, a new small transformaiton is made in that case
    if (ncol(inDataFrameRelevant) == 1) {
        for (i in seq_len(2)) {
            inDataFrameSplit[[i]] <- as.data.frame(inDataFrameSplit[[i]])
            colnames(inDataFrameSplit[[i]]) <- plotElement
        }
    }

    inDataFrameMelt <- lapply(inDataFrameSplit, melt)

    for (i in seq(1, 2)) {
        inDataFrameMelt[[i]]$variable <- paste0(
            inDataFrameMelt[[i]]$variable,
            " ", names(inDataFrameMelt[i])
        )
    }

    # And now, the data frames are merged again
    inDataFrameUnSplit <- do.call("rbind", inDataFrameMelt)

    # And so it is all plotted
    if (createOutput) {
        pdf(file.path(plotDir, paste0("Cluster_", clusterNum, "_violins.pdf")))

        par(las = 3, mar = c(5.1, 4.1, 4.1, 8.1), xpd = TRUE)
        beanplot(value ~ variable,
            data = inDataFrameUnSplit, what = c(0, 1, 0, 0),
            main = paste0(
                "Feature distribution for cluster ",
                clusterNum, " vs all others"
            ),
            ylab = "Expression level, arbitrary units", side = "both",
            border = NA, col = list("grey", clusterCol), maxwidth = 0.8
        )
        legend("topright",
            inset = c(-0.33, 0), legend = c(
                paste0("All - ", clusterNum),
                clusterNum
            ),
            fill = c("grey", clusterCol), title = "Cluster"
        )

        dev.off()
    }
}

####################### dSplsda
#context("sPLSDA")
#data("testData")
#data("testDataDepeche")
#data("testDataSNE")
#allDataRows <- c(
#    1:200, 1001:1200, 2001:2200, 3001:3200, 4001:4200,
#    10001:10200, 11001:11200, 12001:12200, 13001:13200,
#    14001:14200
#)
#xYData <- testDataSNE$Y[allDataRows, ]
#idsVector <- testData$ids[allDataRows]
#groupVector <- testData$label[allDataRows]
#clusterVector <- testDataDepeche$clusterVector[allDataRows]
#subsetVector <- sample(1:2000, size = 100)
#
#sPLSDAObject <- dSplsda(
#    xYData = xYData, idsVector = idsVector,
#    groupVector = groupVector,
#    clusterVector = clusterVector,
#    displayVector = subsetVector,
#    createOutput = FALSE
#)
#
## Alternative usage
##set.seed(1)
##sPLSDAObject <- dSplsda(
##    xYData = xYData, idsVector = idsVector,
##    groupVector = groupVector,
##    clusterVector = clusterVector, paired = TRUE,
##    createOutput = FALSE
##)
#
## Alternative usage
#sPLSDAObject <- dSplsda(
#    xYData = xYData, idsVector = idsVector,
#    groupVector = groupVector,
#    clusterVector = clusterVector,
#    testSampleRows = subsetVector, createOutput = FALSE
#)
#
####################### dSplsdaPreCalculations
#data("testData")
#data("testDataDepeche")
#data("testDataSNE")
#dSplsdaInData <- DepecheR:::dSplsdaPreCalculations(
#    clusterVector = clusterVector, idsVector = idsVector,
#    groupVector = groupVector, groupName1 = "Stim1", groupName2 = "Stim2"
#)
#

# dColorPlot
context("plotting")
data(testData)
data(testDataSNE)
allDataRows <- c(
    1:200, 1001:1200, 10001:10200,
    11001:11200
)
testDataSubset <- testData[allDataRows, ]
testDataSNESubset <- testDataSNE$Y[allDataRows, ]

dColorPlot(
    colorData = testData[1:100, 2], xYData = testDataSNE$Y[1:100, ],
    createOutput = FALSE
)

dColorPlot(
    colorData = testDataSubset$ids, xYData = testDataSNESubset,
    createOutput = FALSE
)

###################### dColorVector
testColor <- dColorVector(testDataSubset$ids, colorScale = "plasma")

##################### dContours
densContour <- DepecheR:::dContours(testDataSNESubset)

###################### dPlotCoFunction
xYData <- dScale(testDataSNESubset, testDataSNESubset,
    scale = c(0, 1),
    robustVarScale = FALSE, center = FALSE
)

DepecheR:::dPlotCoFunction(
    colorVariable = testColor, plotName = "test",
    xYData = xYData, title = FALSE,
    densContour = densContour, bandColor = "black",
    dotSize = 400 / sqrt(nrow(testDataSNESubsetFraction)),
    plotDir = ".", createOutput = FALSE
)

###################### dDensityPlot
dDensityPlot(xYData = testDataSNESubset, createOutput = FALSE)

# Alternative usage
dDensityPlot(
    xYData = testDataSNESubset, idsVector = testDataSubset$ids,
    plotDir = ".", createOutput = FALSE
)

###################### dDensityPlotCoFunction
DepecheR:::dDensityPlotCoFunction(
    xYData = xYData, color = "blue",
    plotName = "test", densContour = densContour,
    bandColor = "black", dotSize = 1.5,
    title = FALSE, plotDir = ".",
    createOutput = FALSE
)

###################### dResidualPlot
data(testDataDepeche)
dResidualPlot(
    xYData = testDataSNESubset, groupVector = testDataSubset$label,
    clusterVector = testDataDepeche$clusterVector[allDataRows],
    createOutput = FALSE
)

###################### dResidualPlot
dataTrans <-
    testDataSubset[
        , c("SYK", "CD16", "CD57", "EAT.2", "CD8", "NKG2C", "CD2", "CD56")
    ]
groupProbPlot(
    xYData = testDataSNESubset, groupVector = testDataSubset$label,
    dataTrans, createOutput = FALSE
)

#' Allocation of observations to pre-established cluster centers.
#'
#'
#' Here, observations of a dataset are allocated to a set of preestablished
#' cluster centers. This is intended to be used for the test set in train-test
#' dataset situations.
#' @importFrom moments kurtosis
#' @param inDataFrame A dataset that should be allocated to a set of cluster
#' centers, for example a richer, but less representative dataset, with all
#' datapoints from all donors, instead of only a set number of values from all.
#' @param depModel This is the result of the original application of the
#' depeche function on the associated, more representative dataset.
#' @seealso \code{\link{depeche}}
#' @return A vector with the same length as number of rows in the inDataFrame,
#' where the cluster identity of each observation is noted.
#'
#' @examples
#' # Retrieve some example data
#' data(testData)
#' \dontrun{
#' # Now arbitrarily (for the sake of the example) divide the data into a
#' # training- and a test set.
#' testDataSample <- sample(1:nrow(testData), size = 10000)
#' testDataTrain <- testData[testDataSample, ]
#' testDataTest <- testData[-testDataSample, ]
#'
#' # Run the depeche function for the train set
#'
#' depeche_train <- depeche(testDataTrain[, 2:15],
#'     maxIter = 20,
#'     sampleSize = 1000
#' )
#'
#' # Allocate the test dataset to the centers of the train dataset
#' depeche_test <- dAllocate(testDataTest[, 2:15], depeche_train
#' )
#'
#' # And finally plot the two groups to see how great the overlap was:
#' clustVecList <- list(list("Ids" =testDataTrain$ids,
#'                           "Clusters" = depeche_train$clusterVector),
#'                      list("Ids" =testDataTest$ids,
#'                           "Clusters" = depeche_test))
#' tablePerId <- do.call("rbind", lapply(seq_along(clustVecList), function(x){
#'                                       locDat <- clustVecList[[x]]
#'                                       locRes <- apply(as.matrix(table(
#'                                       locDat$Ids, locDat$Clusters)),
#'                                       1, function(y) y/sum(y))
#'                                       locResLong <- reshape2::melt(locRes)
#'                                       colnames(locResLong) <-
#'                                       c("Cluster", "Donor", "Fraction")
#'                                       locResLong$Group <- x
#'                                       locResLong
#'                                       }))
#' tablePerId$Cluster <- as.factor(tablePerId$Cluster)
#' tablePerId$Group <- as.factor(tablePerId$Group)
#'
#' library(ggplot2)
#' ggplot(data=tablePerId, aes(x=Cluster, y=Fraction,
#'         fill=Group)) + geom_boxplot() + theme_bw()
#' }
dAllocate <- function(inDataFrame, depModel) {
    if (is.matrix(inDataFrame)) {
        inDataFrame <- as.data.frame(inDataFrame)
    }
    clusterCenters <- depModel$clusterCenters
    logCenterSd <- depModel$logCenterSd
    #The very first step is that the need for log transformation is ruled out,
    #as this procedure should preferably be done with the whole dataset, before
    #even entering depeche.
    if(logCenterSd[[1]]){
        stop("dAllocate does not work well with data that has been internally",
             "log-transformed, due to large effects of small differences in ",
             "the extreme negative tail of the data.",
             "It is instead recommended to transform the ",
             "full dataset before entering depeche, which makes the log-",
             "transformation internally superflous.")
    }
    #Step one here is to scale and normalize the data in the best possible way.
    #This is preferably done with pre-defined values.
    if(is.list(logCenterSd)){
        inDataFrameScaled <- depecheLogCenterSd(inDataFrame, log2Off = TRUE,
                                                logCenterSd[[2]],
                                                logCenterSd[[3]])[[1]]
        #Annoyingly, we have to add back the empty clusters here for this all
        #to be straight forward.
        if(ncol(clusterCenters) < length(logCenterSd[[2]]) &
           length(logCenterSd[[2]]) == ncol(inDataFrame) &
           length(which(colnames(clusterCenters) %in%
                        colnames(inDataFrame))) == ncol(clusterCenters)){
            extraCols <- matrix(0, nrow = nrow(clusterCenters),
                                ncol = length(logCenterSd[[2]])-
                                    ncol(clusterCenters))
            colnames(extraCols) <- colnames(inDataFrame)[
                -which(colnames(inDataFrame) %in% colnames(clusterCenters))]
            clusterCentersFull <- cbind(clusterCenters, extraCols)
            clusterCentersUsed <- clusterCentersFull[
                ,match(colnames(inDataFrame), colnames(clusterCentersFull))]
        } else {
            stop("Mismatch between the data to be allocated and the model")
        }
        clusterCentersScaled <-
            as.matrix(depecheLogCenterSd(as.data.frame(clusterCentersUsed),
                                         log2Off = TRUE,
                                         logCenterSd[[2]],
                                         logCenterSd[[3]])[[1]])
        clusterCentersScaled[which(clusterCentersUsed == 0)] <- 0
    } else {
        inDataFrameScaled <- inDataFrame
        clusterCentersScaled <- clusterCenters
    }

    # Here, all variables that do not
    # contribute to defining a single cluster
    # are removed again.
    clusterCentersReduced <-
        clusterCentersScaled[
            which(rowSums(clusterCentersScaled) != 0),
            which(colSums(clusterCentersScaled) != 0)
        ]
    # If some variables have been excluded as
    # they did not contribute to construction
    # of any cluster, they are removed from
    # the inData here. The special case with only one variable is taken
    # into account.
    # There are two different methods here: one for external and one for
    # internal use. In the first case, there are no colnunm names, but the
    # properties of the cluster centers are also more raw and thus informative.
    if (length(colnames(clusterCentersReduced)) > 0) {
        inDataFrameReduced <-
            inDataFrameScaled[, colnames(clusterCentersReduced)]
    } else {
        inDataFrameReduced <-
            inDataFrameScaled[, which(colSums(clusterCentersScaled) != 0)]
    }

    # Here, a specific case, namely that only one variable contains
    # meaningful information, is taken into account.
    if (is.vector(inDataFrameReduced)) {
        clusterCentersReduced <- as.matrix(clusterCentersReduced)
        inDataFrameReduced <- as.matrix(inDataFrameReduced)
    }

    clusterReallocationResult <- allocate_points(
        as.matrix(inDataFrameReduced),
        clusterCentersReduced, 1
    )[[1]]

    # As allocate_points spontaneously likes to throw out a cluster called 0,
    # this behaviour is controlled here
    clusterReallocationResult <- clusterReallocationResult + 1

    return(clusterReallocationResult)
}

# dAllocate

context("dAllocate")
cols <- 150
set.seed(191)
x <- DepecheR:::generateBimodalData(observations = 20, dataCols = cols)
centers <- rbind(runif(cols) + 50, runif(cols) - 50)
out <- DepecheR:::dAllocate(x$samples, list("clusterCenters" =centers,
                                 "logCenterSd" = FALSE))
test_that("dAllocate expected output", {
    expect_true(all(out == x$ids))
})

#' Euclidean neighbor smoothing
#'
#'
#' This function constructs a variable that for each event shows the average
#' value for its euclidean k-nearest neighbors. It builds on the same
#' idea as has been put forward in the Sconify package:
#' -Burns TJ (2019). Sconify: A toolkit for performing KNN-based statistics for
#' flow and mass cytometry data. R package version 1.4.0 and
#' -Hart GT, Tran TM, Theorell J, Schlums H, Arora G, Rajagopalan S, et al.
#' Adaptive NK cells in people exposed to Plasmodium falciparum correlate
#' with protection from malaria. J Exp Med. 2019 Jun 3;216(6):1280–90.
#' First, the k nearest neighbors are defined for cell x. Then, the average
#' value for the k nearest neighbors is returned as the result for cell x.
#' @param focusData The data that should be smoothed. Should be a matrix with
#' the variables to be smoothed as columns.
#' @param euclidSpaceData The data cloud in which the nearest neighbors for the
#' events should be identified. Can be a vector, matrix or dataframe. It is
#' worth noting that if this data has more than 10 dimensions, the first step
#' of the algorithm will be the creation of a 10-dimensional PCA using
#' fast.prcomp from gmodels. So in cases where this function is used iteratively,
#' it might be wiser to run the PCA beforehand.
#' @param neighRows The rows in the dataset that correspond to the neighbors
#' of the focusData points. "default" is all the focusData points, but a subset
#' can be added instead, if preferred. This is good to use to increase
#' robustness, e.g. by running 100 iterations with different sets of neighbors
#' with the same number of points from each group/individual.
#' @param ctrlRows Optionally, a set of control rows that are used to remove
#' background signal from the neighRows data before sending the data back.
#' @param kNeighK The number of nearest neighbors. "default" is the max of
#' 100 and the number of neighbor rows divided by 10000. Mutliple different
#' values here is preferred.
#' @param kMeansK The number of clusters in the initial step of the algorithm.
#' A higher number leads to shorter runtime, but potentially lower accuracy.
#' This is not used if kMeansCenters is provided. "default" is the highest of 1
#' and the number of cells in euclidSpaceData divided by 1000.
#' @param kMeansCenters Here, a pre-clustering of the data can be provided, in
#' which case the clustering will not be performed internally. Wise if for
#' example a bootstrapping scheme is used to define the neighRows iteratively,
#' as the k-means step can be quite time consuming. This part is the cluster
#' centers or centroids.
#' @param kMeansClusters See above. Here, the clusters, instead of the centroids
#' are provided if used.
#' @param method The method to use for the smoothing. Three values possible:
#' mean (default), median and mode.
#' @param nCores The number of cores used. Defaults to number of cores in the
#' computer minus 1.
#' @return An object of the same dimensions as focusData that has been smoothed.
#' @importFrom ClusterR KMeans_rcpp
#' @importFrom parallel detectCores makeCluster stopCluster
#' @importFrom doSNOW registerDoSNOW
#' @importFrom foreach foreach %dopar%
#' @importFrom gmodels fast.prcomp
#' @importFrom FNN knnx.index
#' @examples
#' data(testData)
#' data(testDataSNE)
#' euclidSpaceData <-
#'     testData[, c(
#'         "SYK", "CD16", "CD57", "EAT.2",
#'         "CD8", "NKG2C", "CD2", "CD56"
#'     )]
#' \dontrun{
#' smoothGroupVector <- neighSmooth(
#'     focusData = as.numeric(testData$label),
#'     euclidSpaceData
#' )
#' }
#' @export neighSmooth
neighSmooth <- function(focusData, euclidSpaceData,
                        neighRows = "default",
                        ctrlRows = NULL,
                        kNeighK = "default",
                        kMeansK = "default",
                        kMeansCenters = NULL,
                        kMeansClusters = NULL,
                        method = "mean",
                        nCores = detectCores() - 1) {
    if (is.vector(focusData)) {
        focusData <- data.frame(focusData, 0)
        focusDataIsVector <- TRUE
    } else {
        focusDataIsVector <- FALSE
    }
    if(is.matrix(focusData)){
        focusData <- as.data.frame(focusData)
    }

    if (is.vector(euclidSpaceData)) {
        euclidSpaceData <- data.frame(euclidSpaceData, euclidSpaceData)
    } else if (is.matrix(euclidSpaceData)) {
        euclidSpaceData <- as.data.frame(euclidSpaceData)
    }
    # First, the dimensionality is reduced to 10 dimensions for the
    # euclidSpaceData
    if (ncol(euclidSpaceData) > 10) {
        dataRedDim <- fast.prcomp(euclidSpaceData)$x[, seq(1, 10)]
    } else {
        dataRedDim <- euclidSpaceData
    }
    if(is.character(neighRows) && neighRows == "default"){
        neighRows <- seq_len(nrow(as.matrix(focusData)))
        message("All events will be used as potential neighbors")
    }
    if(is.character(kNeighK) && kNeighK == "default"){
        kNeighK <- max(100,round(nrow(as.matrix(focusData))/10000))
        message("The number of neighbors will be ", kNeighK)
    }

    if(is.character(kMeansK) && kMeansK == "default" &&
       is.null(kMeansCenters)){
        kMeansK <- max(1, round(nrow(
            as.matrix(euclidSpaceData)
        ) / 1000))
        message("The number of k-means centers will be ", kMeansK)
    }

    # Now, the cells are clustered according to this analysis
    if(all(is.null(kMeansCenters) |
       is.null(kMeansClusters))){
        kMeansResult <- KMeans_rcpp(dataRedDim, kMeansK, max_iters = 100)
        kMeansCenters <- kMeansResult$centroids
        kMeansClusters <- kMeansResult$clusters
        message("Done with k-means")
    } else if(any(is.null(kMeansCenters) |
                  is.null(kMeansClusters))){
        message("Illogical input. Please provide both kMeansCenters ",
                "and kMeansClusters or else they will not be used")
    }

    # Here, the rows connected to the neighbors, the control neighbors or
    # neither are defined
    groupVec <- rep("none", nrow(dataRedDim))
    groupVec[neighRows] <- "neigh"
    if (missing(ctrlRows) == FALSE) {
        groupVec[ctrlRows] <- "ctrl"
    }

    rowNumbers <- seq_len(nrow(dataRedDim))

    focusDataClustList <- split(focusData, kMeansClusters)
    dataRedDimClustList <- split(as.data.frame(dataRedDim), kMeansClusters)
    groupVecClustList <- split(groupVec, kMeansClusters)
    rowNumList <- split(rowNumbers, kMeansClusters)

    if(nrow(kMeansCenters) <= 11){
        distCenters <- as.list(as.data.frame(
            t(knnx.index(kMeansCenters, kMeansCenters, nrow(kMeansCenters)))
        ))
    } else if (nrow(kMeansCenters) > 11 &
               nrow(kMeansCenters) < 100) {
        distCenters <- as.list(as.data.frame(
            t(knnx.index(kMeansCenters, kMeansCenters, 11))
        ))
    } else {
        #Here, the situations where the data contains very much data is
        #considered. Here, we consider neighbors in the ten percent
        #of the clusters lying closest to the cluster in question
        numClusts <- round(nrow(kMeansCenters)/10)
        distCenters <- as.list(as.data.frame(
            t(knnx.index(kMeansCenters, kMeansCenters, numClusts))
        ))
    }

    print("Now the first bit is done, and the iterative part takes off")

    cl <- parallel::makeCluster(nCores, type = "SOCK")
    registerDoSNOW(cl)

    allClusters <- names(focusDataClustList)
    firstCluster <- 1
    resultList <- list()
    x <- 1
    while (firstCluster <= length(allClusters)) {
        timeBefore <- Sys.time()
        if (((firstCluster + nCores) - 1) < length(allClusters)) {
            clusterRange <- firstCluster:((firstCluster + nCores) - 1)
        } else {
            clusterRange <- firstCluster:length(allClusters)
        }

        # Now, the datasets are constructed from this cluster range
        locDataRedDimClustList <- dataRedDimClustList[clusterRange]

        # Here, the neighbors and the control neighbors are found in the 11
        # closest clusters for each of the focus clusters.
        locDistCenters <- distCenters[clusterRange]

        neighCtrlNeighReturnList <- lapply(locDistCenters, function(y) {
            locNeighList <- lapply(y, function(z) {
                locNeigh <- dataRedDimClustList[[z]]
                locReturn <- focusDataClustList[[z]]
                locGroup <- groupVecClustList[[z]]
                list(
                    locNeigh[which(locGroup == "neigh"), ],
                    locNeigh[which(locGroup == "ctrl"), ],
                    locReturn[which(locGroup == "neigh"), ],
                    locReturn[which(locGroup == "ctrl"), ]
                )
            })
            locNeighNeigh <- as.matrix(do.call(
                "rbind",
                lapply(locNeighList, "[[", 1)
            ))
            locNeighReturn <- as.matrix(do.call(
                "rbind",
                lapply(locNeighList, "[[", 3)
            ))
            locCtrlNeigh <- as.matrix(do.call(
                "rbind",
                lapply(locNeighList, "[[", 2)
            ))
            locCtrlReturn <- as.matrix(do.call(
                "rbind",
                lapply(locNeighList, "[[", 4)
            ))
            list(locNeighNeigh, locNeighReturn, locCtrlNeigh, locCtrlReturn)
        })

        resultNeigh <- foreach(
            i = seq_along(locDataRedDimClustList),
            .packages = "DepecheR"
        ) %dopar%
            microClust(
                dataCenter = as.matrix(locDataRedDimClustList[[i]]),
                dataNeigh = neighCtrlNeighReturnList[[i]][[1]],
                dataReturn = neighCtrlNeighReturnList[[i]][[2]],
                method = method, k = kNeighK
            )
        if (missing(ctrlRows) == FALSE) {
            resultCtrl <- foreach(
                i = seq_along(locDataRedDimClustList),
                .packages = "DepecheR"
            ) %dopar%
                microClust(
                    dataCenter = as.matrix(locDataRedDimClustList[[i]]),
                    dataNeigh = neighCtrlNeighReturnList[[i]][[3]],
                    dataReturn = neighCtrlNeighReturnList[[i]][[4]],
                    method = method, k = kNeighK
                )

            neighCtrlDifference <- lapply(seq_along(resultNeigh), function(y) {
                resultNeigh[[y]] - resultCtrl[[y]]
            })
        } else {
            neighCtrlDifference <- resultNeigh
        }

        resultList[[x]] <- do.call("rbind", neighCtrlDifference)

        timeAfter <- as.numeric(Sys.time() - timeBefore)
        print(paste0(
            "Clusters ", clusterRange[1], " to ",
            clusterRange[length(clusterRange)],
            " smoothed in ", timeAfter, " ",
            attributes(timeAfter)$units, ". Now, ",
            length(allClusters) - clusterRange[length(clusterRange)],
            " clusters are left."
        ))
        firstCluster <- (clusterRange[length(clusterRange)] + 1)
        x <- x + 1
    }

    parallel::stopCluster(cl)
    fullResult <- do.call("rbind", resultList)
    fullResultOrdered <- fullResult[order(unlist(rowNumList)), ]
    if (focusDataIsVector) {
        fullResultOrdered <- fullResultOrdered[, 1]
    }
    return(fullResultOrdered)
}

context("neighSmooth")
focDat <- c(rep(0, 75), rep(1, 25), rep(0, 50), rep(1, 50))
euSpaceDat <- c(rep(0, 100), rep(1, 100))
set.seed(1111)
resDat <- neighSmooth(focusData = focDat, euclidSpaceData = euSpaceDat)
test_that("neighSmooth expected output", {
    expect_true(all(resDat[seq(1, 100)] == 0.25))
})

#' A depeche clustering of the testData set
#'
#' This is a depeche clustering of the testData dataset. It has
#' been generated with the Rtsne.multicore package
#'
#' produced by J. Theorell.
#'
#'
#' @docType data
#'
#' @usage data(testDataDepeche)
#'
#' @format An object of class \code{'list'};
#'
#' @keywords datasets
#'
"testDataDepeche"

# This function is used by depecheCoFunction.
# It is only used for datasets with less than 10 000 total observations.
# "Unique" parameters
# inDataFrameUsed: the scaled version of inDataFrame. See initial part of
# depeche function for details.
# For information on the other parameters, see depeche.
#' @importFrom parallel detectCores makeCluster stopCluster
#' @importFrom doSNOW registerDoSNOW
#' @importFrom foreach foreach %dopar%
#' @importFrom moments kurtosis
#' @importFrom foreach foreach %dopar%
#' @importFrom gplots heatmap.2
#' @importFrom dplyr sample_n
depecheAllData <- function(inDataFrameUsed, penalty, k, nCores) {
    penaltyForRightSize <- penalty * ((nrow(inDataFrameUsed) *
        sqrt(ncol(inDataFrameUsed))) / 1450)

    dataMat <- data.matrix(inDataFrameUsed)

    if (nCores == "default") {
        nCores <- floor(detectCores() * 0.875)
        if (nCores > 10) {
            nCores <- 10
        }
    }

    i <- 1
    cl <- makeCluster(nCores, type = "SOCK")
    registerDoSNOW(cl)
    return_all <- foreach(i = seq_len(21), .packages = "DepecheR") %dopar%
        sparse_k_means(dataMat, round(k * 3), penaltyForRightSize, 1, i)
    stopCluster(cl)

    # Here, the best iteration is retrieved,
    # if this is not one with only 1 cluster

    logMaxLik <- as.vector(do.call("rbind", lapply(return_all, "[[", 5)))
    nClust <- vapply(return_all,
        FUN.VALUE = 1,
        function(x) sum(rowSums(x[[3]] != 0) != 0)
    )
    logMaxLikNotOne <- logMaxLik[which(nClust > 1)]
    maxN <- max(logMaxLikNotOne)
    returnLowest <- return_all[[which(logMaxLik == maxN)[1]]]

    # And here, the optimal results are retrieved
    clusterVector <- returnLowest$i
    clusterCenters <- returnLowest$c

    # And here, the optimal results are made
    # more dense by removing empty rows and
    # columns, etc.

    colnames(clusterCenters) <- colnames(inDataFrameUsed)

    # Remove all rows and columns that do not
    # contain any information
    reducedClusterCenters <- clusterCenters[
        which(rowSums(clusterCenters) != 0),
        which(colSums(clusterCenters) != 0)
    ]

    # In the specific case that only one row
    # is left, due to a high penalty, the
    # data needs to be converted back to a
    # matrix from a vector. The same is done
    # if the number of informative variables
    # is just one.
    if (length(which(rowSums(clusterCenters) != 0)) == 1) {
        reducedClusterCenters <- t(reducedClusterCenters)
    } else if (length(which(colSums(clusterCenters) != 0) == 1)) {
        reducedClusterCenters <- as.matrix(reducedClusterCenters)
    }

    # Here, the results are combined
    dClustResult <- list(clusterVector, reducedClusterCenters)
    names(dClustResult) <- c("clusterVector", "clusterCenters")
    return(dClustResult)
}

# This function is used by depecheCoFunction, and thus indirectly by depeche.
# Here, the optimal penalty is defined.
# "Unique" parameters
# inDataFrameUsed: the scaled version of inDataFrame. See initial part of
# depeche function for details.
# disableWarnings: Should warnings be disabled?
# For information on the other parameters, see depeche.
#' @importFrom parallel detectCores makeCluster stopCluster
#' @importFrom doSNOW registerDoSNOW
#' @importFrom foreach foreach %dopar%
#' @importFrom Rcpp evalCpp
#' @importFrom graphics box

depechePenaltyOpt <- function(inDataFrameUsed, k, maxIter, minARIImprovement,
                        sampleSize, penalties, createOutput,
                        disableWarnings = FALSE, optimARI, nCores,
                        plotDir) {

    # The constant 1450 was empirically
    # identified by running a large number of
    # penalty values for a few test datasets.
    penaltyConstant <- ((sampleSize * sqrt(ncol(inDataFrameUsed))) / 1450)
    realPenalties <- penalties * penaltyConstant
    roundPenalties <- round(penalties, digits = 1)

    dataMat <- data.matrix(inDataFrameUsed, rownames.force = NA)

    # This command is reiterated the number
    # of times that is needed to reach a
    # minimal improvement of the distance.
    iter <- 1
    lastStd <- 1
    stdChange <- 1
    # distanceBetweenMinAndBestPrevious=-1
    iterTimesNCores <- 1
    allClusterCentersPenaltySorted <- list()
    cl <- makeCluster(nCores, type = "SOCK")
    registerDoSNOW(cl)

    usedPenalties <- realPenalties
    usedRoundPenalties <- roundPenalties
    allPositions <- seq_along(realPenalties)
    usedPositions <- allPositions

    # That the while loop should never terminate before 20 iterations is
    # arbitrary. 
    while ((iterTimesNCores < 20) || (iterTimesNCores < maxIter && 
           stdChange >= minARIImprovement)) {
        ptm <- proc.time()
        optimList <- foreach(
            i = seq_len(nCores),
            .packages = "DepecheR"
        ) %dopar%
            grid_search(dataMat, k, usedPenalties, 1, sampleSize, i)[
                c(2,4,5)] # This exclusion is due to a previous version keeping
        # a cluster in origo, which has since been deprecated.
        
        # Here, we name the output, and further remove half of the data which
        # is solely there due to the old origo solution thing. We also turn the
        # matrices into vectors. 
        optimList <- lapply(optimList, function(x){
            names(x) <- c("ARI", "Clusters", "ClustCenters")
            x[c(1,2)] <- lapply(x[c(1,2)], as.vector)
            x <- lapply(x, 'names<-', usedRoundPenalties)
            x[[3]] <- lapply(x[[3]], function(y){
                y[c(1,2)]
            })
            x
        })

        # Before any further analyses are
        # performed, any penalty that can result
        # in a trivial solution are practically
        # eliminated.
        optimListNonTrivial <- lapply(optimList, function(x){
            x$ARI[which(x$Clusters == 1)] <- 0
            x
        })
        
        # Now, the new list is combined with the older, if there are any
        if (iter == 1) {
            optimListFull <- optimListNonTrivial
        } else {
            optimListFull <- c(optimListFull, optimListNonTrivial)
        }

        # Here, the average and standard deviation of the ARI for each penalty 
        # is retrieved
        ARIList <- lapply(optimListFull, "[[", 1)
        
        ARIVecList <- lapply(as.character(roundPenalties), function(x){
            unlist(lapply(ARIList, function(y){
                if(any(names(y) %in% x)){
                    return(y[which(names(y) == x)])
                }
            }))
        })
        
        meanARIVec <- unlist(lapply(ARIVecList, mean))
        stdOptVec <- unlist(lapply(ARIVecList, sd))
        
        # Return the position of the maximum value
        maxPos <- which.max(meanARIVec)[1]
        
        # Add the standard deviation of this
        # position to its mean
        meanMinus2StdMax <- meanARIVec[maxPos] -
            (2 * stdOptVec[maxPos])
        
        # Now add the standard deviation of each
        # of the non-minimal values to the mean
        
        meanPlus2StdAll <- meanARIVec +
            (2 * stdOptVec)
        
        # And now, the interesting positions are
        # defined. These are the ones that
        # either overlap with uncertainity with
        # the optimal solution, or that are very
        # similar to it. In the first iterations,
        # the critera are more inclusive, to avoid
        # discarding solutions due to coincidence.
        localIter <- ifelse(iter < 3, iter, 3)
        k1 <- switch(EXPR = localIter, 2, 1, 0)
        k2 <- switch(EXPR = localIter, 4, 3, 2)
        usedPositions <-
            which(realPenalties %in% usedPenalties &
                      (meanPlus2StdAll >=
                           (meanMinus2StdMax -
                                (k1 * stdOptVec[maxPos])) |
                           meanARIVec >= max(meanARIVec) -
                           ((1 - optimARI) * k2)))
        
        # Here, all penalties and solutions that
        # do not overlap with the optimal
        # solution are excluded from further
        # optimiations, to reduce calculation
        # time.
        usedPenalties <- realPenalties[usedPositions]
        usedRoundPenalties <- roundPenalties[usedPositions]

        # Finally, another criterion on the gain
        # of adding more iterations is included. Here, we also kill the
        #optimization if we are running only one penalty. 
        if (length(usedPenalties) > 1) {
            if(lastStd != 0){
                stdChange <- ((lastStd - stdOptVec[maxPos])/lastStd)/
                    sqrt(iter * nCores)
            } else {
                stdChange <- 0
            }
        } else {
            stdChange <- 0
        }
        
        lastStd <- stdOptVec[maxPos]
        
        iterTimesNCores <- iter * nCores


        fullTime <- proc.time() - ptm
        message(
            "Set ", iter, " with ", nCores, " iterations completed in ",
            round(fullTime[3]), " seconds."
        )
        iter <- iter + 1
    }

    stopCluster(cl)

    message("The optimization was iterated ", (iter - 1) * nCores, " times.")

    if (iter * nCores >= maxIter && stdChange >
        minARIImprovement) {
        warning("The maximum number of iterations was reached before stable 
                optimal solution was found")
    }


    # Here, the optimal penalty is selected.
    # This is defined as the lowest penalty
    # that yields an ARI that is not lower
    # than 0.01 less than the best ARI.
    optimalPenalties <-
        roundPenalties[which(meanARIVec >= max(meanARIVec) - (1 - optimARI))]
    
    # The best penalty is defined as the
    # median penalty or the optimal penalty
    # with the even number among the two most
    # centrally placed, in the case of an
    # even number of optimal solutions.
    bestPenalty <- optimalPenalties[round(mean(c(seq_along(optimalPenalties))))]
    lowestPenalty <- roundPenalties[1]
    highestPenalty <- roundPenalties[length(roundPenalties)]

    if (length(penalties) > 1) {
        if (bestPenalty == lowestPenalty) {
            warning("The lowest penalty was the most optimal in the range. ",
                    "This might either be due a suboptimal set of penalties,",
                    " or to the use of a too small sample size")
        }
        if (bestPenalty == highestPenalty) {
            warning("The highest penalty was the most optimal in the range. ",
                    "This might either be due a suboptimal set of penalties,",
                    " or to the use of a too small sample size")
        }
    }

    # Here, the optimization is plotted if wanted.
    
    if (createOutput) {
        pdf(file.path(plotDir, "ARI_as_a_function_of_penalties.pdf"))
        par(mar = c(5, 4, 4, 6) + 0.1)
        # Plot the data
        plot(log10(roundPenalties), meanARIVec,
            pch = 16, axes = FALSE,
            ylim = c(0, 1), xlab = "", ylab = "", type = "b", col = "black",
            main = "Difference between samples as a function of penalties"
        )
        axis(2, ylim = c(0, 1), col = "black", las = 1)
        mtext("Adjusted rand index (ARI) between data resamplings",
            side = 2,
            line = 2.5
        )
        graphics::box()

        # Draw the penalty axis
        axis(1, pretty(range(log10(roundPenalties)), n = 10))
        mtext("Log10 of penalty values", side = 1, col = "black", line = 2.5)

        # Add Legend
        legend("topleft",
            legend = "ARI (high is good)", text.col = "black",
            pch = c(16, 15), col = "black"
        )

        dev.off()
    }

    # Now, we create the final output
    #The average number of clusters: 
    nClustList <- lapply(optimListFull, "[[", 2)
    meanClustVec <- unlist(lapply(as.character(roundPenalties), function(x){
        mean(unlist(lapply(nClustList, function(y){
            if(any(names(y) %in% x)){
                return(y[which(names(y) == x)])
            }
        })), na.rm = TRUE)
    }))
    
    meanOptimDf <- data.frame("ARI" = meanARIVec, "nClust" = meanClustVec)
    row.names(meanOptimDf) <- roundPenalties
    # Here, the cluster center information
    # for each run is saved.
    allClustCentRaw <- lapply(optimListFull, "[[", 3)

    bestClusterCenters <- unlist(lapply(allClustCentRaw, "[[", 
                                 as.character(bestPenalty)), recursive = FALSE)

    penaltyOptList <- list("bestPenalty" = bestPenalty, 
                           meanOptimDf, bestClusterCenters)


    return(penaltyOptList)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

sparse_k_means <- function(X, k, reg, no_zero, seed_off_set = 0L) {
    .Call('_DepecheR_sparse_k_means', PACKAGE = 'DepecheR', X, k, reg, no_zero, seed_off_set)
}

grid_search <- function(X, k, reg, iterations, bootstrapSamples, seed_off_set = 0L) {
    .Call('_DepecheR_grid_search', PACKAGE = 'DepecheR', X, k, reg, iterations, bootstrapSamples, seed_off_set)
}

allocate_points <- function(X, mu, no_zero) {
    .Call('_DepecheR_allocate_points', PACKAGE = 'DepecheR', X, mu, no_zero)
}

rand_index <- function(inds1, inds2, k) {
    .Call('_DepecheR_rand_index', PACKAGE = 'DepecheR', inds1, inds2, k)
}

#' SNE of the testData set
#'
#' This is a t-distributed stochastic neighbor embedding of the testData
#' dataset. It has been generated with the Rtsne.multicore package.
#'
#'
#' @docType data
#'
#' @usage data(testDataSNE)
#'
#' @format An object of class \code{'list'};
#'
#' @keywords datasets
#' #'
#' @references Jesse H. Krijthe (2015). Rtsne: T-Distributed Stochastic Neighbor
#' Embedding using a Barnes-Hut Implementation
#' (\href{https://github.com/RGLab/Rtsne.multicore/issues/5}{GitHub})
#'
"testDataSNE"

######################
context("scaling")
# dScale
set.seed(19191)
x <- DepecheR:::generateBimodalData(observations = 100)
y_df <- DepecheR:::dScale(x[[1]])

###################### dScaleCoFunction
x <- DepecheR:::generateBimodalData()
result <- DepecheR:::dScaleCoFunction(x[[1]][, 2],
    control = x[[1]][, 2],
    scale = c(0.001, 0.999),
    robustVarScale = TRUE,
    truncate = c(0.001, 0.999),
    center = "mean", multiplicationFactor = 1
)

#' Define and plot group probabilities
#'
#'
#' This function defines and plots the single-observation probability for
#' belonging to either of two groups. It uses the \code{\link{neighSmooth}}
#' function with the special case that the values are binary: For each set of
#' k nearest neighbors, cell x is assigned a probability to belong to one group
#' or the other based on the percentage of the neighbors belonging to each
#' group. In other words, if 20 out of 100 neighbors belong to group
#' A and 80 belong to group B, and the value for the cell will be 20% for group
#' A or 80% for group B, depending on the point of view. This will also be
#' accordingly reflected in the color scale on the resulting plot.
#'
#' @param xYData A dataframe or matrix with two columns. Each row contains
#' information about the x and y positition in the field for that observation.
#' @param groupVector Vector with the same length as xYData containing
#' information about the group identity of each observation.
#' @param euclidSpaceData The data cloud in which the nearest neighbors for the
#' events should be identified.
#' @param kNeighK The number of nearest neighbors.
#' @param kMeansK The number of clusters in the initial step of the algorithm.
#' A higher number leads to shorter runtime, but potentially lower accuracy.
#' @param plotName The main name for the graph and the analysis.
#' @param densContour If density contours should be created for the plot(s) or
#' not. Defaults to TRUE. a
#' @param groupName1 The name for the first group
#' @param groupName2 The name for the second group
#' @param title If there should be a title displayed on the plotting field. As
#' the plotting field is saved as a png, this title cannot be removed as an
#' object afterwards, as it is saved as coloured pixels. To simplify usage for
#' publication, the default is FALSE, as the files are still named, eventhough
#' no title appears on the plot.
#' @param plotDir If different from the current directory. If specified and
#' non-existent, the function creates it. If "." is specified, the plots will be
#' saved at the current directory.
#' @param bandColor The color of the contour bands. Defaults to black.
#' @param dotSize Simply the size of the dots. The default makes the dots
#' smaller the more observations that are included.
#' @param returnProb Should a probability vector be returned? Mutually exclusive
#' with returnProbColVec.
#' @param returnProbColVec Should the color vector be returned as part of the
#' output? Mutually exclusive with returnProb.
#' @param createOutput For testing purposes. Defaults to TRUE. If FALSE, no
#' output is generated.
#' @importFrom parallel detectCores makeCluster stopCluster
#' @importFrom doSNOW registerDoSNOW
#' @importFrom foreach foreach %dopar%
#' @importFrom FNN knnx.index
#' @importFrom stats kmeans
#' @return A graph showing the probability as a color scale from blue over white
#' to red for each event to belong to one group or the other, with a separate
#' color scale. Optionally also the color vector, if returnProbColVec is TRUE.
#' @examples
#' data(testData)
#' data(testDataSNE)
#' euclidSpaceData <-
#'     testData[, c(
#'         "SYK", "CD16", "CD57", "EAT.2",
#'         "CD8", "NKG2C", "CD2", "CD56"
#'     )]
#' \dontrun{
#' groupProbPlot(
#'     xYData = testDataSNE$Y, groupVector = testData$label,
#'     euclidSpaceData
#' )
#' }
#'
#' @export groupProbPlot

groupProbPlot <- function(xYData, groupVector, euclidSpaceData,
                          kNeighK = max(
                              100, round(nrow(euclidSpaceData) / 10000)
                          ),
                          kMeansK = round(nrow(euclidSpaceData) / 1000),
                          densContour = TRUE,
                          groupName1 = unique(groupVector)[1],
                          groupName2 = unique(groupVector)[2],
                          plotName = "default", title = FALSE,
                          bandColor = "black", plotDir = ".",
                          dotSize = 400 / sqrt(nrow(xYData)),
                          returnProb = FALSE,
                          returnProbColVec = FALSE, createOutput = TRUE) {
    if (plotDir != ".") {
        dir.create(plotDir)
    }

    if (length(unique(groupVector)) != 2) {
        stop("More or less than two groups are present. Please correct this.")
    }

    if (is.matrix(xYData)) {
        xYData <- as.data.frame(xYData)
    }

    if (plotName == "default") {
        plotName <- paste0(groupName1, "_vs_", groupName2)
    }

    rowNumbers <- seq_along(groupVector)
    rowsGroup1 <- rowNumbers[which(groupVector == unique(groupVector)[1])]
    rowsGroup2 <- rowNumbers[which(groupVector == unique(groupVector)[2])]
    groupValVec <- rep(100, length.out = nrow(euclidSpaceData))
    groupValVec[rowsGroup2] <- -100

    # Here, a new, balanced group neighbor vector is introduced
    if (length(rowsGroup1) != length(rowsGroup2)) {
        groupLengths <- c(length(rowsGroup1), length(rowsGroup2))
        if (min(groupLengths) < 10000) {
            if (min(groupLengths) < 5000) {
                stop(paste0(
                    "At least one of the groups has less than 5000",
                    " events, which will lead to a too low resolution ",
                    "for this analysis to be meaningful. Use ",
                    "dResidualPlot instead."
                ))
            }
            message(paste0(
                "The groups are not matched in size and the ",
                "smaller group has only ", min(groupLengths),
                " rows, leading to a reduction in neighbor",
                " resolution to this level for both groups"
            ))
        }
        rowsGroup1 <- rowsGroup1[sample(
            seq(1, length(rowsGroup1)),
            min(groupLengths)
        )]
        rowsGroup2 <- rowsGroup2[sample(
            seq(1, length(rowsGroup2)),
            min(groupLengths)
        )]
    }

    # And here, we use the neighbor smoothing algorithm to identify the
    # probabilities
    fullResult <- neighSmooth(
        focusData = groupValVec,
        euclidSpaceData = euclidSpaceData,
        neighRows = c(rowsGroup1, rowsGroup2),
        kNeighK = kNeighK, kMeansK = kMeansK
    )

    #Now, we make a vital change to the scale: the values never go below
    #50%, as we are projecting two scale on each other, thus showing that
    #a value of -49 also means +51 and vice versa. Therefore the values
    #need to be changed.
    fullTempResult <- fullResult+(50-(50*fullResult)/100)
    fullResult <- vapply(fullTempResult, function(x) if(x<50){x-100}else{x}, 100)

    # Here the data that will be used for
    # plotting is scaled.
    colnames(xYData) <- c("V1", "V2")

    # Make a color vector with the same
    # length as the data
    residual.df <- as.data.frame(fullResult)

    # make a breaks vector to define each bin
    # for the colors
    brks <- c(-100, -90, -80, -70, -60, 0, 60, 70, 80, 90, 100)

    # assign each value to a bin
    grps <- with(residual.df, cut(residual.df[, 1],
        breaks = brks,
        include.lowest = TRUE
    ))
    colors <- colorRampPalette(c("#FF0000", "white", "#0000FF"))(10)
    xYData$col <- colors[grps]

    dPlotCoFunction(
        colorVariable = xYData$col, plotName =
            paste0(plotName, "_probabilities"),
        xYData = xYData, title = title,
        densContour = densContour, bandColor = bandColor,
        dotSize = dotSize, plotDir = plotDir,
        createOutput = createOutput
    )

    # Create a color legend with text

    yname <- "Probability for group identity, percent"
    topText <- paste0(groupName1, " more probable")
    bottomText <- paste0(groupName2, " more probable")
    z <- matrix(seq_len(10), nrow = 1)
    x <- 1
    y <- seq(-100, 100, len = 10)

    if (createOutput) {
        pdf(file.path(plotDir, paste0(plotName, "_probability_scale.pdf")))
        par(fig = c(0.35, 0.65, 0, 1), xpd = NA)
        image(x, y, z, col = colors, axes = FALSE, xlab = "", ylab = yname)
        text(0.4, -100, labels = 100)
        text(0.4, -50, labels = 75)
        text(x = 0.4, y = 0, labels = 50)
        text(0.4, 50, labels = 75)
        text(0.4, 100, labels = 100)
        text(1, 120, labels = topText, cex = 1.1)
        text(1, -120, labels = bottomText, cex = 1.1)
        box()
        dev.off()
    }
    if (returnProb) {
        return(fullResult)
    } else if (returnProbColVec) {
        return(xYData$col)
    }
}

# Function that generates synthetic data from a bimodal distribution.
# centers: A list with two items, each item a vector specifying the center of a
# data cluster.
# prop: A scalar determining relative number of data points in the two clusters
# dataCols: A scalar determining the number of dimensions. Superfluous if
# "centers" is provided.
# observations: A scalar determining the number of sampled data points.
generateBimodalData <- function(centers, prop = 0.3, dataCols = 5,
                                observations = 10000) {
    if (missing(centers)) {
        centers <- rbind(runif(dataCols) + 50, runif(dataCols) - 50)
    } else {
        dataCols <- ncol(centers)
    }
    stopifnot(nrow(centers) == 2)

    ids <- c(
        rep(1, floor(prop * observations)),
        rep(2, observations - floor(prop * observations))
    )
    rands <- matrix(
        rnorm(observations * dataCols, mean = 0, sd = 1),
        observations, dataCols
    )
    pop1 <- rands[seq_len(floor(prop * observations)), ] +
        rep(centers[1, ], each = floor(prop * observations))

    pop2 <- tail(rands, n = observations - floor(prop * observations)) +
        rep(centers[2, ], each = observations - floor(prop * observations))

    samples <- rbind(pop1, pop2)
    result <- list(samples, ids)
    names(result) <- c("samples", "ids")

    return(result)
}

# Function that generates synthetic data, divided over a number of cluster with
# increasing number of zero-dimensions (starting from 0). The sampled points are
# generated equally between the clusters.

# modeN: A scalar determining the number of clusters.
# dataCols: A scalar determing the number of dimensions. DataCols must be
# greater than modeN, to avoid clusters with only zeros.
# obsrevations: A scalar determining the number of sampled data points.

generateSparseData <- function(modeN = 5, dataCols = 100,
                               observations = 10000) {
    # Check if input ok
    if ((observations / modeN) %% 1 != 0) {
        stop("Observations has to be divisible by modeN")
    }
    obsPerMode <- observations / modeN

    # generate the centers numbers

    randInts <- sample(c(-50, 50), dataCols * modeN, replace = TRUE)
    centers <- matrix(randInts, nrow = modeN, byrow = TRUE)
    # put in sparsity
    for (i in seq_len(modeN)) {
        inds <- sample(seq_len(dataCols), i)
        centers[i, inds] <- 0
    }

    # generate the data
    samples <- matrix(0, nrow = observations, ncol = dataCols)
    ids <- matrix(0, nrow = observations, ncol = 1)
    for (i in seq_len(modeN)) {
        temp <- matrix(
            rnorm(obsPerMode * dataCols, mean = 0, sd = 1),
            obsPerMode, dataCols
        )
        samples[seq((1 + obsPerMode * (i - 1)), obsPerMode * (i)), ] <- temp +
            rep(centers[i, ], each = obsPerMode)
        ids[seq((1 + obsPerMode * (i - 1)), obsPerMode * (i)), ] <- i
    }
    result <- list(samples, ids, centers)
    names(result) <- c("samples", "ids", "centers")
    return(result)
}

###################### generateBimodalData
context("dataGen")

centers <- rbind(c(0, 0), c(10, 10))
n <- 1000
prop <- 0.1
set.seed(110011)
x <- DepecheR:::generateBimodalData(centers, observations = n, prop = prop)
test_that("generateBimodalData expected output simple centers", {
    expect_equal(nrow(x$samples), n)
    expect_equal(ncol(x$samples), 2)
    expect_true(all(x$ids[1:n * prop] == 1))
    expect_true(all(tail(x$ids, n - n * prop) == 2))
    expect_true(all(colMeans(x$samples) < centers[2, ] * (1 - prop) + 0.1))
    expect_true(all(colMeans(x$samples) > centers[2, ] * (1 - prop) - 0.1))
})

set.seed(0101101)
x <- DepecheR:::generateBimodalData(observations = n)
test_that("generateBimodalData expected output default centers", {
    expect_equal(nrow(x$samples[x$ids == 1, ]), n * 0.3)
    expect_equal(nrow(x$samples[x$ids == 2, ]), n * 0.7)
    expect_true(mean(x$samples[x$ids == 1, ]) < 51)
    expect_true(mean(x$samples[x$ids == 1, ]) > 49)
})

set.seed(10001)
x <- DepecheR:::generateSparseData(modeN = 3, observations = 30)
test_that("generateSparseData expected output default centers", {
    binSamples <- abs(x$samples) < 25
    groundTruth <-
        t(cbind(
            replicate(10, x$centers[1, ] == 0),
            replicate(10, x$centers[2, ] == 0),
            replicate(10, x$centers[3, ] == 0)
        ))
    expect_true(all(binSamples == groundTruth))
})

###################### depeche

context("depeche")
set.seed(101)
x <- DepecheR:::generateBimodalData(observations = 100, dataCols = 150)
out <- depeche(x$samples, maxIter = 8, nCores = 2, createOutput = FALSE)
test_that("depeche expected output", {
    expect_true(max(out$clusterVector) == 2)
    expect_true(all(x$ids == out$clusterVector) ||
        all(x$ids %% 2 + 1 == out$clusterVector))
})

# sparsity test
set.seed(10)
x <- DepecheR:::generateSparseData(observations = 500)
out <- depeche(x$samples,
    maxIter = 8, createOutput = FALSE, nCores = 2,
    center = FALSE
)
binCenters <- x$centers == 0
binClCenters <- out$clusterCenters == 0
test_that("depeche expected sparsity", {
    for (i in 1:5) {
        expect_true(all(binCenters[i, ] ==
            binClCenters[which(rowSums(binClCenters) == i), ]))
    }
})

#' This function is the core of the neighSmooth. See the documentation
#' there for details.
#' @param dataCenter The original data.
#' @param dataNeigh The data for the neighbors. Often stronly overlapping with
#' the dataCenter, but for internal reasons, this data cloud is larger
#' than the dataCenter cloud.
#' @param dataReturn The neighbor data that should be aggregated and sent back.
#' @param method Should median or mean be calculated?
#' @param k Number of neighbors.
#' @param trim If mean of the neighbors is returned, should it be calculated
#' with trimming?
#' @return A dataset with the same shape as dataCenter, filled with
#' aggregated information from the k nearest neighbors.
#' @importFrom FNN knnx.index
#' @importFrom robustbase colMedians
#' @importFrom collapse fmode
#' @keywords internal
microClust <- function(dataCenter, dataNeigh, dataReturn,
                       method = "median", k = 11, trim = 0) {
    closest10Pos <- knnx.index(dataNeigh, dataCenter,
        k = k,
        algorithm = "cover_tree"
    )
    if (method == "median") {
        if (is.matrix(dataReturn)) {
            closest10Result <- lapply(
                seq_len(nrow(closest10Pos)),
                function(x) {
                      colMedians(dataReturn[closest10Pos[x, ], ],
                          hasNA = FALSE
                      )
                  }
            )
            resultDf <- do.call("rbind", closest10Result)
        } else {
            closest10Result <- vapply(
                seq_len(nrow(closest10Pos)),
                function(x) {
                      median(dataReturn[closest10Pos[x, ]])
                  }, 1
            )
        }
    } else if (method == "mean"){
        if (is.matrix(dataReturn)) {
            closest10Result <- lapply(
                seq_len(nrow(closest10Pos)),
                function(x) {
                      colMeans(dataReturn[closest10Pos[x, ], ])
                  }
            )
            resultDf <- do.call("rbind", closest10Result)
        } else {
            closest10Result <- vapply(
                seq_len(nrow(closest10Pos)),
                function(x)
                    mean(dataReturn[closest10Pos[x, ]]),
                1
            )
        }
    } else if (method == "nUnique"){
      closest10Result <- vapply(
        seq_len(nrow(closest10Pos)),
        function(x)
            length(unique(dataReturn[closest10Pos[x, ]])),
        1
      )
    } else if (method == "mode"){
        #Here, we have to deal with some peculearities in the neighSmoooth
        #code, that
        #means that the indata always is a matrix. Therefore, we will work
        #with matrices at this stage.
        #closest10Result <- vapply(
        #    seq_len(nrow(closest10Pos)),
        #    function(x)
        #        fmode(dataReturn[closest10Pos[x, ]]),
        #    1
        #)
        closest10Result <- lapply(
            seq_len(nrow(closest10Pos)),
            function(x) {
                fmode(dataReturn[closest10Pos[x, ], ]
                )
            }
        )
        resultDf <- do.call("rbind", closest10Result)
    }
}

#' Wilcoxon rank-sum or signed rank test comparison of subject groups in a
#' dClust result
#'
#'
#' This function is used to compare groups of individuals from whom comparable
#' cytometry or other complex data has been generated.
#' @importFrom matrixStats rowMedians
#' @param xYData A dataframe or matrix with two columns. Each row contains
#' information about the x and y positition in the field for that observation.
#' @param idsVector Vector with the same length as xYData containing information
#' about the id of each observation.
#' @param groupVector Vector with the same length as xYData containing
#' information about the group identity of each observation.
#' @param clusterVector Vector with the same length as xYData containing
#' information about the cluster identity of each observation.
#' @param displayVector Optionally, if the dataset is very large and the SNE
#' calculation hence becomes impossible to perform for the full dataset, this
#' vector can be included. It should contain the set of rows from the data used
#' for statistics, that has been used to generate the xYData.
#' @param paired Defaults to FALSE, i.e. no assumption of pairing is made and
#' Wilcoxon rank sum-test is performed. If true, the software will by default
#' pair the first id in the first group with the firs id in hte second group
#' and so forth.
#' @param multipleCorrMethod Which method that should be used for adjustment
#' of multiple comparisons. Defaults to Benjamini-Hochberg,
#' but all other methods available in \code{\link{p.adjust}} can be used.
#' @param densContour If density contours should be created for the plot(s) or
#' not. Defaults to TRUE. a
#' @param plotName The main name for the graph and the analysis.
#' @param groupName1 The name for the first group
#' @param groupName2 The name for the second group
#' @param lowestPlottedP If multiple plots should be compared, it might be
#' useful to define a similar color scale for all plots, so that the same color
#' always means the same statistical value. A p-value that determines this can
#' be added here. Default is a p-value of 0.05. In cases where no datapoints
#' have any lower p-values than this, a Wilcoxon-statistic corresponding as
#' closely as possible to 0.05 will be identified with iterations of datasets
#' with the same size as indicated by hte group vector. If one value is
#' lowerthan 0.05, the wilcoxon statistic from this comparison is used instead.
#' @param title If there should be a title displayed on the plotting field.
#' As the plotting field is saved as a png, this title cannot be removed as an
#' object afterwards, as it is saved as coloured pixels. To simplify usage for
#' publication, the default is FALSE, as the files are still named, eventhough
#' no title appears on the plot.
#' @param plotDir If different from the current directory. If specified and
#' non-existent, the function creates it. If "." is specified, the plots will be
#' saved at the current directory.
#' @param bandColor The color of the contour bands. Defaults to black.
#' @param dotSize Simply the size of the dots. The default makes the dots
#' smaller the more observations that are included.
#' @param createOutput For testing purposes. Defaults to TRUE. If FALSE, no
#' plots are generated.
#' @seealso \code{\link{dColorPlot}}, \code{\link{dDensityPlot}},
#' \code{\link{dResidualPlot}}
#' @return This function always returns a dataframe showing the Wilcoxon
#' statistic and the p-value for each cluster, with an included adjustment for
#' multiple comparisons (see above). It also returns a sne based plot showing
#' which events that belong to a cluster dominated by the first or the second
#' group.
#' @examples
#'
#' # Load some data
#' data(testData)
#' \dontrun{
#' # Load or create the dimensions that you want to plot the result over.
#' # uwot::umap recommended due to speed, but tSNE or other method would
#' # work as fine.
#' data(testDataSNE)
#'
#' # Run the clustering function. For more rapid example execution,
#' # a depeche clustering of the data is inluded
#' # testDataDepeche <- depeche(testData[,2:15])
#' data(testDataDepeche)
#'
#' # Run the function
#' dWilcoxResult <- dWilcox(
#'     xYData = testDataSNE$Y, idsVector = testData$ids,
#'     groupVector = testData$label, clusterVector =
#'     testDataDepeche$clusterVector
#' )
#'
#' # Here is an example of how the display vector can be used.
#' subsetVector <- sample(1:nrow(testData), size = 10000)
#'
#' # Now, the SNE for this displayVector could be created
#' # testDataSubset <- testData[subsetVector, 2:15]
#' # testDataSNESubset <- Rtsne(testDataDisplay, pca=FALSE)$Y
#' # But we will just subset the testDataSNE immediately
#' testDataSNESubset <- testDataSNE$Y[subsetVector, ]
#'
#' # And now, this new SNE can be used for display, although all
#' # the data is used for the Wilcoxon calculations
#' dWilcoxResult <- dWilcox(
#'     xYData = testDataSNESubset, idsVector = testData$ids,
#'     groupVector = testData$label, clusterVector =
#'         testDataDepeche$clusterVector, displayVector = subsetVector
#' )
#' }
#' @export dWilcox
dWilcox <- function(xYData, idsVector, groupVector, clusterVector,
                    displayVector, paired = FALSE,
                    multipleCorrMethod = "BH", densContour = TRUE,
                    plotName = "default", groupName1 = unique(groupVector)[1],
                    groupName2 = unique(groupVector)[2], title = FALSE,
                    lowestPlottedP = 0.05, plotDir = ".", bandColor = "black",
                    dotSize = 500 / sqrt(nrow(xYData)), createOutput = TRUE) {
    if (plotDir != ".") {
        dir.create(plotDir)
    }

    if (length(unique(groupVector)) != 2) {
        stop("More or less than two groups are present. Please correct this.")
    }

    if (length(unique(idsVector)) < 8) {
        warning("NB! The number of unique ids is smaller than 8, so",
                " statisticac omparison is not suitable. Use dResidualPlot ",
                "instead to view differences.")
    }

    if (is.matrix(xYData)) {
        xYData <- as.data.frame(xYData)
    }

    if (plotName == "default") {
        plotName <- paste0(groupName1, "_vs_", groupName2)
    }

    # Here, the statistical evaluation is
    # performed.  First, the data is divided
    # into each group.
    clusterVectorGroup1 <- clusterVector[groupVector == unique(groupVector)[1]]
    clusterVectorGroup2 <- clusterVector[groupVector == unique(groupVector)[2]]
    idsVectorGroup1 <- as.character(idsVector[groupVector ==
        unique(groupVector)[1]])
    idsVectorGroup2 <- as.character(idsVector[groupVector ==
        unique(groupVector)[2]])

    # Now, a table with the percentage of
    # cells in each cluster for each
    # individual is created for both groups.

    clusterTable1 <-
        as.matrix(as.data.frame.matrix(table(
            clusterVectorGroup1,
            idsVectorGroup1
        )))
    clusterTable2 <-
        as.matrix(as.data.frame.matrix(table(
            clusterVectorGroup2,
            idsVectorGroup2
        )))

    # In the very unlikely event that there
    # is not a single observation for one
    # cluster from one of the groups, this
    # cluster is substituted to that table
    # with a row of zeros.
    if (nrow(clusterTable1) < length(unique(clusterVector))) {
        zeroMat <- matrix(data = 0, nrow = length(unique(clusterVector)) -
            nrow(clusterTable1), ncol = ncol(clusterTable1))

        colnames(zeroMat) <- colnames(clusterTable1)
        allRowNames <- as.character(sort(unique(clusterVector)))
        row.names(zeroMat) <- allRowNames[-which(allRowNames %in%
            row.names(clusterTable1))]
        # Here, rows are added to the cluster
        # table to make the number of rows the
        # same as the unique values of the
        # cluster vector.
        clusterTable1big <- rbind(clusterTable1, zeroMat)

        # The rows of the table are re-sorted
        clusterTable1bigResorted <-
            clusterTable1big[order(as.numeric(row.names(clusterTable1big))), ]
        clusterTable1 <- clusterTable1bigResorted
    }

    # And the same procedure is done for the
    # second group
    if (nrow(clusterTable2) < length(unique(clusterVector))) {
        zeroMat <- matrix(data = 0, nrow = length(unique(clusterVector)) -
            nrow(clusterTable2), ncol = ncol(clusterTable2))

        colnames(zeroMat) <- colnames(clusterTable2)
        allRowNames <- as.character(sort(unique(clusterVector)))
        row.names(zeroMat) <- allRowNames[-which(allRowNames %in%
            row.names(clusterTable2))]
        # Here, rows are added to the cluster
        # table to make the number of rows the
        # same as the unique values of the
        # cluster vector.
        clusterTable2big <- rbind(clusterTable2, zeroMat)

        # The rows of the table are re-sorted
        clusterTable2bigResorted <-
            clusterTable2big[order(as.numeric(row.names(clusterTable2big))), ]
        clusterTable2 <- clusterTable2bigResorted
    }

    countTable1 <- table(idsVectorGroup1)
    countTable2 <- table(idsVectorGroup2)

    clusterFractionsForAllIds1 <- clusterTable1
    clusterFractionsForAllIds2 <- clusterTable2

    for (i in seq_len(length(countTable1))) {
        x <- clusterTable1[, i] / countTable1[i]
        clusterFractionsForAllIds1[, i] <- x
    }

    for (i in seq_len(length(countTable2))) {
        x <- clusterTable2[, i] / countTable2[i]
        clusterFractionsForAllIds2[, i] <- x
    }

    # And here the statistical test is
    # performed for each cluster individually
    statisticList <- mapply(wilcox.test,
        as.data.frame.matrix(t(clusterFractionsForAllIds1)),
        as.data.frame.matrix(t(clusterFractionsForAllIds2)),
        MoreArgs = list(
            alternative = "two.sided",
            paired = paired, exact = FALSE
        ),
        SIMPLIFY = FALSE
    )

    # Now, the statistics and the p-values
    # are retrieved
    statistic <- unlist(lapply(statisticList, `[[`, 1))
    p_values <- unlist(lapply(statisticList, `[[`, 3))

    # Here, adjustments for multiple
    # comparisons are performed
    p_adjusted <- p.adjust(p_values, method = multipleCorrMethod)

    # Now the median for each group and
    # cluster is calculated
    median1 <- 100 * rowMedians(clusterFractionsForAllIds1)
    median2 <- 100 * rowMedians(clusterFractionsForAllIds2)

    # Combine the four
    result <- data.frame(
        as.numeric(names(p_values)),
        median1, median2, statistic, p_values, p_adjusted
    )
    row.names(result) <- c(seq_len(nrow(result)))
    colnames(result) <- c(
        "Cluster", paste0("Median_%_for_", groupName1),
        paste0("Median_%_for_", groupName2),
        "Wilcoxon_statistic", "p-value",
        paste0(multipleCorrMethod, "_corrected_p-value")
    )

    # Here, a vector with the same length as
    # the cluster vector is generated, but
    # where the cluster info has been
    # substituted with the statistic.
    # Here, the p-values are transformed to
    # be useful for plotting
    medianClustDiff <- median1 - median2
    p_adjusted_log <- log10(p_adjusted)
    p_adjusted_log_inv <- p_adjusted_log
    pVector <- clusterVector
    for (i in seq_len(length(p_adjusted_log))) {
        if (medianClustDiff[i] > 0) {
            p_adjusted_log_inv[i] <- -p_adjusted_log[i]
        }
        pVector[clusterVector == result$Cluster[i]] <- p_adjusted_log_inv[i]
    }

    # If a
    # displayVector has been included, it is
    # used here, to subset the clusterVector
    if (missing(displayVector) == FALSE) {
        pVector <- pVector[displayVector]
    }

    # Here the data that will be used for
    # plotting is scaled.
    colnames(xYData) <- c("V1", "V2")

    # Here, the maximum values for the
    # plotting are defined. If not added by
    # the user, they are obtained from the
    # data.

    if (min(p_adjusted) < lowestPlottedP) {
        message(
            "NB!, The lowest p-value with this dataset was ",
            min(p_adjusted), ". Therefore, this p-value will define the color",
            " scale instead than the chosen value of ", lowestPlottedP, "."
        )
        lowestPlottedP <- min(p_adjusted)
    }

    # Now, the lowest value is
    # log-transformed
    lowestPlottedPLog <- log10(lowestPlottedP)
    # Here, the breaks for the color
    # interpretation are created
    brks <- seq(lowestPlottedPLog, -lowestPlottedPLog, length.out = 10)

    # assign each value to a bin
    grps <- cut(pVector, breaks = brks, include.lowest = TRUE)
    colors <- colorRampPalette(c("#FF0000", "white", "#0000FF"))(9)
    xYData$col <- colors[grps]

    dPlotCoFunction(
        colorVariable = xYData$col, plotName =
            paste0(plotName, "_Wilcox_result"),
        xYData = xYData, title = title,
        densContour = densContour, bandColor = bandColor,
        dotSize = dotSize, plotDir = plotDir,
        createOutput = createOutput
    )

    # Create a color legend with text
    # Here the scale is created
    scaleHighPart <- 10^seq(0, lowestPlottedPLog, len = 3)
    scaleLowPart <- rev(scaleHighPart[2:3])
    plotScale <- c(scaleLowPart, scaleHighPart)

    yname <- paste(multipleCorrMethod, " corrected p-values", sep = "")
    topText <- paste(groupName1, " is more abundant", sep = "")
    bottomText <- paste(groupName2, " is more abundant", sep = "")

    if (createOutput) {
        pdf(file.path(plotDir, paste0(
            plotName,
            "_Wilcoxon_scale.pdf"
        )))
        par(fig = c(0.35, 0.65, 0, 1), xpd = NA)
        z <- matrix(seq_len(9), nrow = 1)
        x <- 1
        y <- seq(lowestPlottedPLog, -lowestPlottedPLog, length.out = 9)
        image(x, y, z, col = colors, axes = FALSE, xlab = "", ylab = yname)
        text(0.32, -lowestPlottedPLog, labels = (round(plotScale[5],
            digits = 5
        )))
        text(0.32, -lowestPlottedPLog / 2, labels = (round(plotScale[4],
            digits = 5
        )))
        text(x = 0.32, y = 0, labels = 1)
        text(0.32, lowestPlottedPLog / 2, labels = (round(plotScale[2],
            digits = 5
        )))
        text(0.32, lowestPlottedPLog, labels = (round(plotScale[1],
            digits = 5
        )))
        text(1, -lowestPlottedPLog * 1.2, labels = topText, cex = 1.1)
        text(1, lowestPlottedPLog * 1.2, labels = bottomText, cex = 1.1)
        box()
        dev.off()
    }

    if (createOutput) {
        write.csv(result, file.path(plotDir, paste0(
            plotName,
            "_WilcoxResult.csv"
        )))
    }

    return(result)
}

#' How many donors are contained among the nearest neighbors?
#'
#'
#' This function constructs a variable that for each event shows the number of
#' donors in its nearest neighbor surroundings. It builds on the same
#' idea as has been put forward in the Sconify package:
#' -Burns TJ (2019). Sconify: A toolkit for performing KNN-based statistics for
#' flow and mass cytometry data. R package version 1.4.0 and
#' -Hart GT, Tran TM, Theorell J, Schlums H, Arora G, Rajagopalan S, et al.
#' Adaptive NK cells in people exposed to Plasmodium falciparum correlate
#' with protection from malaria. J Exp Med. 2019 Jun 3;216(6):1280–90.
#' First, the k nearest neighbors are defined for cell x. Then, the number of
#' donors in the k nearest neighbor cloud is returned.
#' @param donorData The donor information.
#' @param euclidSpaceData The data cloud in which the nearest neighbors for the
#' events should be identified. Can be a vector, matrix or dataframe.
#' @param neighRows The rows in the dataset that correspond to the neighbors
#' of the donorData points. This can be all the donorData points, or a subset,
#' depending on the setup.
#' @param ctrlRows Optionally, a set of control rows that are used to remove
#' background signal from the neighRows data before sending the data back.
#' @param kNeighK The number of nearest neighbors.
#' @param kMeansK The number of clusters in the initial step of the algorithm.
#' A higher number leads to shorter runtime, but potentially lower accuracy.
#' @return An object of the same dimensions as donorData that has been smoothed.
#' @importFrom parallel detectCores makeCluster stopCluster
#' @importFrom doSNOW registerDoSNOW
#' @importFrom foreach foreach %dopar%
#' @importFrom gmodels fast.prcomp
#' @importFrom FNN knnx.index
#' @examples
#' data(testData)
#' data(testDataSNE)
#' euclidSpaceData <-
#'     testData[, c(
#'         "SYK", "CD16", "CD57", "EAT.2",
#'         "CD8", "NKG2C", "CD2", "CD56"
#'     )]
#' \dontrun{
#' nDonorsVector <- nUniqueNeighDons(
#'     donorData = as.numeric(testData$label),
#'     euclidSpaceData
#' )
#' }
#' @export nUniqueNeighDons
nUniqueNeighDons <- function(donorData, euclidSpaceData,
                        neighRows = seq_len(nrow(as.matrix(donorData))),
                        ctrlRows, kNeighK = max(100, round(nrow(
                            as.matrix(euclidSpaceData)
                        ) / 10000)),
                        kMeansK = max(1, round(nrow(
                            as.matrix(euclidSpaceData)
                        ) / 1000))) {
    if (is.vector(donorData)) {
        donorData <- data.frame(donorData, 0)
        donorDataIsVector <- TRUE
    } else {
        donorDataIsVector <- FALSE
    }

    if (is.vector(euclidSpaceData)) {
        euclidSpaceData <- data.frame(euclidSpaceData, euclidSpaceData)
    } else if (is.matrix(euclidSpaceData)) {
        euclidSpaceData <- as.data.frame(euclidSpaceData)
    }
    # First, the dimensionality is reduced to 10 dimensions for the
    # euclidSpaceData
    if (ncol(euclidSpaceData) > 10) {
        dataRedDim <- fast.prcomp(euclidSpaceData)$x[, seq(1, 10)]
    } else {
        dataRedDim <- euclidSpaceData
    }
    # Now, the cells are clustered according to this analysis
    kMeansResult <- kmeans(dataRedDim, kMeansK, iter.max = 100)
    kMeansCenters <- kMeansResult$centers
    kMeansClusters <- kMeansResult$cluster
    print("Done with k-means")

    # Here, the rows connected to the neighbors, the control neighbors or
    # neither are defined
    groupVec <- rep("none", nrow(dataRedDim))
    groupVec[neighRows] <- "neigh"
    if (missing(ctrlRows) == FALSE) {
        groupVec[ctrlRows] <- "ctrl"
    }

    rowNumbers <- seq_len(nrow(dataRedDim))

    donorDataClustList <- split(donorData, kMeansClusters)
    dataRedDimClustList <- split(as.data.frame(dataRedDim), kMeansClusters)
    groupVecClustList <- split(groupVec, kMeansClusters)
    rowNumList <- split(rowNumbers, kMeansClusters)
    # clusterIds <- split(kMeansClustersData, kMeansClustersData)

    if (nrow(kMeansCenters) > 11) {
        distCenters <- as.list(as.data.frame(
            t(knnx.index(kMeansCenters, kMeansCenters, 11))
        ))
    } else {
        distCenters <- as.list(as.data.frame(
            t(knnx.index(kMeansCenters, kMeansCenters, nrow(kMeansCenters)))
        ))
    }

    print("Now the first bit is done, and the iterative part takes off")
    nCores <- detectCores() - 1

    cl <- parallel::makeCluster(nCores, type = "SOCK")
    registerDoSNOW(cl)

    allClusters <- names(donorDataClustList)
    firstCluster <- 1
    resultList <- list()
    x <- 1
    while (firstCluster <= length(allClusters)) {
        timeBefore <- Sys.time()
        if (((firstCluster + nCores) - 1) < length(allClusters)) {
            clusterRange <- firstCluster:((firstCluster + nCores) - 1)
        } else {
            clusterRange <- firstCluster:length(allClusters)
        }

        # Now, the datasets are constructed from this cluster range
        locDataRedDimClustList <- dataRedDimClustList[clusterRange]

        # Here, the neighbors and the control neighbors are found in the 11
        # closest clusters for each of the focus clusters.
        locDistCenters <- distCenters[clusterRange]

        neighCtrlNeighReturnList <- lapply(locDistCenters, function(y) {
            locNeighList <- lapply(y, function(z) {
                locNeigh <- dataRedDimClustList[[z]]
                locReturn <- donorDataClustList[[z]]
                locGroup <- groupVecClustList[[z]]
                list(
                    locNeigh[which(locGroup == "neigh"), ],
                    locNeigh[which(locGroup == "ctrl"), ],
                    locReturn[which(locGroup == "neigh"), ],
                    locReturn[which(locGroup == "ctrl"), ]
                )
            })
            locNeighNeigh <- as.matrix(do.call(
                "rbind",
                lapply(locNeighList, "[[", 1)
            ))
            locNeighReturn <- as.matrix(do.call(
                "rbind",
                lapply(locNeighList, "[[", 3)
            ))
            locCtrlNeigh <- as.matrix(do.call(
                "rbind",
                lapply(locNeighList, "[[", 2)
            ))
            locCtrlReturn <- as.matrix(do.call(
                "rbind",
                lapply(locNeighList, "[[", 4)
            ))
            list(locNeighNeigh, locNeighReturn, locCtrlNeigh, locCtrlReturn)
        })

        resultList[[x]] <- unlist(foreach(
            i = seq_along(locDataRedDimClustList),
            .packages = "DepecheR"
        ) %dopar%
            microClust(
                dataCenter = as.matrix(locDataRedDimClustList[[i]]),
                dataNeigh = neighCtrlNeighReturnList[[i]][[1]],
                dataReturn = neighCtrlNeighReturnList[[i]][[2]],
                method = "nUnique", k = kNeighK
            ))

        timeAfter <- as.numeric(Sys.time() - timeBefore)
        print(paste0(
            "Clusters ", clusterRange[1], " to ",
            clusterRange[length(clusterRange)],
            " smoothed in ", timeAfter, " ",
            attributes(timeAfter)$units, ". Now, ",
            length(allClusters) - clusterRange[length(clusterRange)],
            " clusters are left."
        ))
        firstCluster <- (clusterRange[length(clusterRange)] + 1)
        x <- x + 1
    }

    parallel::stopCluster(cl)
    fullResult <- unlist(resultList)
    fullResultOrdered <- fullResult[order(unlist(rowNumList))]

    return(fullResultOrdered)
}

#' Showing the residuals when subtracting the values from one group from
#' another on a SNE plot
#'
#'
#' This function is used to visually compare groups of individuals from whom
#' comparable cytometry or other complex data has been generated, but where the
#' number of individuals does not permit any statistical comparisons.
#' @param xYData A dataframe or matrix with two columns. Each row contains
#' information about the x and y positition in the field for that observation.
#' @param groupVector Vector with the same length as xYData containing
#' information about the group identity of each observation.
#' @param clusterVector Vector with the same length as xYData containing
#' information about the cluster identity of each observation.
#' @param densContour If density contours should be created for the plot(s) or
#' not. Defaults to TRUE.
#' @param plotName The main name for the graph and the analysis.
#' @param groupName1 The name for the first group
#' @param groupName2 The name for the second group
#' @param maxAbsPlottingValues If multiple plots should be compared, it might
#' be useful to define a similar color scale for all plots, so that the same
#' color always means the same value. Such a value can be added here. It
#' defaults to the maximum Wilcoxon statistic that is generated in the analysis.
#' @param title If there should be a title displayed on the plotting field. As
#' the plotting field is saved as a png, this title cannot be removed as an
#' object afterwards, as it is saved as coloured pixels. To simplify usage for
#' publication, the default is FALSE, as the files are still named, eventhough
#' no title appears on the plot.
#' @param bandColor The color of the contour bands. Defaults to black.
#' @param dotSize Simply the size of the dots. The default makes the dots
#' smaller the more observations that are included.
#' @param plotDir If different from the current directory. If specified and
#' non-existent, the function creates it. If "." is specified, the plots will be
#' saved at the current directory.
#' @param createOutput For testing purposes. Defaults to TRUE. If FALSE, no
#' plots are generated.
#' @seealso \code{\link{dColorPlot}}, \code{\link{dDensityPlot}},
#' \code{\link{dWilcox}}
#' @return A sne based plot showing which events that belong to a cluster
#' dominated by the first or the second group.
#' @examples
#' # Load some data
#' data(testData)
#' \dontrun{
#' # Load or create the dimensions that you want to plot the result over.
#' # uwot::umap recommended due to speed, but tSNE or other method would
#' # work as fine.
#' data(testDataSNE)
#'
#' # Run the clustering function. For more rapid example execution,
#' # a depeche clustering of the data is inluded
#' # testDataDepeche <- depeche(testData[,2:15])
#' data(testDataDepeche)
#'
#'
#' # And finally run the function
#' dResidualPlot(
#'     xYData = testDataSNE$Y, groupVector = testData[, 16],
#'     clusterVector = testDataDepeche$clusterVector
#' )
#' }
#' @export dResidualPlot
dResidualPlot <- function(xYData, groupVector, clusterVector,
                          densContour = TRUE,
                          groupName1 = unique(groupVector)[1],
                          groupName2 = unique(groupVector)[2],
                          plotName = "default", title = FALSE,
                          maxAbsPlottingValues,
                          bandColor = "black", plotDir = ".",
                          dotSize = 400 / sqrt(nrow(xYData)),
                          createOutput = TRUE) {
    if (plotDir != ".") {
        dir.create(plotDir)
    }

    if (length(unique(groupVector)) != 2) {
        stop("More or less than two groups are present. Please correct this.")
    }

    if (is.matrix(xYData)) {
        xYData <- as.data.frame(xYData)
    }

    if (plotName == "default") {
        plotName <- paste0(groupName1, "_vs_", groupName2)
    }

    # Here, the residuals are identified.  A
    # table with the percentage of cells in
    # each cluster for each group is created.

    clusterTableRaw <- table(clusterVector, groupVector)

    # In the case where the groups do not occur in order in the groupVector,
    # the order of the table is changed here
    clusterTable <- clusterTableRaw[, as.character(unique(groupVector))]

    countTable <- table(groupVector)

    clusterPercentagesForGroups <- clusterTable

    for (i in seq_len(length(countTable))) {
        x <- 100 * clusterTable[, i] / countTable[i]
        clusterPercentagesForGroups[, i] <- x
    }

    # Now the residual matrix is constructed
    residualVector <- as.vector(clusterPercentagesForGroups[, 1] -
        clusterPercentagesForGroups[, 2])

    names(residualVector) <- as.numeric(row.names(clusterPercentagesForGroups))

    # Here, the maximum values for the
    # plotting is defined if not added by the
    # user.
    if (missing(maxAbsPlottingValues)) {
        maxAbsPlottingValues <- max(abs(clusterPercentagesForGroups))
    }

    # It might be that the
    # maxAbsPlottingValues defined by the
    # user are lower than the most extreme
    # values in the residual data. Then, the
    # data is truncated to fit the range.
    residualVector[residualVector > maxAbsPlottingValues] <-
        maxAbsPlottingValues
    residualVector[residualVector < -maxAbsPlottingValues] <-
        -maxAbsPlottingValues

    # Here, a vector with the same length as
    # the cluster vector is generated, but
    # where the cluster info has been
    # substituted with the statistic.
    residualVectorLong <- clusterVector
    for (i in seq_len(length(residualVector))) {
        residualVectorLong[clusterVector ==
            names(residualVector)[i]] <- residualVector[i]
    }

    # Here the data that will be used for
    # plotting is scaled.
    colnames(xYData) <- c("V1", "V2")

    # Make a color vector with the same
    # length as the data
    residual.df <- as.data.frame(residualVectorLong)

    # make a breaks vector to define each bin
    # for the colors
    brks <- seq(-maxAbsPlottingValues, maxAbsPlottingValues, length.out = 10)

    # assign each value to a bin
    grps <- with(residual.df, cut(residual.df[, 1],
        breaks = brks,
        include.lowest = TRUE
    ))
    colors <- colorRampPalette(c("#FF0000", "white", "#0000FF"))(9)
    xYData$col <- colors[grps]

    dPlotCoFunction(
        colorVariable = xYData$col, plotName =
            paste0(plotName, "_residuals"),
        xYData = xYData, title = title,
        densContour = densContour, bandColor = bandColor,
        dotSize = dotSize, plotDir = plotDir,
        createOutput = createOutput
    )

    # Create a color legend with text

    yname <- "Residual values"
    topText <- paste0(groupName1, " is more abundant")
    bottomText <- paste0(groupName2, " is more abundant")

    if (createOutput) {
        pdf(file.path(plotDir, paste0(plotName, "_residual_scale.pdf")))
        par(fig = c(0.35, 0.65, 0, 1), xpd = NA)
        z <- matrix(seq_len(9), nrow = 1)
        x <- 1
        y <- seq(-maxAbsPlottingValues, maxAbsPlottingValues, len = 9)
        image(x, y, z, col = colors, axes = FALSE, xlab = "", ylab = yname)
        axis(2)
        text(1, maxAbsPlottingValues * 1.2, labels = topText, cex = 1.1)
        text(1, -maxAbsPlottingValues * 1.2, labels = bottomText, cex = 1.1)
        box()
        dev.off()
    }
}

---
title: Probability plot usage
author: 
- name: Jakob Theorell
  affiliation: 
  - Oxford Autoimmune Neurology Group, Nuffield Department of Clinical Neurosciences, University of Oxford, Oxford, United Kingdom
  - Department of Clinical Neurosciences, Karolinska Institutet, Stockholm, Sweden
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{Using the groupProbPlot plot function for single-cell probability display}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}    
output: 
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
library(knitr)
opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
knitr::opts_chunk$set(echo = TRUE)
```

# Introduction

In this document, a typical analysis using the groupProbPlot function is shown. 
The indention with this function is to display differences between groups, 
tissues, stimulations or similar, with a single-cell resolution. The idea is 
that a cell that comes from a cell type that is specific for one of the two 
investigated groups will be surrounded exclusively by euclidean nearest 
neighbors that come from the same group. This is the basis for the analysis: 
in the standard case, the individual cell is given a number between -1 and 1 
that reflects which fraction of the 100 closest neighbors in the euclidean space
created by all the input markers that come from one group (-1) or the other (1).
The scale is tweaked to reflect that the middle in this case corresponds to a 
likelihood of a perfect mix with 50% of the cells from each group. For an
introduction to the package and example data description, see the general 
DepecheR package vinjette. 

# Installation
This is how to install the package, if that has not already been done: 
```{r, eval = FALSE}
if(!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("DepecheR")
```

# Preparations of example data
For visualization purposes, some 2-dimensional representation of the data is
necessary. This could just be two of the variables used to construct the 
probability vector, but it is more informative to include data on all variables,
aided by e.g. tSNE or UMAP. In this case, we will display the data with tSNE. 

```{r}
library(DepecheR)
data("testData")
data("testDataSNE")
```

# Group probability plotting
This function differs from the other group differentiation functions in the 
DepecheR package in that no clustering output from the depeche function or any
other clustering algorithm is needed as input. Instead, all the indata that the
euclidean nearest neighbors should be identified from needs to be added,
together with a group identity vector and the 2D data used to display the data.
Optionally, the resulting group probability vector can be returned, which will
be the case in this example. 

```{r, eval = FALSE}
dataTrans <-
  testData[, c("SYK", "CD16", "CD57", "EAT.2", "CD8", "NKG2C", "CD2", "CD56")]

testData$groupProb <- groupProbPlot(xYData = testDataSNE$Y, 
                                    groupVector = testData$label, 
                                    groupName1 = "Group_1",
                                    groupName2 = "Group_2",
                                    dataTrans = dataTrans)
## [1] "Done with k-means"
## [1] "Now the first bit is done, and the iterative part takes off"
## [1] "Clusters 1 to 7 smoothed in 2.9159369468689 . Now, 13 clusters are 
## [1] left."
## [1] "Clusters 8 to 14 smoothed in 0.925199031829834 . Now, 6 clusters are 
## [1] left."
## [1] "Clusters 15 to 20 smoothed in 0.905373096466064 . Now, 0 clusters are 
## [1] left."
```

When running this function, the output is  a high-resolution plot saved to disc.
A low resolution variant of the result (made small for BioConductor size 
constraint reasons) is shown here. In this case, the groups are so separated,
that almost all cells show a 100% probability of belonging to one of the groups
or the other. This is unusual with real data, so the white fields are generally
larger. 

![Group probaility plot](https://raw.githubusercontent.com/jtheorell/DepecheRVinjettePics/master/ProbPlotExample.png)

# Session information
```{r}
sessionInfo()
```

---
title: Example of a cytometry data analysis with DepecheR
author: 
- name: Jakob Theorell
  affiliation: 
  - Oxford Autoimmune Neurology Group, Nuffield Department of Clinical Neurosciences, University of Oxford, Oxford, United Kingdom
  - Department of Clinical Neurosciences, Karolinska Institutet, Stockholm, Sweden
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{Example of a cytometry data analysis with DepecheR}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}    
output: 
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
library(knitr)
opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
knitr::opts_chunk$set(echo = TRUE)
```

# Introduction

In this document, the user is presented with an analysis that DepecheR
has been written to perform. There are lots of tweaks to this general outline, 
so the user is encouraged to read the help files for each function individually
in addition. In cases where bugs are identified, feedback is most welcome, 
primarily on the github site github.com/theorell/DepecheR. Now let us get 
started. 

# Installation
This is how to install the package, if that has not already been done: 
```{r, eval = FALSE}
if(!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("DepecheR")
```

# Example data description

The data used in this example is a semi-simulated dataset, consisting of 1000
cytotoxic lymphocytes from each of 20 individuals. These have been categorized 
into two groups, and after this, alterations have been added to the sizes of 
some cell populations in both groups. This means that the groups can be 
separated based onthe sizes of certain cell types in the data. And this 
excersize will show how to identify these, and tell us what markers that define 
the separating cell types in question. 

Importantly, DepecheR does not provide any pre-processing tools, such as for
compensation/spectral unmixing of flow cytometry files. The clustering function
does have an internal algorithm to detect data with extreme tails, but this does
not circumvent the need to transform flow- or mass cytometry data. This can be 
done using either commercially available software or with R packages, such as 
Biocpkg("flowSpecs"), Biocpkg("flowCore") or Biocpkg("flowVS"). 

```{r}
library(DepecheR)
data('testData')
str(testData)
```

As can be noted here, the expected input format is either a 
dataframe or a matrix with cells as rows and markers/variables as columns. This 
is in accordance with the .fcs file convention. In this case, however, the 
different samples (coming from donors) should be added to the same dataframe, 
and a donor column should specify which cells that belong to which donor.
If you have .fcs files, you can do this conversion easily using the
"flowSet2LongDf" function in Biocpkg("flowSpecs").

# depeche clustering

With the depeche clustering function, all necessary scaling and parameter
selection is performed under the hood, so all we have to do, when we have the 
file of interest in the right format, is to run the function on the variables
that we want to cluster on.

```{r eval=FALSE}
testDataDepeche <- depeche(testData[, 2:15])
```
```{r}
## [1] "Files will be saved to ~/Desktop"
## [1] "As the dataset has less than 100 columns, peak centering is applied."
## [1] "Set 1 with 7 iterations completed in 14 seconds."
## [1] "Set 2 with 7 iterations completed in 6 seconds."
## [1] "Set 3 with 7 iterations completed in 6 seconds."
## [1] "The optimization was iterated 21 times."
```

```{r echo=FALSE}
data("testDataDepeche")
```
```{r}
str(testDataDepeche)
```

As can be seen above, the output from the function is a relatively complex list.
If the names of each list element is not suficiently self explanatory, see 
(?depeche) for information about each slot. 

## depeche function output graphs
Two graphs are part of the output from the depeche function. 

### Adjusted Rand Index as a function of penalty values
This graph shows 
how internally reproducible the results were for each of the tested penalties. 
An Adjusted Rand Index of 1 shows that if any random subset of observations is 
clustered two times, each observation will be assigned to the same cluster both
times. Conversely, an Adjusted Rand Index of 0 indicates the opposite, i.e. 
totally random distribution. The adjustment in "Adjusted" Rand index takes the 
divering probabilities of ending up with a high or low overlap in the special 
cases of very few and very many clusters into consideration. 

![Adjusted Rand Index as a function of penalty values. Each value is based on at least 7 clustering pairs (DepecheR standard in a 8 core machine).](https://github.com/jtheorell/DepecheRVinjettePics/raw/master/Fig_1.png)

### Cluster centers
This graph shows in a heatmap format where the cluster center is located for 
each of the markers that are defined for the cluster in question. A light color 
indicates a high expression, whereas a dark color indicates low or absent 
expression. Grey color, on the other hand, indicates that the cluster in 
question did not contribute to defining the cluster in question. In some cases,
the results might seem strange, as a cluster might have an expression very close
to the center of the full dataset, but this expression still defined the 
cluster. This is due to an internal, and for stability reasons necessary, effect
of the algotihm: a specific penalty will have a larger effect on a cluster with
fewer observations, than on a cluster with many observations.

![Cluster center heatmap. Marker names on the x-axis, cluster numbers on the y-axis.](https://github.com/jtheorell/DepecheRVinjettePics/raw/master/Fig_2.png)

# tSNE/umap generation

To be able to visualize the results, we need to generate a two-dimensional 
representation of the data used to generate the depeche clustering. Any 
sutiable method, such as tSNE or UMAP can be used for this purpose. I would 
today use uwot::umap, mainly as it in its R implementation is considerably 
faster than tSNE, but we will keep the tSNE here, as it well represents the 
data.

```{r eval=FALSE}
library(Rtsne)
testDataSNE <- Rtsne(testData[,2:15], pca=FALSE)
```

# Visualization of depeche clusters on 2D representation

Now, we want to evaluate how the different clusters are distributed on the 2D
representation.
To do this, we need to generate a color vector from the cluster vector in the 
testDataDepeche. This cluster vector is then overlayed over the tSNE, 
and to make things easier to interpret, a separate legend is included as well.
The reason that the legend is in a separate plot is for making it easier to use
the plots for publication purposes. For file size reasons, it has namely been
necessary to use PNG and not PDF for the plot files. 

NB! The resolution of the files normally generated by DepecheR is considerably 
higher than in this vinjette, due to size restrictions. 

```{r echo=FALSE}
    data("testDataSNE")
```

```{r}
dColorPlot(colorData = testDataDepeche$clusterVector, xYData = testDataSNE$Y, 
           colorScale = "dark_rainbow", plotName = "Cluster")
```

![Cluster distribution over the tSNE field.](https://github.com/jtheorell/DepecheRVinjettePics/raw/master/Fig_3.png)


# Visualization of markers on tSNE
Here, we once again use the dColorPlot function, with different settings. Note
that titles are included in this case. As they are then becoming embedded in the
png picture, this is not the standard, for publication reasons. It is also worth
noting, that nothing is printed to screen, but rather all graphics are saved as
separate files. This is done to save some computational time and effort.

```{r}
dColorPlot(colorData = testData[2], xYData = testDataSNE$Y)

```
![Example of one marker (SYK) distributed over the tSNE field.](https://github.com/jtheorell/DepecheRVinjettePics/raw/master/Fig_4.png)


# Density distribution of groups
Now, we are getting into separating the groups from each other. The first thing
we want to do is to visually compare the densities of the groups. This is done 
in the following way. First, the density for all events are plotted. This is 
followed by plotting of the first and second group of individuals, but keeping 
the density contours from the full dataset. 

```{r}
densContour <- dContours(testDataSNE$Y)

dDensityPlot(xYData = testDataSNE$Y, plotName = 'All_events', 
             colorScale="purple3", densContour = densContour)

#Here the data for the first group is plotted
dDensityPlot(xYData = testDataSNE$Y[testData$label==0,], plotName = 'Group_0', 
             colorScale="blue", densContour = densContour)

#And here comes the second group
dDensityPlot(xYData = testDataSNE$Y[testData$label==1,], plotName = 'Group_1', 
             colorScale="red", densContour = densContour)

```
![Distribution of events in the different groups.](https://github.com/jtheorell/DepecheRVinjettePics/raw/master/Fig_5.png)

# Separating groups from each other
Now, we have arrived at a crucial point: we are now going to see which 
clusters that separate the two groups from each other. There are three functions
in the DepecheR package that can help us do this. The first one is the 
dResidualPlot. 

```{r}
dResidualPlot(
    xYData = testDataSNE$Y, groupVector = testData$label,
    clusterVector = testDataDepeche$clusterVector)
```

This function shows the difference on a 
per-group/per-cluster basis. This means that it is non-statistical, and thus
applicable even if the groups consist of only one or a few samples each. 
However, it cannot distinguish between a rare but very pronounced phenotype and
a common difference: i.e., an individual donor can be responsible for the full 
difference noted. To circumvent this, and to get some statistical inferences, 
two other functions are available: the dWilcox and the dSplsda. These functions
have identical input, but where the former performs a Wilcoxon rank-sum test
(also called Mann-whitney U test) on a per-cluster basis and thus results in
multiple comparisons, the latter (based on sparse projection to latent 
structures (aka partial least squares) discriminant analysis) instead identifies
the angle through the multi-dimensional datacloud created by the individual 
donor frequencies in each cluster that most optimally separates the groups. It 
then internally checks how well the groups are separated along this vector, and
plots the clusters that contribute to this separation with colors relative to 
how well the groups are separated. As this method is a sparse version of the 
method, it, like depeche, only identifies clusters that contribute robustly to 
separating the clusters, and has its internal tuning algorithm to define how the
penalty term should be set. For both methods, there are paired alternatives. In
this case, however, the data is not paired, and thus, the normal methods will be 
used. The standard use of the methods are: 

```{r}
dWilcoxResult <- dWilcox(
    xYData = testDataSNE$Y, idsVector = testData$ids,
    groupVector = testData$label, clusterVector = testDataDepeche$clusterVector)
```

```{r eval=FALSE}
sPLSDAObject <- dSplsda(xYData = testDataSNE$Y, idsVector = testData$ids,
                        groupVector = testData$label, 
                        clusterVector = testDataDepeche$clusterVector)
## Saving 3 x 3 in image

## [1] "The separation of the datasets was perfect, with no overlap between 
## the groups"

## [1] "Files were saved at /Users/jakthe/Labbet/GitHub/DepecheR/vignettes"

```
The object rendered by the dSplsda function is inherited from the 
[mixOmics](https://bioconductor.org/packages/release/bioc/html/mixOmics.html)
package. The object rendered by the dWilcox function is a matrix containing
information about the cluster number, the median in each group, the Wilcoxon 
statistic, the p-value and a p-value corrected for multiple comparisons. 
In addition to the graphs, the dWilcox and the dSplsda functions also output
result files. See ?dSplsda and ?dWilcox for more information. 

![Residual, Wilcoxon, sPLS-DA plots.](https://github.com/jtheorell/DepecheRVinjettePics/raw/master/Fig_6.png)

When investigating the results from the sPLS-DA or the dWilcox analysis, it was
clear that half of the clusters were highly significantly different between the
groups in this case.

# Visualization of defining markers for clusters.
This function is especially useful to view the cluster distributions for 
specific clusters of interest, such as the significant clusters from the 
previousstep. dViolins serves as a compliment to the cluster center heatmap 
(see step 2). 
In this function, the sparsityMatrix from the depeche run is used. The function 
will in addition to producing all the graphs also produce a hierarchy of folders 
where the graphs are placed. 
EDIT 2019-09-12: This function currently does not support most CyTOF data, as
the zero-peak is so dominant, which would make the interpretation of the plots
impossible. This will be fixed in the near future, by reducing the negative 
population.  

```{r eval}
dViolins(testDataDepeche$clusterVector, inDataFrame = testData, 
         plotClusters = 3, plotElements = testDataDepeche$essenceElementList)
```

![Example of the distribution of events in the four markers defining cluster 3.](https://github.com/jtheorell/DepecheRVinjettePics/raw/master/Fig_7.png)

# Summary
In this document, a typical analysis of a cytometry dataset is shown. There are
however very many other possibilities with this package. One of the major ones
is that it accuratly classifies scRNAseq data, and in that process reduces
the complexity of the data up to 1000-fold, as very few transcripts actually
define each cluster. For further information on how to do this, the reader
is currently encouraged to read the publication connected to this package: 
https://journals.plos.org/plosone/article?id=10.1371/journal.pone.0203247.

# Session information
```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DepecheR.R
\docType{package}
\name{DepecheR-package}
\alias{DepecheR}
\alias{DepecheR-package}
\title{DepecheR: Determination of essential phenotypic elements of clusters in high-dimensional entities}
\description{
The purpose of this package is to identify traits in a dataset that can separate groups. This is done on two levels. First, clustering is performed, using an implementation of sparse K-means. Secondly, the generated clusters are used to predict outcomes of groups of individuals based on their distribution of observations in the different clusters. As certain clusters with separating information will be identified, and these clusters are defined by a sparse number of variables, this method can reduce the complexity of data, to only emphasize the data that actually matters.
}
\details{
The package is indirectly clearly dependent on Rtsne for generation of
output. See comment on the github wiki for how to speed Rtsne up.
}
\author{
\strong{Maintainer}: Jakob Theorell \email{jakob.theorell@ki.se} (\href{https://orcid.org/0000-0001-8752-3151}{ORCID})

Authors:
\itemize{
  \item Axel Theorell \email{axel.theorell@gmail.com}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/groupProbPlot.R
\name{groupProbPlot}
\alias{groupProbPlot}
\title{Define and plot group probabilities}
\usage{
groupProbPlot(
  xYData,
  groupVector,
  euclidSpaceData,
  kNeighK = max(100, round(nrow(euclidSpaceData)/10000)),
  kMeansK = round(nrow(euclidSpaceData)/1000),
  densContour = TRUE,
  groupName1 = unique(groupVector)[1],
  groupName2 = unique(groupVector)[2],
  plotName = "default",
  title = FALSE,
  bandColor = "black",
  plotDir = ".",
  dotSize = 400/sqrt(nrow(xYData)),
  returnProb = FALSE,
  returnProbColVec = FALSE,
  createOutput = TRUE
)
}
\arguments{
\item{xYData}{A dataframe or matrix with two columns. Each row contains
information about the x and y positition in the field for that observation.}

\item{groupVector}{Vector with the same length as xYData containing
information about the group identity of each observation.}

\item{euclidSpaceData}{The data cloud in which the nearest neighbors for the
events should be identified.}

\item{kNeighK}{The number of nearest neighbors.}

\item{kMeansK}{The number of clusters in the initial step of the algorithm.
A higher number leads to shorter runtime, but potentially lower accuracy.}

\item{densContour}{If density contours should be created for the plot(s) or
not. Defaults to TRUE. a}

\item{groupName1}{The name for the first group}

\item{groupName2}{The name for the second group}

\item{plotName}{The main name for the graph and the analysis.}

\item{title}{If there should be a title displayed on the plotting field. As
the plotting field is saved as a png, this title cannot be removed as an
object afterwards, as it is saved as coloured pixels. To simplify usage for
publication, the default is FALSE, as the files are still named, eventhough
no title appears on the plot.}

\item{bandColor}{The color of the contour bands. Defaults to black.}

\item{plotDir}{If different from the current directory. If specified and
non-existent, the function creates it. If "." is specified, the plots will be
saved at the current directory.}

\item{dotSize}{Simply the size of the dots. The default makes the dots
smaller the more observations that are included.}

\item{returnProb}{Should a probability vector be returned? Mutually exclusive
with returnProbColVec.}

\item{returnProbColVec}{Should the color vector be returned as part of the
output? Mutually exclusive with returnProb.}

\item{createOutput}{For testing purposes. Defaults to TRUE. If FALSE, no
output is generated.}
}
\value{
A graph showing the probability as a color scale from blue over white
to red for each event to belong to one group or the other, with a separate
color scale. Optionally also the color vector, if returnProbColVec is TRUE.
}
\description{
This function defines and plots the single-observation probability for
belonging to either of two groups. It uses the \code{\link{neighSmooth}}
function with the special case that the values are binary: For each set of
k nearest neighbors, cell x is assigned a probability to belong to one group
or the other based on the percentage of the neighbors belonging to each
group. In other words, if 20 out of 100 neighbors belong to group
A and 80 belong to group B, and the value for the cell will be 20% for group
A or 80% for group B, depending on the point of view. This will also be
accordingly reflected in the color scale on the resulting plot.
}
\examples{
data(testData)
data(testDataSNE)
euclidSpaceData <-
    testData[, c(
        "SYK", "CD16", "CD57", "EAT.2",
        "CD8", "NKG2C", "CD2", "CD56"
    )]
\dontrun{
groupProbPlot(
    xYData = testDataSNE$Y, groupVector = testData$label,
    euclidSpaceData
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dViolins.R
\name{dViolins}
\alias{dViolins}
\title{Create violin plots for any variables of choise}
\usage{
dViolins(
  clusterVector,
  inDataFrame,
  plotClusters = unique(clusterVector),
  plotElements = "all",
  colorOrder = plotClusters,
  colorScale = "viridis",
  plotDir = "dViolin_result",
  createOutput = TRUE
)
}
\arguments{
\item{clusterVector}{Vector with the same length as inDataFrame containing
information about the cluster identity of each observation.}

\item{inDataFrame}{The data used to generate the depecheObject}

\item{plotClusters}{This vector of numbers define which cluster(s) to plot
the violins for. Defaults to all.}

\item{plotElements}{This provides information on which features to plot. In
the typical case, this is the essenceElementList from a depeche run. Other
input formats are however accepted: if a vector of column names is provided,
then these features will be plotted for all clusters. A custom list of
features specific for each cluster is also accepted. A final alternative is
to return "all" (default), in which case all markers will be plotted for all
clusters.If more than a 100 markers are provided, however, this will return
an error.}

\item{colorOrder}{The order of the cluster colors. Defaults to the order
that the unique values in clusterVector occurs.}

\item{colorScale}{The color scale. Options identical to dColorVector.}

\item{plotDir}{The name of the created directory.}

\item{createOutput}{For testing purposes. Defaults to TRUE. If FALSE, no
plots are generated.}
}
\value{
One graph is created for each cluster, containing a bean per
specified variable.
}
\description{
Here, assymetrical violin plots for each cluster vs all other clusters are
plotted for variables either retrieved from a depeche analysis or
user-defined.
}
\examples{
# Load some data
data(testData)

# Run the clustering function. For more rapid example execution,
# a depeche clustering of the data is inluded
# testDataDepeche <- depeche(testData[,2:15])
data(testDataDepeche)

# Create the plots of the variables that contribute to creating cluster 3

\dontrun{
dViolins(testDataDepeche$clusterVector,
    inDataFrame = testData,
    plotClusters = 3, plotElements = testDataDepeche$essenceElementList
)
}
}
\seealso{
\code{\link{dDensityPlot}}, \code{\link{dColorPlot}},
\code{\link{dColorVector}}, \code{\link{depeche}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/microClust.R
\name{microClust}
\alias{microClust}
\title{This function is the core of the neighSmooth. See the documentation
there for details.}
\usage{
microClust(
  dataCenter,
  dataNeigh,
  dataReturn,
  method = "median",
  k = 11,
  trim = 0
)
}
\arguments{
\item{dataCenter}{The original data.}

\item{dataNeigh}{The data for the neighbors. Often stronly overlapping with
the dataCenter, but for internal reasons, this data cloud is larger
than the dataCenter cloud.}

\item{dataReturn}{The neighbor data that should be aggregated and sent back.}

\item{method}{Should median or mean be calculated?}

\item{k}{Number of neighbors.}

\item{trim}{If mean of the neighbors is returned, should it be calculated
with trimming?}
}
\value{
A dataset with the same shape as dataCenter, filled with
aggregated information from the k nearest neighbors.
}
\description{
This function is the core of the neighSmooth. See the documentation
there for details.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/testData.R
\docType{data}
\name{testData}
\alias{testData}
\title{A 14 color flow cytometry dataset for example execution and playing around}
\format{
An object of class \code{'data.frame'};
}
\usage{
data("testData")
}
\description{
This dataset is a 14 color pre-compensated, transformed flow cytometry
dataset focusing on cytotoxic lymphocytes, where dead cells have been
removed. To make examples very obvious, differences have been artificially
exaggregated in the data. The dataset is produced by J. Theorell.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dScale.R
\name{dScale}
\alias{dScale}
\title{Scaling of a vector or a dataframe.}
\usage{
dScale(
  x,
  control,
  scale = TRUE,
  robustVarScale = TRUE,
  center = "peak",
  truncate = FALSE,
  multiplicationFactor = 1,
  returnCenter = FALSE,
  nCores = "default"
)
}
\arguments{
\item{x}{A numeric/integer vector or dataframe}

\item{control}{A numeric/integer vector or dataframe of values that could be
used to define the range. If no control data is present, the function
defaults to using the indata as control data.}

\item{scale}{If scaling should be performed. Three possible values: a vector
with two values indicating the low and high threshold quantiles for the
scaling, TRUE, which equals the vector 'c(0.001, 0.999)', and FALSE.}

\item{robustVarScale}{If the data should be scaled to its standard deviation
within the quantiles defined by the scale values above. If TRUE
(the default), the data is unit variance scaled based on the standard
deviation of the data within the range defined by scale.}

\item{center}{If centering should be performed. Alternatives are mean',
'peak' and FALSE. 'peak' results in centering around the highest peak in the
data, which is useful in most cytometry situations. 'mean' results in mean
centering.}

\item{truncate}{If truncation of the most extreme values should be performed.
Three possible values: TRUE, FALSE, and a vector with two values indicating
the low and high threshold quantiles for truncation.}

\item{multiplicationFactor}{A value that all values will be multiplied with.
Useful e.g. if the results preferrably should be returned as percent.}

\item{returnCenter}{Boolean. If center=TRUE, should the value at the center
be returned?}

\item{nCores}{If the function is run in multicore mode, which it will if the
dataset is large (nrow*ncol>10^6), this decides the number of cores. The
default is currently 87.5 percent with a cap on 10 cores, as no speed
increase is generally seen above 10 cores for normal computers to date.}
}
\value{
A vector or dataframe with the same size but where all values in the
vector or column of the dataframe have been internally scaled. In addition,
if returnCenter=TRUE, a value, or a vector if x is a matrix or a data frame.
}
\description{
This is a scaling function with a number of alternatives. This method for
scaling takes the shape of the data into somewhat more of a consideration
than minMaxScale does, but still gives less influence of outliers than more
conventional scalin alternatives, such as unit variance scaling.
}
\examples{
# Load some data
data(testData)

# Retrieve the first column
x <- testData[, 2]

# The maximum and minimum values are
max(x)
min(x)

# Run the function without mean centering and with the quantiles set to 0
# and 1.
y <- dScale(x, scale = c(0, 1), robustVarScale = FALSE, center = FALSE)

# And the data has been scaled to the range between 0 and 1.
max(y)
min(y)

# Now run the default function for a dataframe
summary(testData[, 2:15])

y_df <- dScale(testData[, 2:15])

# Here, the data has first been truncated to the default percentiles, then
# scaled to the standard deviation in the remaining interval and finally the
# center has been placed where the highest peak in the data is present.
# NB! Here, no truncation has been performed in the scaling, only to obtain
# the scaling values.

summary(y_df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dAllocate.R
\name{dAllocate}
\alias{dAllocate}
\title{Allocation of observations to pre-established cluster centers.}
\usage{
dAllocate(inDataFrame, depModel)
}
\arguments{
\item{inDataFrame}{A dataset that should be allocated to a set of cluster
centers, for example a richer, but less representative dataset, with all
datapoints from all donors, instead of only a set number of values from all.}

\item{depModel}{This is the result of the original application of the
depeche function on the associated, more representative dataset.}
}
\value{
A vector with the same length as number of rows in the inDataFrame,
where the cluster identity of each observation is noted.
}
\description{
Here, observations of a dataset are allocated to a set of preestablished
cluster centers. This is intended to be used for the test set in train-test
dataset situations.
}
\examples{
# Retrieve some example data
data(testData)
\dontrun{
# Now arbitrarily (for the sake of the example) divide the data into a
# training- and a test set.
testDataSample <- sample(1:nrow(testData), size = 10000)
testDataTrain <- testData[testDataSample, ]
testDataTest <- testData[-testDataSample, ]

# Run the depeche function for the train set

depeche_train <- depeche(testDataTrain[, 2:15],
    maxIter = 20,
    sampleSize = 1000
)

# Allocate the test dataset to the centers of the train dataset
depeche_test <- dAllocate(testDataTest[, 2:15], depeche_train
)

# And finally plot the two groups to see how great the overlap was:
clustVecList <- list(list("Ids" =testDataTrain$ids,
                          "Clusters" = depeche_train$clusterVector),
                     list("Ids" =testDataTest$ids,
                          "Clusters" = depeche_test))
tablePerId <- do.call("rbind", lapply(seq_along(clustVecList), function(x){
                                      locDat <- clustVecList[[x]]
                                      locRes <- apply(as.matrix(table(
                                      locDat$Ids, locDat$Clusters)),
                                      1, function(y) y/sum(y))
                                      locResLong <- reshape2::melt(locRes)
                                      colnames(locResLong) <-
                                      c("Cluster", "Donor", "Fraction")
                                      locResLong$Group <- x
                                      locResLong
                                      }))
tablePerId$Cluster <- as.factor(tablePerId$Cluster)
tablePerId$Group <- as.factor(tablePerId$Group)

library(ggplot2)
ggplot(data=tablePerId, aes(x=Cluster, y=Fraction,
        fill=Group)) + geom_boxplot() + theme_bw()
}
}
\seealso{
\code{\link{depeche}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dResidualPlot.R
\name{dResidualPlot}
\alias{dResidualPlot}
\title{Showing the residuals when subtracting the values from one group from
another on a SNE plot}
\usage{
dResidualPlot(
  xYData,
  groupVector,
  clusterVector,
  densContour = TRUE,
  groupName1 = unique(groupVector)[1],
  groupName2 = unique(groupVector)[2],
  plotName = "default",
  title = FALSE,
  maxAbsPlottingValues,
  bandColor = "black",
  plotDir = ".",
  dotSize = 400/sqrt(nrow(xYData)),
  createOutput = TRUE
)
}
\arguments{
\item{xYData}{A dataframe or matrix with two columns. Each row contains
information about the x and y positition in the field for that observation.}

\item{groupVector}{Vector with the same length as xYData containing
information about the group identity of each observation.}

\item{clusterVector}{Vector with the same length as xYData containing
information about the cluster identity of each observation.}

\item{densContour}{If density contours should be created for the plot(s) or
not. Defaults to TRUE.}

\item{groupName1}{The name for the first group}

\item{groupName2}{The name for the second group}

\item{plotName}{The main name for the graph and the analysis.}

\item{title}{If there should be a title displayed on the plotting field. As
the plotting field is saved as a png, this title cannot be removed as an
object afterwards, as it is saved as coloured pixels. To simplify usage for
publication, the default is FALSE, as the files are still named, eventhough
no title appears on the plot.}

\item{maxAbsPlottingValues}{If multiple plots should be compared, it might
be useful to define a similar color scale for all plots, so that the same
color always means the same value. Such a value can be added here. It
defaults to the maximum Wilcoxon statistic that is generated in the analysis.}

\item{bandColor}{The color of the contour bands. Defaults to black.}

\item{plotDir}{If different from the current directory. If specified and
non-existent, the function creates it. If "." is specified, the plots will be
saved at the current directory.}

\item{dotSize}{Simply the size of the dots. The default makes the dots
smaller the more observations that are included.}

\item{createOutput}{For testing purposes. Defaults to TRUE. If FALSE, no
plots are generated.}
}
\value{
A sne based plot showing which events that belong to a cluster
dominated by the first or the second group.
}
\description{
This function is used to visually compare groups of individuals from whom
comparable cytometry or other complex data has been generated, but where the
number of individuals does not permit any statistical comparisons.
}
\examples{
# Load some data
data(testData)
\dontrun{
# Load or create the dimensions that you want to plot the result over.
# uwot::umap recommended due to speed, but tSNE or other method would
# work as fine.
data(testDataSNE)

# Run the clustering function. For more rapid example execution,
# a depeche clustering of the data is inluded
# testDataDepeche <- depeche(testData[,2:15])
data(testDataDepeche)


# And finally run the function
dResidualPlot(
    xYData = testDataSNE$Y, groupVector = testData[, 16],
    clusterVector = testDataDepeche$clusterVector
)
}
}
\seealso{
\code{\link{dColorPlot}}, \code{\link{dDensityPlot}},
\code{\link{dWilcox}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dDensityPlot.R
\name{dDensityPlot}
\alias{dDensityPlot}
\title{Display density on 2D plot}
\usage{
dDensityPlot(
  xYData,
  colorScale = "default",
  plotName = "All_density",
  idsVector,
  densContour = TRUE,
  title = FALSE,
  plotDir = "default",
  bandColor = "black",
  dotSize = 500/sqrt(nrow(xYData)),
  createOutput = TRUE
)
}
\arguments{
\item{xYData}{A dataframe or matrix with two columns. Each row contains
information about the x and y positition in the field for that observation.}

\item{colorScale}{This gives the specific color for the densest part of the
plot(s). It has three possible values:
\describe{
    \item{A specific color, e.g. 'red' or '#FF0000'}{If no idsVector
    provided}
    \item{A color scale from dColorVector}{If idsVector provided. See
    \code{\link{dColorVector}} for alternatives.}
    \item{"default"}{"One color (blue) if no idsVector is provided, and
    otherwise the viridis color scale.}
}}

\item{plotName}{A name that is common to all density plots created. It can
be the groups name, e.g. 'Malaria patients' or 'Clusters'. If only one plot
is created, the name is still taken from here.}

\item{idsVector}{Optional. Vector with the same length as xYData containing
information about the id of each observation. If provided, density plots for
each individual id and all ids together are produced.}

\item{densContour}{If density contours should be created for the plot(s) or
not. Defaults to TRUE. If a density object, as generated by dContours, is
included, this will be used for the internal scaling of the plot, allowing
for density distribution checks of different subcompartments of the data with
the same scaling.}

\item{title}{If there should be a title displayed on the plotting field. As
the plotting field is saved as a png, this title cannot be removed as an
object afterwards, as it is saved as coloured pixels. To simplify usage for
publication, the default is FALSE, as the files are still named, eventhough
no title appears on the plot.}

\item{plotDir}{If different from the current directory. If not "." and
non-existent, the function creates it. Default is "." if idsVector is not
specified and otherwise paste0("Density plots for ", plotName, "s").}

\item{bandColor}{The color of the contour bands. Defaults to black.}

\item{dotSize}{Simply the size of the dots. The default makes the dots
smaller the more observations that are included.}

\item{createOutput}{For testing purposes. Defaults to TRUE. If FALSE, no
output is generated.}
}
\value{
Plots showing the densities of the specific xYData (subset) displayed
as color on the field created by the same xYData (subset).
}
\description{
Function to show density for a set of observations on a field created by two
variables. The plot is constructed primarily for displaying density of
2D-stochastic neighbour embedding fields, but can be used for any sets of
two known for the same observations. As the number of datapoints is often
very high, the files would, if saved as pdf of another vector based file
type become big. For this reason, the plots are saved as jpeg and no axes or
anything alike are added, to simplify usage in publications.
}
\examples{

# Load some data
data(testData)
\dontrun{
# Load or create the dimensions that you want to plot the result over.
# uwot::umap recommended due to speed, but tSNE or other method would
# work as fine.
data(testDataSNE)

# Plot all data together
dDensityPlot(xYData = testDataSNE$Y)

# Now each depeche cluster is plotted separately and together.

# Run the clustering function. For more rapid example execution,
# a depeche clustering of the data is included
# testDataDepeche <- depeche(testData[,2:15])
data(testDataDepeche)

dDensityPlot(
    xYData = testDataSNE$Y,
    idsVector = testDataDepeche$clusterVector,
    plotName = "cluster"
)
}
}
\seealso{
\code{\link{dColorPlot}}, \code{\link{dResidualPlot}},
\code{\link{dWilcox}}, \code{\link{dColorVector}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dColorPlot.R
\name{dColorPlot}
\alias{dColorPlot}
\title{Display third variable as color on a 2D plot}
\usage{
dColorPlot(
  colorData,
  controlData,
  xYData,
  colorScale = "rich_colors",
  plotName = "default",
  densContour = TRUE,
  title = FALSE,
  plotDir = "default",
  truncate = TRUE,
  bandColor = "black",
  dotSize = 500/sqrt(nrow(xYData)),
  continuous = "default",
  multiCore = "default",
  nCores = "default",
  createOutput = TRUE
)
}
\arguments{
\item{colorData}{A numeric matrix or dataframe or a vector, be it numeric,
charater or factor, that should be used to define the colors on the plot.
A pre-made vector of colors is also accepted.}

\item{controlData}{Optional. A numeric/integer vector or dataframe of values
that could be used to define the range of the colorData. If no control data
is present, the function defaults to using the colorData as control data.}

\item{xYData}{These variables create the field on which the colorData will
be displayed. It needs to be a matrix or dataframe with two columns and the
same number of rows as the colorData object.}

\item{colorScale}{This argument controls the colors in the plot. See
\code{\link{dColorVector}} for alternatives.}

\item{plotName}{The name(s) for the plot(s). 'default' returns the column
names of the colorData object in the case this is a dataframe and otherwise
returns the somewhat generic name 'testVariable'. It can be substituted with
a string (in the case colorData is a vector) or vector of strings, as long as
it has the same length as the number of columns in colorData.}

\item{densContour}{If density contours should be created for the plot(s) or
not. Defaults to TRUE. If a density object, as generated by dContours, is
included, this will be used instead.}

\item{title}{If there should be a title displayed on the plotting field. As
the plotting field is saved a jpeg, this title cannot be removed as an object
afterwards, as it is saved as coloured pixels. To simplify usage for
publication, the default is FALSE, as the files are still named, eventhough
no title appears on the plot.}

\item{plotDir}{If different from the current directory. If specified and
non-existent, the function creates it. If "." is specified, the plots will be
saved at the current directory. By default, a new directory is added if the
created plots will be more than 1.}

\item{truncate}{If truncation of the most extreme values should be performed
for the visualizations. Three possible values: TRUE, FALSE, and a vector
with two values indicating the low and high threshold quantiles for
truncation.}

\item{bandColor}{The color of the contour bands. Defaults to black.}

\item{dotSize}{Simply the size of the dots. The default makes the dots
maller the more observations that are included.}

\item{continuous}{Boolean. Is the colorData parameter continuous? If
default, then only numeric vectors with more than 20 values are considered
continuous.This only applies to situations with single vectors. In situations
where a dataframe is added as colorData, all variables are considered
continuous.}

\item{multiCore}{If the algorithm should be performed on multiple cores.
This increases the speed if the dataset is medium-large (>100000 rows) and
has at least 5 columns. Default is TRUE when these above criteria are met and
FALSE otherwise.}

\item{nCores}{If multiCore is TRUE, then this sets the number of parallel
processes. The default is currently 87.5 percent with a cap on 10 cores, as
no speed increase is generally seen above 10 cores for normal computers.}

\item{createOutput}{For testing purposes. Defaults to TRUE. If FALSE, no
plots are generated.}
}
\value{
Plots showing the colorData displayed as color on the field created
by xYData.
}
\description{
Function to overlay one variable for a set of observations on a field
created by two other variables known for the same observations. The plot is
constructed primarily for displaying variables on 2D-stochastic neighbour
embedding fields, but can be used for any sets of (two or) three variables
known for the same observations. As the number of datapoints is often very
high, the files would, if saved as pdf of another vector based file type
become extremely big. For this reason, the plots are saved as jpeg and no
axes or anything alike are added, to simplify usage in publications.
}
\examples{

# Load some data
data(testData)
\dontrun{
# Load or create the dimensions that you want to plot the result over.
# uwot::umap recommended due to speed, but tSNE or other method would
# work as fine.
data(testDataSNE)

# Run the function for two of the variables
dColorPlot(colorData = testData[2:3], xYData = testDataSNE$Y)

# Now each depeche cluster is plotted separately and together.

# Run the clustering function. For more rapid example execution,
# a depeche clustering of the data is included
# testDataDepeche <- depeche(testData[,2:15])
data(testDataDepeche)

dColorPlot(
    colorData = testDataDepeche$clusterVector,
    xYData = testDataSNE$Y, plotName = "clusters"
)
}
}
\seealso{
\code{\link{dDensityPlot}}, \code{\link{dResidualPlot}},
\code{\link{dWilcox}}, \code{\link{dColorVector}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dColorVector.R
\name{dColorVector}
\alias{dColorVector}
\title{Create a vector of colors of the same length as the data}
\usage{
dColorVector(x, colorOrder = unique(x), colorScale = "viridis")
}
\arguments{
\item{x}{Any vector.}

\item{colorOrder}{The order that the colors should be in in the output
vector. Defaults to the order that the unique values in x occurs.}

\item{colorScale}{The color scale. Inherited from the viridis, gplots and
grDevices packages (and the package-specific 'dark_rainbow'). Seven possible
scales are pre-made: inferno, magma, plasma, viridis, rich_colors, rainbow
and dark_rainbow. User specified vectors of colors
(e.g. c('#FF0033', '#03AF49')) are also accepted.}
}
\value{
A vector, the same length as x with each unique value substitutet
with a color.
}
\description{
This function takes a vector x and a shorter ordering vector with all the
unique values of the x vector in the specific order that the colors should be
in and returns a vector of RGB colors the same length as the initial x
vector.
}
\examples{
# Load some data
data(testData)

testColor <- dColorVector(testData$ids, colorScale = "plasma")

# In this case, each of the 97 individual donors in the dataset has gotten
# their own color code:
table(testColor)
}
\seealso{
\code{\link{dDensityPlot}}, \code{\link{dColorPlot}},
\code{\link{dViolins}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dSplsda.R
\name{dSplsda}
\alias{dSplsda}
\title{Sparse partial least squares discriminant analysis with paired and unpaired
data}
\usage{
dSplsda(
  xYData,
  idsVector,
  groupVector,
  clusterVector,
  displayVector,
  testSampleRows,
  paired = FALSE,
  densContour = TRUE,
  plotName = "default",
  groupName1 = unique(groupVector)[1],
  groupName2 = unique(groupVector)[2],
  thresholdMisclassRate = 0.05,
  title = FALSE,
  plotDir = ".",
  bandColor = "black",
  dotSize = 500/sqrt(nrow(xYData)),
  createOutput = TRUE
)
}
\arguments{
\item{xYData}{A dataframe or matrix with two columns. Each row contains
information about the x and y positition in the field for that observation.}

\item{idsVector}{Vector with the same length as xYData containing information
about the id of each observation.}

\item{groupVector}{Vector with the same length as xYData containing
information about the group identity of each observation.}

\item{clusterVector}{Vector with the same length as xYData containing
information about the cluster identity of each observation.}

\item{displayVector}{Optionally, if the dataset is very large
(>100 000 observations) and hence the SNE calculation becomes impossible to
perform for the full dataset, this vector can be included. It should contain
the set of rows from the data used for statistics, that has been used to
generate the xYData.}

\item{testSampleRows}{Optionally, if a train-test setup is wanted, the rows
specified in this vector are used to divide the dataset into a training set,
used to generate the analysis, and a test set, where the outcome is predicted
based on the outcome of the training set. All rows that are not labeled as
test rows are assumed to be train rows.}

\item{paired}{Defaults to FALSE, i.e. no assumption of pairing is made and
Wilcoxon rank sum-test is performed. If true, the software will by default
pair the first id in the first group with the firs id in the second group
and so forth, so make sure the order is correct!}

\item{densContour}{If density contours should be created for the plot(s) or
not. Defaults to TRUE. a}

\item{plotName}{The main name for the graph and the analysis.}

\item{groupName1}{The name for the first group}

\item{groupName2}{The name for the second group}

\item{thresholdMisclassRate}{This threshold corresponds to the usefulness of
the model in separating the groups: a misclassification rate of the default
0.05 means that 5 percent of the individuals are on the wrong side of the
theoretical robust middle line between the groups along the sPLS-DA axis,
defined as the middle point between the 3:rd quartile of the lower group and
the 1:st quartile of the higher group.}

\item{title}{If there should be a title displayed on the plotting field. As
the plotting field is saved as a png, this title cannot be removed as an
object afterwards, as it is saved as coloured pixels. To simplify usage for
publication, the default is FALSE, as the files are still named, eventhough
no title appears on the plot.}

\item{plotDir}{If different from the current directory. If specified and
non-existent, the function creates it. If "." is specified, the plots will be
saved at the current directory.}

\item{bandColor}{The color of the contour bands. Defaults to black.}

\item{dotSize}{Simply the size of the dots. The default makes the dots
smaller the more observations that are included.}

\item{createOutput}{For testing purposes. Defaults to TRUE. If FALSE, no
output is generated.}
}
\value{
This function returns the full result of the sPLS-DA. It also returns
a SNE based plot showing which events that belong to a cluster dominated by
the first or the second group defined by the sparse partial least squares
loadings of the clusters.
}
\description{
This function is used to compare groups of individuals from whom comparable
cytometry or other complex data has been generated. It is superior to just
running a Wilcoxon analysis in that it does not consider each cluster
individually, but instead uses a sparse partial least squares discriminant
analysis to first identify which vector thourgh the multidimensional data
cloud, created by the cluster-donor matrix, that optimally separates the
groups, and as it is a sparse algorithm, applies a penalty to exclude the
clusters that are orthogonal, or almost orthogonal to the discriminant
vector, i.e. that do not contribute to separating the groups. This is in
large a wrapper for the \code{\link[mixOmics]{splsda}} function from
the mixOmics package.
}
\examples{

# Load some data
data(testData)
\dontrun{
# Load or create the dimensions that you want to plot the result over.
# uwot::umap recommended due to speed, but tSNE or other method would
# work as fine.
data(testDataSNE)

# Run the clustering function. For more rapid example execution,
# a depeche clustering of the data is inluded
# testDataDepeche <- depeche(testData[,2:15])
data(testDataDepeche)


# Run the function. This time without pairing.
sPLSDAObject <- dSplsda(
    xYData = testDataSNE$Y, idsVector = testData$ids,
    groupVector = testData$label,
    clusterVector = testDataDepeche$clusterVector
)


# Here is an example of how the display vector can be used.
subsetVector <- sample(1:nrow(testData), size = 10000)

# Now, the SNE for this displayVector could be created
# testDataSubset <- testData[subsetVector, 2:15]
# testDataSNESubset <- Rtsne(testDataDisplay, pca=FALSE)$Y
# But we will just subset the testDataSNE immediately
testDataSNESubset <- testDataSNE$Y[subsetVector, ]

# And now, this new SNE can be used for display, although all
# the data is used for the sPLS-DA calculations
sPLSDAObject <- dSplsda(
    xYData = testDataSNESubset, idsVector = testData$ids,
    groupVector = testData$label, clusterVector =
        testDataDepeche$clusterVector,
    displayVector = subsetVector
)

# Finally, an example of a train-test set situation, where a random half the
# dataset is used for training and the second half is used for testing. It
# is naturally more biologically interesting to use two independent datasets
# for training and testing in the real world.
sPLSDAObject <- dSplsda(
    xYData = testDataSNE$Y, idsVector = testData$ids,
    groupVector = testData$label, clusterVector =
        testDataDepeche$clusterVector, testSampleRows = subsetVector
)
}
}
\seealso{
\code{\link[mixOmics]{splsda}}, \code{\link{dColorPlot}},
\code{\link{dDensityPlot}}, \code{\link{dResidualPlot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/testDataDepeche.R
\docType{data}
\name{testDataDepeche}
\alias{testDataDepeche}
\title{A depeche clustering of the testData set}
\format{
An object of class \code{'list'};
}
\usage{
data(testDataDepeche)
}
\description{
This is a depeche clustering of the testData dataset. It has
been generated with the Rtsne.multicore package
}
\details{
produced by J. Theorell.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/depeche.R
\name{depeche}
\alias{depeche}
\title{Perform optimization and penalized K-means clustering}
\usage{
depeche(
  inDataFrame,
  samplingSubset = seq_len(nrow(inDataFrame)),
  penalties = 2^seq(0, 5, by = 0.5),
  sampleSize = "default",
  selectionSampleSize = "default",
  k = 30,
  minARIImprovement = 0.01,
  optimARI = 0.95,
  maxIter = 100,
  log2Off = FALSE,
  center = "default",
  scale = TRUE,
  nCores = "default",
  plotDir = ".",
  createOutput = TRUE
)
}
\arguments{
\item{inDataFrame}{A dataframe or matrix with the data that will be used to
create the clustering. Cytometry data should be transformed using
biexponential, arcsinh transformation or similar, and day-to-day
normalizations should to be performed for all data if not all data has been
acquired on the same run. Scaling, etc, is on the other hand performed
within the function.}

\item{samplingSubset}{If the dataset is made up of an unequal number of cells
from multiple individuals, it might be wise to pre-define a subset of the
rows, which includes equal or near-equal numbers of cells from each
individual, to avoid a few outliers to dominate the analysis. This can be
done here. Should be a vector of row numbers in the inDataFrame.}

\item{penalties}{This argument decides whether a single penalty will be used
for clustering, or if multiple penalties will be evaluated to identify the
optimal one. A single value, a vector of values, or possibly a list of two
vectors, if dual clustering is performed can be given here. The suggested
default values are empirically defined and might not be optimal for a
specific dataset, but the algorithm will warn if the most optimal values are
on the borders of the range. Note that when the penalty is 0, there is no
penalization, which means that the algorithm runs standard K-means
clustering.}

\item{sampleSize}{This controls what fraction of the dataset that will be
used to run the penalty optimization. 'default' results in the full file in
files up to 10000 events. In cases where the sampleSize argument is larger
than 10000, default leads to the generation of a random subset to the same
size also for the selectionSampleSize. A user specified number is also
accepted.}

\item{selectionSampleSize}{The size of the dataset used to find the optimal
solution out of the many generated by the penalty optimization at each sample
size. 'default' results in the full file in files up to 10000 events. In
cases where the sampleSize argument is larger than 10000, default leads to
the generation of a random subset to the same size also for the
selectionSampleSize. A user specified number is also accepted.}

\item{k}{Number of initial cluster centers. The higher the number, the
greater the precision of the clustering, but the computing time also
increases linearly with the number of starting points. Default is 30. If
penalties=0, k-means clustering with k clusters will be performed.}

\item{minARIImprovement}{This is the stop criterion for the penalty
optimization algorithm: the more iterations that are run, the smaller will
the improvement of the corrected Rand index be, and this sets the threshold
when the inner iterations stop. Defaults to 0.01.}

\item{optimARI}{Above this level of ARI, all solutions are considered equally
valid, and the median solution is selected among them.}

\item{maxIter}{The maximal number of iterations that are performed in the
penalty optimization.}

\item{log2Off}{If the automatic detection for high kurtosis, and followingly,
the log2 transformation, should be turned off.}

\item{center}{If centering should be performed. Alternatives are 'default',
'mean', 'peak', FALSE and a vector of numbers with the same length as the
number of columns in the inDataFrame. 'peak' results in centering around the
highest peak in the data, which is useful in most cytometry situations.
'mean' results in  mean centering. 'default' gives different results
depending on the data: datasets with 100+ variables are mean centered,
and otherwise, peak centering is used. If a numeric vector is provided, it
is used to center the values to the numbers. This is preferable to
pre-centering the data and using the FALSE command, as it will lead to better
internal visualization procedures, etc. FALSE results in no centering, mainly
for testing purposes.}

\item{scale}{If scaling should be performed. If TRUE, the dataset will be
divided by the combined standard deviation of the whole dataset. If a number
is provided, the dataset is divided by this number. This scaling
procedure makes the default penalties fit most datasets with some precision.}

\item{nCores}{If multiCore is TRUE, then this sets the number of parallel
processes. The default is currently 87.5 percent with a cap on 10 cores, as
no speed increase is generally seen above 10 cores for normal computers.}

\item{plotDir}{Where should the diagnostic plots be printed?}

\item{createOutput}{For testing purposes. Defaults to TRUE. If FALSE, no
plots are generated.}
}
\value{
A nested list:
\describe{
   \item{clusterVector}{A vector with the same length as number of rows in
   the inDataFrame, where the cluster identity of each observation is
   noted.}
   \item{clusterCenters}{A matrix containing information
   about where the centers are in all the variables that contributed to
   creating the cluster with the given penalty term. An exact zero here
   indicates that the variable in question was sparsed out for that cluster.
   If a variable did not contribute to the separation of any cluster, it will
   not be present here.}
   \item{essenceElementList}{A per-cluster list of the items that were
   used to separate that cluster from the rest, i.e. the items that survived
   the penalty.}
   \item{penaltyOptList}{A list of two dataframes:
   \describe{
             \item{penaltyOpt.df}{A one row dataframe with the settings for
             the optimal penalty.}
             \item{meanOptimDf}{A dataframe with the information about the
             results with all tested penalty values.}
           }
    }
    \item{logCenterScale}{The values used to center and scale the
    data and information on if the data was log transformed. This
    information is used internally in dAllocate.}
}
}
\description{
This is the central function of the package. As input, only a dataset is
required. It starts by performing optimizations and then performs clustering
based on the values identified in the optimization step.
}
\examples{
# Load some data
data(testData)

# Here a run with the standard settings
\dontrun{
testDataDepecheResult <- depeche(testData[, 2:15])

# Look at the result
str(testDataDepecheResult)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nUniqueNeighDons.R
\name{nUniqueNeighDons}
\alias{nUniqueNeighDons}
\title{How many donors are contained among the nearest neighbors?}
\usage{
nUniqueNeighDons(
  donorData,
  euclidSpaceData,
  neighRows = seq_len(nrow(as.matrix(donorData))),
  ctrlRows,
  kNeighK = max(100, round(nrow(as.matrix(euclidSpaceData))/10000)),
  kMeansK = max(1, round(nrow(as.matrix(euclidSpaceData))/1000))
)
}
\arguments{
\item{donorData}{The donor information.}

\item{euclidSpaceData}{The data cloud in which the nearest neighbors for the
events should be identified. Can be a vector, matrix or dataframe.}

\item{neighRows}{The rows in the dataset that correspond to the neighbors
of the donorData points. This can be all the donorData points, or a subset,
depending on the setup.}

\item{ctrlRows}{Optionally, a set of control rows that are used to remove
background signal from the neighRows data before sending the data back.}

\item{kNeighK}{The number of nearest neighbors.}

\item{kMeansK}{The number of clusters in the initial step of the algorithm.
A higher number leads to shorter runtime, but potentially lower accuracy.}
}
\value{
An object of the same dimensions as donorData that has been smoothed.
}
\description{
This function constructs a variable that for each event shows the number of
donors in its nearest neighbor surroundings. It builds on the same
idea as has been put forward in the Sconify package:
-Burns TJ (2019). Sconify: A toolkit for performing KNN-based statistics for
flow and mass cytometry data. R package version 1.4.0 and
-Hart GT, Tran TM, Theorell J, Schlums H, Arora G, Rajagopalan S, et al.
Adaptive NK cells in people exposed to Plasmodium falciparum correlate
with protection from malaria. J Exp Med. 2019 Jun 3;216(6):1280–90.
First, the k nearest neighbors are defined for cell x. Then, the number of
donors in the k nearest neighbor cloud is returned.
}
\examples{
data(testData)
data(testDataSNE)
euclidSpaceData <-
    testData[, c(
        "SYK", "CD16", "CD57", "EAT.2",
        "CD8", "NKG2C", "CD2", "CD56"
    )]
\dontrun{
nDonorsVector <- nUniqueNeighDons(
    donorData = as.numeric(testData$label),
    euclidSpaceData
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/testDataSNE.R
\docType{data}
\name{testDataSNE}
\alias{testDataSNE}
\title{SNE of the testData set}
\format{
An object of class \code{'list'};
}
\usage{
data(testDataSNE)
}
\description{
This is a t-distributed stochastic neighbor embedding of the testData
dataset. It has been generated with the Rtsne.multicore package.
}
\references{
Jesse H. Krijthe (2015). Rtsne: T-Distributed Stochastic Neighbor
Embedding using a Barnes-Hut Implementation
(\href{https://github.com/RGLab/Rtsne.multicore/issues/5}{GitHub})
}
\keyword{#'}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dWilcox.R
\name{dWilcox}
\alias{dWilcox}
\title{Wilcoxon rank-sum or signed rank test comparison of subject groups in a
dClust result}
\usage{
dWilcox(
  xYData,
  idsVector,
  groupVector,
  clusterVector,
  displayVector,
  paired = FALSE,
  multipleCorrMethod = "BH",
  densContour = TRUE,
  plotName = "default",
  groupName1 = unique(groupVector)[1],
  groupName2 = unique(groupVector)[2],
  title = FALSE,
  lowestPlottedP = 0.05,
  plotDir = ".",
  bandColor = "black",
  dotSize = 500/sqrt(nrow(xYData)),
  createOutput = TRUE
)
}
\arguments{
\item{xYData}{A dataframe or matrix with two columns. Each row contains
information about the x and y positition in the field for that observation.}

\item{idsVector}{Vector with the same length as xYData containing information
about the id of each observation.}

\item{groupVector}{Vector with the same length as xYData containing
information about the group identity of each observation.}

\item{clusterVector}{Vector with the same length as xYData containing
information about the cluster identity of each observation.}

\item{displayVector}{Optionally, if the dataset is very large and the SNE
calculation hence becomes impossible to perform for the full dataset, this
vector can be included. It should contain the set of rows from the data used
for statistics, that has been used to generate the xYData.}

\item{paired}{Defaults to FALSE, i.e. no assumption of pairing is made and
Wilcoxon rank sum-test is performed. If true, the software will by default
pair the first id in the first group with the firs id in hte second group
and so forth.}

\item{multipleCorrMethod}{Which method that should be used for adjustment
of multiple comparisons. Defaults to Benjamini-Hochberg,
but all other methods available in \code{\link{p.adjust}} can be used.}

\item{densContour}{If density contours should be created for the plot(s) or
not. Defaults to TRUE. a}

\item{plotName}{The main name for the graph and the analysis.}

\item{groupName1}{The name for the first group}

\item{groupName2}{The name for the second group}

\item{title}{If there should be a title displayed on the plotting field.
As the plotting field is saved as a png, this title cannot be removed as an
object afterwards, as it is saved as coloured pixels. To simplify usage for
publication, the default is FALSE, as the files are still named, eventhough
no title appears on the plot.}

\item{lowestPlottedP}{If multiple plots should be compared, it might be
useful to define a similar color scale for all plots, so that the same color
always means the same statistical value. A p-value that determines this can
be added here. Default is a p-value of 0.05. In cases where no datapoints
have any lower p-values than this, a Wilcoxon-statistic corresponding as
closely as possible to 0.05 will be identified with iterations of datasets
with the same size as indicated by hte group vector. If one value is
lowerthan 0.05, the wilcoxon statistic from this comparison is used instead.}

\item{plotDir}{If different from the current directory. If specified and
non-existent, the function creates it. If "." is specified, the plots will be
saved at the current directory.}

\item{bandColor}{The color of the contour bands. Defaults to black.}

\item{dotSize}{Simply the size of the dots. The default makes the dots
smaller the more observations that are included.}

\item{createOutput}{For testing purposes. Defaults to TRUE. If FALSE, no
plots are generated.}
}
\value{
This function always returns a dataframe showing the Wilcoxon
statistic and the p-value for each cluster, with an included adjustment for
multiple comparisons (see above). It also returns a sne based plot showing
which events that belong to a cluster dominated by the first or the second
group.
}
\description{
This function is used to compare groups of individuals from whom comparable
cytometry or other complex data has been generated.
}
\examples{

# Load some data
data(testData)
\dontrun{
# Load or create the dimensions that you want to plot the result over.
# uwot::umap recommended due to speed, but tSNE or other method would
# work as fine.
data(testDataSNE)

# Run the clustering function. For more rapid example execution,
# a depeche clustering of the data is inluded
# testDataDepeche <- depeche(testData[,2:15])
data(testDataDepeche)

# Run the function
dWilcoxResult <- dWilcox(
    xYData = testDataSNE$Y, idsVector = testData$ids,
    groupVector = testData$label, clusterVector =
    testDataDepeche$clusterVector
)

# Here is an example of how the display vector can be used.
subsetVector <- sample(1:nrow(testData), size = 10000)

# Now, the SNE for this displayVector could be created
# testDataSubset <- testData[subsetVector, 2:15]
# testDataSNESubset <- Rtsne(testDataDisplay, pca=FALSE)$Y
# But we will just subset the testDataSNE immediately
testDataSNESubset <- testDataSNE$Y[subsetVector, ]

# And now, this new SNE can be used for display, although all
# the data is used for the Wilcoxon calculations
dWilcoxResult <- dWilcox(
    xYData = testDataSNESubset, idsVector = testData$ids,
    groupVector = testData$label, clusterVector =
        testDataDepeche$clusterVector, displayVector = subsetVector
)
}
}
\seealso{
\code{\link{dColorPlot}}, \code{\link{dDensityPlot}},
\code{\link{dResidualPlot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dContours.R
\name{dContours}
\alias{dContours}
\title{Create density contours for two-dimensional data.}
\usage{
dContours(xYData, control, n = 100)
}
\arguments{
\item{xYData}{A dataframe with two columns containing position information
for each observation in the dataset. Typically, this is the raw result from
the SNE analysis.}

\item{control}{A numeric/integer vector or dataframe of values that could be
used to define the range in the internal dScale. If no control data is
present, the function defaults to using the indata as control data.}

\item{n}{The number fo grid points. Default is 100.}
}
\value{
A list of three components
\describe{
    \item{x, y}{The x and y coordinates of the grid points,
    vectors of length n.}
    \item{z}{An n[1] by n[2] matrix of the estimated density: rows
    correspond to the value of x, columns to the value of y.}
}
}
\description{
Here, contour lines for two-dimensional data are construced. It is primarily
thought to be used in the context of SNE plots in this package. This function
is used both internally in other functions suchas sneFluoroPlot and
sneDensityPlot, but also as a standalone function, as it increases speed
greatly to generate the density curves only once per overall analysis.
}
\examples{
# Load the test SNE data
data(testDataSNE)


# Run the function
contour_result <- dContours(testDataSNE$Y)
}
\seealso{
\code{\link{dColorPlot}}, \code{\link{dDensityPlot}},
\code{\link{dResidualPlot}}, \code{\link{dWilcox}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/neighSmooth.R
\name{neighSmooth}
\alias{neighSmooth}
\title{Euclidean neighbor smoothing}
\usage{
neighSmooth(
  focusData,
  euclidSpaceData,
  neighRows = "default",
  ctrlRows = NULL,
  kNeighK = "default",
  kMeansK = "default",
  kMeansCenters = NULL,
  kMeansClusters = NULL,
  method = "mean",
  nCores = detectCores() - 1
)
}
\arguments{
\item{focusData}{The data that should be smoothed. Should be a matrix with
the variables to be smoothed as columns.}

\item{euclidSpaceData}{The data cloud in which the nearest neighbors for the
events should be identified. Can be a vector, matrix or dataframe. It is
worth noting that if this data has more than 10 dimensions, the first step
of the algorithm will be the creation of a 10-dimensional PCA using
fast.prcomp from gmodels. So in cases where this function is used iteratively,
it might be wiser to run the PCA beforehand.}

\item{neighRows}{The rows in the dataset that correspond to the neighbors
of the focusData points. "default" is all the focusData points, but a subset
can be added instead, if preferred. This is good to use to increase
robustness, e.g. by running 100 iterations with different sets of neighbors
with the same number of points from each group/individual.}

\item{ctrlRows}{Optionally, a set of control rows that are used to remove
background signal from the neighRows data before sending the data back.}

\item{kNeighK}{The number of nearest neighbors. "default" is the max of
100 and the number of neighbor rows divided by 10000. Mutliple different
values here is preferred.}

\item{kMeansK}{The number of clusters in the initial step of the algorithm.
A higher number leads to shorter runtime, but potentially lower accuracy.
This is not used if kMeansCenters is provided. "default" is the highest of 1
and the number of cells in euclidSpaceData divided by 1000.}

\item{kMeansCenters}{Here, a pre-clustering of the data can be provided, in
which case the clustering will not be performed internally. Wise if for
example a bootstrapping scheme is used to define the neighRows iteratively,
as the k-means step can be quite time consuming. This part is the cluster
centers or centroids.}

\item{kMeansClusters}{See above. Here, the clusters, instead of the centroids
are provided if used.}

\item{method}{The method to use for the smoothing. Three values possible:
mean (default), median and mode.}

\item{nCores}{The number of cores used. Defaults to number of cores in the
computer minus 1.}
}
\value{
An object of the same dimensions as focusData that has been smoothed.
}
\description{
This function constructs a variable that for each event shows the average
value for its euclidean k-nearest neighbors. It builds on the same
idea as has been put forward in the Sconify package:
-Burns TJ (2019). Sconify: A toolkit for performing KNN-based statistics for
flow and mass cytometry data. R package version 1.4.0 and
-Hart GT, Tran TM, Theorell J, Schlums H, Arora G, Rajagopalan S, et al.
Adaptive NK cells in people exposed to Plasmodium falciparum correlate
with protection from malaria. J Exp Med. 2019 Jun 3;216(6):1280–90.
First, the k nearest neighbors are defined for cell x. Then, the average
value for the k nearest neighbors is returned as the result for cell x.
}
\examples{
data(testData)
data(testDataSNE)
euclidSpaceData <-
    testData[, c(
        "SYK", "CD16", "CD57", "EAT.2",
        "CD8", "NKG2C", "CD2", "CD56"
    )]
\dontrun{
smoothGroupVector <- neighSmooth(
    focusData = as.numeric(testData$label),
    euclidSpaceData
)
}
}
